package eu.semiramis.federationproxy;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.EnumSet;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Set;

import javax.servlet.Servlet;
import javax.servlet.ServletException;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.velocity.app.VelocityEngine;
import org.apache.velocity.runtime.RuntimeConstants;
import org.opensaml.common.SAMLObject;
import org.opensaml.common.SAMLObjectBuilder;
import org.opensaml.common.binding.BasicSAMLMessageContext;
import org.opensaml.common.binding.decoding.SAMLMessageDecoder;
import org.opensaml.saml2.binding.encoding.HTTPPostEncoder;
import org.opensaml.saml2.common.Extensions;
import org.opensaml.saml2.core.Assertion;
import org.opensaml.saml2.core.Attribute;
import org.opensaml.saml2.core.AttributeQuery;
import org.opensaml.saml2.core.AttributeStatement;
import org.opensaml.saml2.core.AuthnRequest;
import org.opensaml.saml2.core.AuthzDecisionQuery;
import org.opensaml.saml2.core.Conditions;
import org.opensaml.saml2.core.EncryptedAssertion;
import org.opensaml.saml2.core.RequestAbstractType;
import org.opensaml.saml2.core.Response;
import org.opensaml.saml2.core.StatusCode;
import org.opensaml.saml2.metadata.AssertionConsumerService;
import org.opensaml.saml2.metadata.Endpoint;
import org.opensaml.ws.transport.http.HttpServletResponseAdapter;
import org.opensaml.xacml.ctx.RequestType;
import org.opensaml.xml.Configuration;
import org.opensaml.xml.XMLObject;
import org.opensaml.xml.XMLObjectBuilderFactory;
import org.opensaml.xml.schema.XSString;
import org.opensaml.xml.schema.impl.XSAnyImpl;
import org.opensaml.xml.signature.SignableXMLObject;
import org.opensaml.xml.util.XMLHelper;

import eu.semiramis.commons.SAMLAttribute;
import eu.semiramis.commons.SAMLUtil;
import eu.semiramis.commons.audit.Transaction;
import eu.semiramis.federationproxy.auditclient.Audit;
import eu.semiramis.federationproxy.auditclient.Audit.Level;
import eu.semiramis.federationproxy.config.FederationProxyConfigurator;
import eu.semiramis.federationproxy.route.RouteBean;
import eu.semiramis.federationproxy.policy.PolicyBean;
import eu.semiramis.federationproxy.route.RouteManager;
import eu.semiramis.federationproxy.saml.SamlManager;
import eu.semiramis.federationproxy.translation_old.TranslationBean;
import eu.semiramis.federationproxy.trust.TrustBean;
import eu.semiramis.federationproxy.trust.TrustManager;
import eu.semiramis.federationproxy.wayf.WAYFState;
import eu.semiramis.util.file.FileUtil;


public class FederationProxyServlet extends FederationProxyConfigurator implements Servlet {

	
	public static final String PARAMETERNAME_RELAYSTATE					= "RelayState";
	public static final String PARAMETERNAME_SAMLREQUEST				= "SAMLRequest";
	public static final String PARAMETERNAME_SAMLRESPONSE				= "SAMLResponse";
	public static final String PARAMETERNAME_SIGNATURE					= "Signature";
	public static final String PARAMETERNAME_SIGALG						= "SigAlg";
	public static final String PARAMETERNAME_KEYINFO					= "KeyInfo";
	
	//remote auditing
	private static Audit audit = null;
	private Transaction initializationTransaction;
	private String OriginWayfRouteBeanKey = "";

	/**
	 * executeServlet
	 * @param request
	 * @param response
	 * @throws ServletException
	 * @throws IOException
	 */
	protected void executeServlet(HttpServletRequest request, HttpServletResponse response) throws Exception {
		initializationTransaction = new Transaction();
		fpAuditManager.audit(Level.INFO, initializationTransaction, Audit.EventId.REQUEST_PROCESSING_INFO, "Request processing started...");
		ProxyBean proxyBean = new ProxyBean();
		WAYFState wayfStatus = (WAYFState)request.getSession().getAttribute("fpWayfStatus");
		wayfStatus = wayfStatus == null ? WAYFState.WAYF_INITIATED : wayfStatus;
		if(request.getParameter(FederationProxyServlet.PARAMETERNAME_RELAYSTATE) != null)
			request.setAttribute(FederationProxyServlet.PARAMETERNAME_RELAYSTATE, request.getParameter(FederationProxyServlet.PARAMETERNAME_RELAYSTATE));
		switch(wayfStatus) {
			case WAYF_INITIATED:
				fpAuditManager.audit(Level.INFO, initializationTransaction, Audit.EventId.WAYF_INFO, "WAYF discovery initiated...");
				String encodedSaml = request.getParameter("SAMLRequest");
				//WAYF:check if request is coming from IA or FP
				//if coming from IA display entrusted FP's, opposite for FP
				SAMLObject originSaml = getSamlObject(request);
				String issuer = federationProxySamlManager.extractSamlIssuerValue(originSaml);
				TrustBean originTrustBean = federationProxyTrustManager.getTrustBean(issuer);
				List<RouteBean> wayfResult = federationProxyWAYFManager.
					getWAYFRouteList(originTrustBean.getTrustType().equals(
						TrustManager.FP_TYPE) ? TrustManager.IA_TYPE : TrustManager.FP_TYPE);
				//request.getSession().setAttribute("originTrustKey",originTrustBean.getTrustKey());
				request.setAttribute("wayfRouteList", wayfResult);
				request.setAttribute("wayfPostAction", federationProxyConfigManager.getProxyUrl());
				request.setAttribute("incomingSaml", encodedSaml);
				request.setAttribute("wayfRoutingType", originTrustBean.getTrustType().equals(
						TrustManager.FP_TYPE) ? "domain" : "federation");
				request.getSession().setAttribute("fpWayfStatus", WAYFState.WAYF_STARTED);
				RouteBean originRouteBean = federationProxyRouteManager.getRouteBean(issuer);
				
				if (originTrustBean.getTrustType().equals(TrustManager.FP_TYPE)){
					request.setAttribute("FPFriendlyName", federationProxyConfigManager.getProxyFriendlyName());
					getServletConfig().getServletContext().getRequestDispatcher("/WEB-INF/WAYF/fp_wayf_domain.jsp").forward(request,response);
					
				} else {
					getServletConfig().getServletContext().getRequestDispatcher("/WEB-INF/WAYF/fp_wayf.jsp").forward(request,response);
					
				}
				break;
			case WAYF_STARTED: case WAYF_REDIRECTED: case WAYF_REDIRECTED_AND_COMPLETED: case NO_WAYF_QUERY:
				if(wayfStatus.equals(WAYFState.WAYF_STARTED) && getWayfRouteBeanKey(request) != null) {
					fpAuditManager.audit(Level.INFO, initializationTransaction, Audit.EventId.WAYF_INFO, "WAYF discovery completed, destination: " + getWayfRouteBeanKey(request));
						request.getSession().setAttribute("wayfRouteBeanKey", getWayfRouteBeanKey(request));
						
						request.getSession().setAttribute("fpWayfStatus", WAYFState.WAYF_REDIRECTED);
				} else if(wayfStatus.equals(WAYFState.WAYF_REDIRECTED) ) {
					if(federationProxySamlManager.getSamlObjectType(getSamlObject(request)) == SamlManager.SAMLTYPE_RESPONSEIMPL){
						request.getSession().invalidate();
						request.getSession().setAttribute("fpWayfStatus", WAYFState.WAYF_REDIRECTED_AND_COMPLETED);
					} else {
						fpAuditManager.audit(Level.ERROR, initializationTransaction, Audit.EventId.WAYF_ERROR, "WAYF discovery error, restarting...");
						restartWayfService(request,response);
						return;
					}
					
				} else {
					restartWayfService(request, response);
				return;
				}
				try {
					fpAuditManager.audit(Level.INFO, initializationTransaction, Audit.EventId.FP_CONTEXT_INFO, "Proxy processing starting...");
					logger.debug("[FederationProxyServlet.executeServlet()] Start ");
					logger.debug("[FederationProxyServlet.executeServlet()] 1.- retrievePkcsParameters");
					fpAuditManager.audit(Level.INFO, initializationTransaction, Audit.EventId.FP_PKCS_INFO, "Retrieving PKCS params...");
					proxyBean = retrievePkcsParameters(request, response, proxyBean);
					logger.debug("[FederationProxyServlet.executeServlet()] 2.- retrieveSourceParameters");
					fpAuditManager.audit(Level.INFO, initializationTransaction, Audit.EventId.FP_CONTEXT_INFO, "Retrieving source params...");
					proxyBean = retrieveSourceParameters(request, response, proxyBean);
					logger.debug("[FederationProxyServlet.executeServlet()] 3.- retrieveTrustParameters");
					fpAuditManager.audit(Level.INFO, initializationTransaction, Audit.EventId.FP_TRUST_INFO, "Retrieving trust params...");
					proxyBean = retrieveTrustParameters(request, response, proxyBean);
					//logger.debug("[FederationProxyServlet.executeServlet()] 4.- retriveTranslationParameters");
					//fpAuditManager.audit(Level.INFO, initializationTransaction, Audit.EventId.FP_TRANSLATION_INFO, "Retrieving translation params...");
					//proxyBean = retriveTranslationParameters(request, response, proxyBean);
					logger.debug("[FederationProxyServlet.executeServlet()] 5.- retrieveRouteParameters");
					fpAuditManager.audit(Level.INFO, initializationTransaction, Audit.EventId.FP_ROUTE_INFO, "Retrieving route params...");
					proxyBean = retrieveRouteParameters(request, response, proxyBean, getWayfRouteBeanKey(request));
					logger.debug("[FederationProxyServlet.executeServlet()] 6.- retrieveMessageParameters");
					fpAuditManager.audit(Level.INFO, initializationTransaction, Audit.EventId.FP_CONTEXT_INFO, "Retrieving message params...");
					proxyBean = retrieveMesssageParameters(request, response, proxyBean);
					logger.debug("[FederationProxyServlet.executeServlet()] 7.- executeProxy");
					fpAuditManager.audit(Level.INFO, initializationTransaction, Audit.EventId.FP_CONTEXT_INFO, "Executing proxy...");
					proxyBean = executeProxy(request, response, proxyBean);
				} catch (Exception e) {
					logger.error("[FederationProxyServlet.executeServlet()] EXCEPTION: " + e.getMessage());
					logger.error("[FederationProxyServlet.executeServlet()] \t exceptionProxyBean: " + proxyBean);
					fpAuditManager.audit(Level.ERROR, initializationTransaction, Audit.EventId.FP_CONTEXT_ERROR, e);
					throw e;
				} finally {
					logger.debug("[FederationProxyServlet.executeServlet()] End");
					fpAuditManager.audit(Level.INFO, initializationTransaction, Audit.EventId.FP_CONTEXT_INFO, "Proxy processing finished.");
					if(request.getSession().getAttribute("fpWayfStatus") == WAYFState.WAYF_REDIRECTED_AND_COMPLETED)
						request.getSession().setAttribute("fpWayfStatus", WAYFState.NO_WAYF_QUERY);
				}
				break;
		}
	}
	
	String getWayfRouteBeanKey(HttpServletRequest req) {
		if(req.getSession().getAttribute("wayfRouteBeanKey") != null)
			return req.getSession().getAttribute("wayfRouteBeanKey").toString();
		else if (req.getParameter("wayfRouteBeanKey") != null)
			return  req.getParameter("wayfRouteBeanKey");
		else
		return null;
	}
	
	void restartWayfService(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException {
		req.getSession().invalidate();
		getServletContext().getRequestDispatcher(
        req.getServletPath()).forward(req,res);
	}
	/**
	 * retrievePkcsParameters
	 * @param proxyBean
	 * @return
	 */
	private ProxyBean retrievePkcsParameters(HttpServletRequest request, HttpServletResponse response, ProxyBean proxyBean) throws Exception{
		try {
			logger.debug("[FederationProxyServlet.retrievePkcsParameters()] Start");
			if (federationProxyTrustManager.getTrustPkcsPrivateKey() == null){
				throw new NullPkcsPrivateKeyException();
			}
			if (federationProxyTrustManager.getTrustPkcsPublicKey() == null){
				throw new NullPkcsPublicKeyException();
			}
			return proxyBean;			
		} catch (Exception e){
			throw e;
		} finally {
			logger.debug("[FederationProxyServlet.retrievePkcsParameters()] End");			
		}
	}
	
	/**
	 * retrieveSourceParameters
	 * @param request
	 * @param samlRequest
	 * @return
	 * @throws Exception
	 */
	private ProxyBean retrieveSourceParameters(HttpServletRequest request, HttpServletResponse response, ProxyBean proxyBean) throws Exception {
		try {
			logger.debug("[FederationProxyServlet.retrieveSourceParameters()] Start");
			//LoadSourceSamlObject
			SAMLObject sourceSamlObject = getSamlObject(request);
			logger.debug("[FederationProxyServlet.retrieveSourceParameters()] \t sourceSamlObjectId: " + federationProxySamlManager.extractSamlObjectId(sourceSamlObject));
			proxyBean.setSourceSamlObject(sourceSamlObject);
			
			//CheckSourceSamlObject			
			if (sourceSamlObject == null){
				new NullSourceSamlObjectException();
			}
			//LoadSourceRelayState
			String sourceRelayState = request.getParameter(FederationProxyServlet.PARAMETERNAME_RELAYSTATE);
			logger.debug("[FederationProxyServlet.retrieveSourceParameters()] \t sourceRelayState: " + sourceRelayState);
			if (sourceRelayState == null){
				logger.debug("[FederationProxyServlet.retrieveSourceParameters()] \t\t CaseNullSourceRelayState");
				logger.debug("[FederationProxyServlet.retrieveSourceParameters()] \t\t\t setSourceRelayStateAsRequestURL");
				sourceRelayState = request.getRequestURL().toString();
				logger.debug("[FederationProxyServlet.retrieveSourceParameters()] \t\t\t\t defaultSourceRelayState: " + sourceRelayState);
			}
			proxyBean.setSourceRelayState(sourceRelayState);
			//CheckSourceRelayState
			if (sourceRelayState == null){
				throw new NullSourceRelayStateException();
			}
			
			switch (federationProxySamlManager.getSamlObjectType(proxyBean.getSourceSamlObject())){
				case SamlManager.SAMLTYPE_AUTHNREQUEST:
					break;
				case SamlManager.SAMLTYPE_ATTRIBUTEQUERY:
					//Origin Issuer and/or Origin Certificate (for end-to-end privacy)
					RequestAbstractType req = (RequestAbstractType)sourceSamlObject;
					Extensions extensions = req.getExtensions();
			        if (extensions != null) {
			        	proxyBean.setProxySamlOriginCertificate(SAMLUtil.recoverX509Certificate(req));
			        	proxyBean.setProxySamlOriginIssuer(SAMLUtil.recoverOriginIssuer(extensions));
			        }else {
			        	throw new NoOriginIssuerOrCertificateException();        
			        }
					break;
				case SamlManager.SAMLTYPE_AUTHDECISIONQUERY:
					break;
				case SamlManager.SAMLTYPE_RESPONSEIMPL:
					break;
			}
			
			
			return proxyBean;
		} catch (Exception e){
			throw e;
		} finally {
			logger.debug("[FederationProxyServlet.retrieveSourceParameters()] End");
		}
	}
	
	
	/**
	 * retriveTranslationParameters
	 * @param proxyBean
	 * @return
	 */
	/*private ProxyBean retriveTranslationParameters(HttpServletRequest request, HttpServletResponse response, ProxyBean proxyBean) throws Exception{
		try {
			logger.debug("[FederationProxyServlet.retriveTranslationParameters()] Start");
			if (federationProxyTranslationManager.getTranslationEnabledFlag()){
				//*********************************************************************************************
				//SourceTranslationBean 
				//*********************************************************************************************
				//SourceTranslationBean.-	Its the issuerTranslationBean
				//							Must be not null
				String sourceTranslationBeanKey = federationProxySamlManager.extractSamlIssuerValue(proxyBean.getSourceSamlObject());
				logger.debug("[FederationProxyServlet.retriveTranslationParameters()] \t sourceTranslationBeanKey: " + sourceTranslationBeanKey);
				TranslationBean sourceTranslationBean = federationProxyTranslationManager.getTranslationBean(sourceTranslationBeanKey);
				//CheckSourceTranslationBean
				if (sourceTranslationBean == null){
					new UndefinedTranslationBeanException("sourceTranslationBean: " + sourceTranslationBean);
				}
				//proxySourceTranslationBaseName.- Is the translationBasename for the sourceTranslationBean
				String sourceTranslationBeanBaseName = sourceTranslationBean.getTranslationBaseName();
				logger.debug("[FederationProxyServlet.retriveTranslationParameters()] \t sourceTranslationBeanBaseName: " + sourceTranslationBeanBaseName);
				proxyBean.setProxyFromResourceBundleBaseName(sourceTranslationBeanBaseName);
				//proxyFromLocale.- Is the translationLocale for the sourceTranslationBean
				String sourceTranslationBeanLanguage = sourceTranslationBean.getTranslationLanguage();
				logger.debug("[FederationProxyServlet.retriveTranslationParameters()] \t sourceTranslationBeanLanguage: " + sourceTranslationBeanLanguage);
				String sourceTranslationBeanCountry = sourceTranslationBean.getTranslationCountry();
				logger.debug("[FederationProxyServlet.retriveTranslationParameters()] \t sourceTranslationBeanCountry: " + sourceTranslationBeanCountry);
				String sourceTranslationBeanVariant = sourceTranslationBean.getTranslationVariant();
				logger.debug("[FederationProxyServlet.retriveTranslationParameters()] \t sourceTranslationBeanVariant: " + sourceTranslationBeanVariant);
				Locale proxyFromLocale = new Locale(sourceTranslationBeanLanguage, sourceTranslationBeanCountry, sourceTranslationBeanVariant);
				proxyBean.setProxyFromLocale(proxyFromLocale);
				//*********************************************************************************************
				//TargetTranslationBean 
				//*********************************************************************************************
				//TargetTranslationBean.-	It is the routeTargetTranslationBean for Request. It is the storedProxyIssuerTranslationBean for Responses
				//							Must be not null. 
				String targetTranslationBeanKey = null;			
				String routeBeanKey = null;
				RouteBean routeBean = null;
				switch (federationProxySamlManager.getSamlObjectType(proxyBean.getSourceSamlObject())){
					case SamlManager.SAMLTYPE_AUTHNREQUEST:
						routeBeanKey = getWayfRouteBeanKey(request);
						logger.debug("[FederationProxyServlet.retriveTranslationParameters()] \t\t routeBeanKey: " + routeBeanKey);					
						routeBean = federationProxyRouteManager.getRouteBean(routeBeanKey);
						targetTranslationBeanKey = routeBean.getRouteTranslationKey();
						break;
					case SamlManager.SAMLTYPE_ATTRIBUTEQUERY:
						routeBeanKey = getWayfRouteBeanKey(request);
						logger.debug("[FederationProxyServlet.retriveTranslationParameters()] \t\t routeBeanKey: " + routeBeanKey);					
						routeBean = federationProxyRouteManager.getRouteBean(routeBeanKey);
						targetTranslationBeanKey = routeBean.getRouteTranslationKey();
						break;
					case SamlManager.SAMLTYPE_AUTHDECISIONQUERY:
						//CUIDADO: AQUI SE UTILIZA EL SUBJECTNAMEQUALIFIER, PERO SE HA QUITADO DE LOS MENSAJES SAML?
						routeBeanKey = federationProxySamlManager.extractSamlSubjectNameQualifier(proxyBean.getSourceSamlObject());
						logger.debug("[FederationProxyServlet.retriveTranslationParameters()] \t\t routeBeanKey: " + routeBeanKey);					
						routeBean = federationProxyRouteManager.getRouteBean(routeBeanKey);
						targetTranslationBeanKey = routeBean.getRouteTranslationKey();
						break;
					case SamlManager.SAMLTYPE_RESPONSEIMPL:
						String storedMapppingBeanKey = federationProxySamlManager.extractSamlInResponseTo(proxyBean.getSourceSamlObject());
						logger.debug("[FederationProxyServlet.retriveTranslationParameters()] \t\t storedMapppingBeanKey: " + storedMapppingBeanKey);
						ProxyBean storedProxyBean = federationProxyRouteManager.loadProxyBean(storedMapppingBeanKey);
						targetTranslationBeanKey = federationProxySamlManager.extractSamlIssuerValue(storedProxyBean.getSourceSamlObject());
						break;					
				}
				TranslationBean targetTranslationBean = federationProxyTranslationManager.getTranslationBean(targetTranslationBeanKey);
				//CheckTargetTranslationBean
				if (targetTranslationBean == null){
					new UndefinedTranslationBeanException("targetTranslationBean: " + targetTranslationBean);
				}
				//proxyTargetTranslationBaseName.- Is the translationBasename for the targetTranslationBean
				String targetTranslationBeanBaseName = targetTranslationBean.getTranslationBaseName();
				logger.debug("[FederationProxyServlet.retriveTranslationParameters()] \t targetTranslationBeanBaseName: " + targetTranslationBeanBaseName);
				proxyBean.setProxyToResourceBundleBaseName(targetTranslationBeanBaseName);
				//proxyToLocale.- Is the translationLocale for the targetTranslationBean
				String targetTranslationBeanLanguage = targetTranslationBean.getTranslationLanguage();
				logger.debug("[FederationProxyServlet.retriveTranslationParameters()] \t targetTranslationBeanLanguage: " + targetTranslationBeanLanguage);
				String targetTranslationBeanCountry = targetTranslationBean.getTranslationCountry();
				logger.debug("[FederationProxyServlet.retriveTranslationParameters()] \t targetTranslationBeanCountry: " + targetTranslationBeanCountry);
				String targetTranslationBeanVariant = targetTranslationBean.getTranslationVariant();
				logger.debug("[FederationProxyServlet.retriveTranslationParameters()] \t targetTranslationBeanVariant: " + targetTranslationBeanVariant);
				Locale proxyToLocale = new Locale(targetTranslationBeanLanguage, targetTranslationBeanCountry, targetTranslationBeanVariant);
				proxyBean.setProxyToLocale(proxyToLocale);
			}
			return proxyBean;			
		} catch (Exception e){
			throw e;
		} finally {
			logger.debug("[FederationProxyServlet.retriveTranslationParameters()] End");			
		}
	}*/
	
	
	/**
	 * retrieveTrustParameters
	 * @param proxyBean
	 * @return
	 */
	private ProxyBean retrieveTrustParameters(HttpServletRequest request, HttpServletResponse response, ProxyBean proxyBean) throws Exception{
		try {
			logger.debug("[FederationProxyServlet.retrieveTrustParameters()] Start");
			//*********************************************************************************************
			//SourceTrustBean 
			//*********************************************************************************************
			//SourceTrustBean.-	Its the issuerTrustBean
			//					Must be not null
			System.out.println("source: " + federationProxySamlManager.extractSamlIssuerValue(proxyBean.getSourceSamlObject()));
			String sourceTrustBeanKey = federationProxySamlManager.extractSamlIssuerValue(proxyBean.getSourceSamlObject());
			logger.debug("[FederationProxyServlet.retrieveTrustParameters()] \t sourceTrustBeanKey: " + sourceTrustBeanKey);
			TrustBean sourceTrustBean = federationProxyTrustManager.getTrustBean(sourceTrustBeanKey);
			//CheckSourceTrustBean
			if (sourceTrustBean == null){
				new UndefinedTrustBeanException("sourceTrustBeanKey: " + sourceTrustBeanKey);
			}
			//proxyValidateSignFlag.- Is the validateSignFlag for the sourceTrustBean
			boolean sourceTrustBeanValidateSignFlag = sourceTrustBean.getTrustValidateSignFlag();
			logger.debug("[FederationProxyServlet.retrieveTrustParameters()] \t sourceTrustBeanValidateSignFlag: " + sourceTrustBeanValidateSignFlag);
			proxyBean.setProxyValidateSignFlag(sourceTrustBeanValidateSignFlag);
			//proxyValidateSignCertFilePath.- Is the certPath for the sourceTrustBean.- It must be an existent file
			String sourceTrustBeanValidateSignCertFilePath = sourceTrustBean.getTrustCertFilePath();
			logger.debug("[FederationProxyServlet.retrieveTrustParameters()] \t sourceTrustBeanValidateSignCertFilePath: " + sourceTrustBeanValidateSignCertFilePath);
			proxyBean.setProxyValidateSignCertFilePath(sourceTrustBeanValidateSignCertFilePath);
			//CheckSourceTrustBeanValidateSignCertFilePath
			if (sourceTrustBeanValidateSignFlag && !FileUtil.isExistentFile(sourceTrustBeanValidateSignCertFilePath)){
				throw new UnexistentTrustCertFilePathException("validateSignCertFilePath: " + sourceTrustBeanValidateSignCertFilePath);
			}
			//*********************************************************************************************
			//TargetTrustBean 
			//*********************************************************************************************
			//TargetTrustBean.-	It is the RouteTargetTrustBean for Request. It is the storedProxyIssuerTrustBean for Responses
			//					Must be not null. 
			String targetTrustBeanKey = null;			
			String routeBeanKey = null;
			RouteBean routeBean = null;
			switch (federationProxySamlManager.getSamlObjectType(proxyBean.getSourceSamlObject())){
				case SamlManager.SAMLTYPE_AUTHNREQUEST:
					routeBeanKey = getWayfRouteBeanKey(request);
					logger.debug("[FederationProxyServlet.retrieveTrustParameters()] \t\t routeBeanKey: " + routeBeanKey);					
					routeBean = federationProxyRouteManager.getRouteBean(routeBeanKey);
					targetTrustBeanKey = routeBean.getRouteTrustKey();
					break;
				case SamlManager.SAMLTYPE_ATTRIBUTEQUERY:
					routeBeanKey = getWayfRouteBeanKey(request);
					logger.debug("[FederationProxyServlet.retrieveTrustParameters()] \t\t routeBeanKey: " + routeBeanKey);					
					routeBean = federationProxyRouteManager.getRouteBean(routeBeanKey);
					targetTrustBeanKey = routeBean.getRouteTrustKey();
					break;
				case SamlManager.SAMLTYPE_AUTHDECISIONQUERY:
					routeBeanKey = federationProxySamlManager.extractSamlSubjectNameQualifier(proxyBean.getSourceSamlObject());
					logger.debug("[FederationProxyServlet.retrieveTrustParameters()] \t\t routeBeanKey: " + routeBeanKey);					
					routeBean = federationProxyRouteManager.getRouteBean(routeBeanKey);
					targetTrustBeanKey = routeBean.getRouteTrustKey();
					break;
				case SamlManager.SAMLTYPE_RESPONSEIMPL:
					String storedMapppingBeanKey = federationProxySamlManager.extractSamlInResponseTo(proxyBean.getSourceSamlObject());
					logger.debug("[FederationProxyServlet.retrieveTrustParameters()] \t\t storedMapppingBeanKey: " + storedMapppingBeanKey);
					ProxyBean storedProxyBean = federationProxyRouteManager.loadProxyBean(storedMapppingBeanKey);
					targetTrustBeanKey = federationProxySamlManager.extractSamlIssuerValue(storedProxyBean.getSourceSamlObject());
					break;					
					
			}
			TrustBean targetTrustBean = federationProxyTrustManager.getTrustBean(targetTrustBeanKey);
			System.out.println("test: " + targetTrustBeanKey);
			//CheckTargetTrustBean
			if (targetTrustBean == null){
				new UndefinedTrustBeanException("targetTrustBeanKey: " + targetTrustBeanKey);
			}
			//proxySignFlag.- It is the signFlag for the targetTrustBean
			boolean targetTrustBeanSignFlag = targetTrustBean.getTrustSignFlag();
			logger.debug("[FederationProxyServlet.retrieveTrustParameters()] \t targetTrustBeanSignFlag: " + targetTrustBeanSignFlag);
			proxyBean.setProxySignFlag(targetTrustBeanSignFlag);
			//proxyEncryptFlag.- It is the encryptFlag for the targetTrustBean
			boolean targetTrustBeanEncryptFlag = targetTrustBean.getTrustEncryptFlag();
			logger.debug("[FederationProxyServlet.retrieveTrustParameters()] \t targetTrustBeanEncryptFlag: " + targetTrustBeanEncryptFlag);
			proxyBean.setProxyEncryptFlag(targetTrustBeanEncryptFlag);
			//proxyEncryptCertFilePath.- It is the encryptFlag for the targetTrustBean
			String targetTrustBeanEncryptCertFilePath = targetTrustBean.getTrustCertFilePath();
			logger.debug("[FederationProxyServlet.retrieveTrustParameters()] \t targetTrustBeanEncryptCertFilePath: " + targetTrustBeanEncryptCertFilePath);
			proxyBean.setProxyEncryptCertFilePath(targetTrustBeanEncryptCertFilePath);
			//CheckTargetTrustBeanEncryptCertFilePath
			if (targetTrustBeanEncryptFlag && !FileUtil.isExistentFile(targetTrustBeanEncryptCertFilePath)){
				throw new UnexistentTrustCertFilePathException("targetTrustBeanEncryptCertFilePath: " + targetTrustBeanEncryptCertFilePath);
			}
			return proxyBean;			
		} catch (Exception e){
			throw e;
		} finally {
			logger.debug("[FederationProxyServlet.retrieveTrustParameters()] End");			
		}
	}
	/**
	 * retrieveRouteParameters
	 * @param proxyBean
	 * @return
	 */
	private ProxyBean retrieveRouteParameters(HttpServletRequest request, HttpServletResponse response, ProxyBean proxyBean, String wayfRouteBeanKey) throws Exception{
		try {
			logger.debug("[FederationProxyServlet.retrieveRouteParameters()] Start");
			//proxyUrl
			//proxyRelayState
			//proxySamlInResponseTo
			//proxySamlInResponseToType
			//proxySamlActionId
			String proxyUrl = null;
			String proxyRelayState = null;
			String proxySamlInResponseTo = null;
			int proxySamlInResponseToType = 0;
			String routeBeanKey = null;
			RouteBean routeBean = null;
			switch (federationProxySamlManager.getSamlObjectType(proxyBean.getSourceSamlObject())){
				case SamlManager.SAMLTYPE_AUTHNREQUEST: 
					logger.debug("[FederationProxyServlet.retrieveRouteParameters()] \t CaseAuthnRequest");
					routeBeanKey = federationProxySamlManager.extractSamlIdpProviderId(proxyBean.getSourceSamlObject());
					//wayfRouteBeanKey = federationProxySamlManager.extractSamlIdpProviderId(proxyBean.getSourceSamlObject());
					logger.debug("[FederationProxyServlet.retrieveRouteParameters()] \t\t routeBeanKey: " + routeBeanKey);
					//logger.debug("[FederationProxyServlet.retrieveRouteParameters()] \t\t routeBeanKey: " + wayfRouteBeanKey);
					//routeBean = federationProxyRouteManager.getRouteBean(routeBeanKey);
					routeBean = federationProxyRouteManager.getRouteBean(wayfRouteBeanKey);
					//CheckRouteBean
					if (routeBean == null){
						throw new UndefinedRouteBeanException("routeBeanKey: " + routeBeanKey);
					}
					proxyUrl = routeBean.getRouteTargetUrl();
					proxyRelayState = federationProxyConfigManager.getProxyUrl();
					proxySamlInResponseTo = null;
					proxySamlInResponseToType = 0;
					
					break;
				case SamlManager.SAMLTYPE_ATTRIBUTEQUERY:
					routeBeanKey = federationProxySamlManager.extractSamlIdpProviderId(proxyBean.getSourceSamlObject());
					routeBean = federationProxyRouteManager.getRouteBean(wayfRouteBeanKey);
					//CheckRouteBean
					if (routeBean == null){
						throw new UndefinedRouteBeanException("routeBeanKey: " + routeBeanKey);
					}
					//translation
					if (routeBean.getRouteType().equals("IA")) { //next step in flow
						proxyBean.setFromIA(false); //SAML came from FP
					} else proxyBean.setFromIA(true);
				case SamlManager.SAMLTYPE_AUTHDECISIONQUERY:
					logger.debug("[FederationProxyServlet.retrieveRouteParameters()] \t CaseSubjectQuery");
					routeBeanKey = federationProxySamlManager.extractSamlSubjectNameQualifier(proxyBean.getSourceSamlObject());
					logger.debug("[FederationProxyServlet.retrieveRouteParameters()] \t\t routeBeanKey: " + routeBeanKey);
					routeBean = federationProxyRouteManager.getRouteBean(wayfRouteBeanKey);
					//CheckRouteBean
					if (routeBean == null){
						throw new UndefinedRouteBeanException("routeBeanKey: " + routeBeanKey);
					}
					proxyUrl = routeBean.getRouteTargetUrl();
					proxyRelayState = federationProxyConfigManager.getProxyUrl();
					proxySamlInResponseTo = null;
					proxySamlInResponseToType = 0;
					break;
				case SamlManager.SAMLTYPE_RESPONSEIMPL:
					logger.debug("[FederationProxyServlet.retrieveRouteParameters()] \t CaseResponseImpl");
					String proxyBeanKey = federationProxySamlManager.extractSamlInResponseTo(proxyBean.getSourceSamlObject());
					logger.debug("[FederationProxyServlet.retrieveRouteParameters()] \t\t proxyBeanKey: " + proxyBeanKey);
					//proxyUrl.- will be obtained from corresponding stored proxy bean 
					ProxyBean storedProxyBean = federationProxyRouteManager.loadProxyBean(proxyBeanKey);
					//CheckStoredProxyBean
					if (storedProxyBean == null){
						throw new UnstoredProxyBeanException("proxyBeanKey: " + proxyBeanKey);
					}
					if (storedProxyBean instanceof AuthnRequest){
						proxyUrl = federationProxySamlManager.extractSamlAssertionConsumerServiceUrl(storedProxyBean.getSourceSamlObject());
					} else if (request.getSession().getAttribute("fpWayfStatus") == WAYFState.WAYF_REDIRECTED_AND_COMPLETED){
						proxyUrl = storedProxyBean.getSourceRelayState();
					} else {
						proxyUrl = storedProxyBean.getSourceRelayState();
					}
					
					proxySamlInResponseTo = federationProxySamlManager.extractSamlObjectId(storedProxyBean.getSourceSamlObject());
					proxySamlInResponseToType = federationProxySamlManager.getSamlObjectType(storedProxyBean.getSourceSamlObject());					
					proxyRelayState = null;
					break;
				default:
					throw new UnexpectedSourceSamlObjectTypeException();
			}
			logger.debug("[FederationProxyServlet.retrieveRouteParameters()] \t\t proxyUrl: " + proxyUrl);
			proxyBean.setProxyUrl(proxyUrl);
			//CheckProxyUrl
			if (proxyUrl == null){
				throw new NullTargetUrlException();
			}
			logger.debug("[FederationProxyServlet.retrieveRouteParameters()] \t\t proxyRelayState: " + proxyRelayState);
			proxyBean.setProxyRelayState(proxyRelayState);
			logger.debug("[FederationProxyServlet.retrieveRouteParameters()] \t\t proxySamlInResponseTo: " + proxySamlInResponseTo);
			proxyBean.setProxySamlInResponseTo(proxySamlInResponseTo);
			logger.debug("[FederationProxyServlet.retrieveRouteParameters()] \t\t proxySamlInResponseToType: " + proxySamlInResponseToType);
			proxyBean.setProxySamlInResponseToType(proxySamlInResponseToType);
			//proxySamlIssuer 
			String proxySamlIssuer = federationProxyConfigManager.getProxyId();
			logger.debug("[FederationProxyServlet.retrieveRouteParameters()] \t\t proxySamlIssuer: " + proxySamlIssuer);
			proxyBean.setProxySamlIssuer(proxySamlIssuer);
			return proxyBean;
		} catch (Exception e){
			throw e;
		} finally {
			logger.debug("[FederationProxyServlet.retrieveRouteParameters()] End");			
		}
	}
	
	/**
	 * retrieveMesssageParameters
	 * @param proxyBean
	 * @return
	 */
	private ProxyBean retrieveMesssageParameters(HttpServletRequest request, HttpServletResponse response, ProxyBean proxyBean) throws Exception{
		try {
			logger.debug("[FederationProxyServlet.retrieveMesssageParameters()] Start");
			//AuthenticationParameters
				//proxySamlIdpProvider			
				String proxySamlIdpProvider = federationProxySamlManager.extractSamlIdpProviderId(proxyBean.getSourceSamlObject());				
				logger.debug("[FederationProxyServlet.retrieveMesssageParameters()] \t\t proxySamlIdpProvider: " + proxySamlIdpProvider);
				proxyBean.setProxySamlIdpProvider(proxySamlIdpProvider);
				//proxySamlConditions			
				Conditions proxySamlConditions = federationProxySamlManager.extractSamlConditions(proxyBean.getSourceSamlObject());
				logger.debug("[FederationProxyServlet.retrieveMesssageParameters()] \t\t proxySamlConditions: " + proxySamlConditions);
				proxyBean.setProxySamlConditions(proxySamlConditions);
			//SubjectQueryParameters (RequestParameter And Authorization)
				//proxySubjectNameId
				String proxySamlSubjectNameId = federationProxySamlManager.extractSamlSubjectNameId(proxyBean.getSourceSamlObject());
				logger.debug("[FederationProxyServlet.retrieveMesssageParameters()] \t\t proxySamlSubjectNameId: " + proxySamlSubjectNameId);
				proxyBean.setProxySamlSubjectNameId(proxySamlSubjectNameId);
				//proxySubjectNameQualifier
				String proxySamlSubjectNameQualifier = federationProxySamlManager.extractSamlSubjectNameQualifier(proxyBean.getSourceSamlObject());
				logger.debug("[FederationProxyServlet.retrieveMesssageParameters()] \t\t proxySamlSubjectNameQualifier: " + proxySamlSubjectNameQualifier);
				proxyBean.setProxySamlSubjectNameQualifier(proxySamlSubjectNameQualifier);
				
				//SUBJECT DOMAIN??
				
			//AuthorizationParameters
				//proxySamlActionId
				String proxySamlActionId = federationProxySamlManager.extractSamlActionId(proxyBean.getSourceSamlObject());
				logger.debug("[FederationProxyServlet.retrieveMesssageParameters()] \t\t proxySamlActionId: " + proxySamlActionId);			
				proxyBean.setProxySamlActionId(proxySamlActionId);
				//proxySamlResourceId
				String proxySamlResourceId = federationProxySamlManager.extractSamlResourceId(proxyBean.getSourceSamlObject());
				logger.debug("[FederationProxyServlet.retrieveMesssageParameters()] \t\t proxySamlResourceId: " + proxySamlResourceId);
				proxyBean.setProxySamlResourceId(proxySamlResourceId);
			//Assertions
				logger.debug("[FederationProxyServlet.retrieveMesssageParameters()] \t\t proxySamlAssertionList[released + encrypted]: ");				
				//sourceSamlAssertionList
				List<Assertion> sourceSamlAssertionList = federationProxySamlManager.extractSamlAssertionList(proxyBean.getSourceSamlObject());
				int sourceSamlAssertionListSize = 0;
				if (sourceSamlAssertionList != null){
					sourceSamlAssertionListSize = sourceSamlAssertionList.size();
				}
				logger.debug("[FederationProxyServlet.retrieveMesssageParameters()] \t\t\t assertionList[" + sourceSamlAssertionListSize + "]");
				//sourceSamlEncryptedAssertionlist
				List<EncryptedAssertion> sourceSamlEncryptedAssertionList = federationProxySamlManager.extractSamlEncryptedAssertionList(proxyBean.getSourceSamlObject());
				int sourceSamlEncryptedAssertionListSize = 0;
				if (sourceSamlEncryptedAssertionList != null){
					sourceSamlEncryptedAssertionListSize = sourceSamlEncryptedAssertionList.size();
				}
				logger.debug("[FederationProxyServlet.retrieveMesssageParameters()] \t\t\t encryptedAssertionList[" + sourceSamlEncryptedAssertionListSize + "]");				
				//decryptSourceSamlEncryptedAssertionlist
				List<Assertion> sourceSamlDecryptedAssertionList = new ArrayList<Assertion>();
				if (sourceSamlEncryptedAssertionListSize > 0){
					Iterator<EncryptedAssertion> sourceSamlEncryptedAssertionListIterator = sourceSamlEncryptedAssertionList.iterator();
					int encryptedAssertionIndex = 0;
					while (sourceSamlEncryptedAssertionListIterator.hasNext()) {
						encryptedAssertionIndex ++;
						EncryptedAssertion encryptedAssertion = sourceSamlEncryptedAssertionListIterator.next();
						Assertion decryptedAssertion = federationProxySamlManager.decryptAssertion(encryptedAssertion, federationProxyTrustManager.getTrustPkcsPublicKey(), federationProxyTrustManager.getTrustPkcsPrivateKey());
						logger.debug("[FederationProxyServlet.retrieveMesssageParameters()] \t\t\t\t encryptedAssertion[" + encryptedAssertionIndex + " of " + sourceSamlEncryptedAssertionListSize + "] found and decrypted");
						sourceSamlDecryptedAssertionList.add(decryptedAssertion);
					}
				}
				//proxySamlAssertionList
				List<Assertion> proxySamlAssertionList = new ArrayList<Assertion>();
				if (sourceSamlAssertionList != null){
					proxySamlAssertionList.addAll(sourceSamlAssertionList);
				}
				if (sourceSamlDecryptedAssertionList != null){
					proxySamlAssertionList.addAll(sourceSamlDecryptedAssertionList);
				}
				logger.debug("[FederationProxyServlet.retrieveMesssageParameters()] \t\t\t Total[" + sourceSamlAssertionListSize + "|" +  sourceSamlEncryptedAssertionListSize + "]");
				proxyBean.setProxySamlAssertionList(proxySamlAssertionList);
			return proxyBean;
		} catch (Exception e){
			throw e;
		} finally {
			logger.debug("[FederationProxyServlet.retrieveMesssageParameters()] End");			
		}
	}
	/**
	 * executeProxy
	 * @param proxyBean
	 * @return
	 */
	private ProxyBean executeProxy(HttpServletRequest request, HttpServletResponse response, ProxyBean proxyBean) throws Exception{		
		try {
			logger.debug("[FederationProxyServlet.executeProxy()] Start");
			logger.debug("[FederationProxyServlet.executeProxy()] \t ValidateSourceSign");
			if (proxyBean.getProxyValidateSignFlag()) {
				SignableXMLObject signableXmlObject = (SignableXMLObject)proxyBean.getSourceSamlObject();
				SAMLUtil.validateSign(signableXmlObject, proxyBean.getProxyValidateSignCertFilePath());
				boolean validationAchievedFlag = true;
				logger.debug("[FederationProxyServlet.executeProxy()] \t\t validationAchievedFlag: " + validationAchievedFlag);
				proxyBean.setProxyValidateSignAchievedFlag(true);
				if (!validationAchievedFlag){
					throw new NotTrustAchievedException();
				}
			}
			logger.debug("[FederationProxyServlet.executeProxy()] \t ValidateSourceResponseStatusCode");
			//TODO Debera el FP validar si la respuesta ha tenido exito o no, o simplemente redirigirla al IA para que la trate?
			//Yo creo que solo deberia dirigirla al IA para que la trate
			
			
			
			
			
			String sourceResponseStatusCodeValue = federationProxySamlManager.extractSamlResponseStatusCode(proxyBean.getSourceSamlObject());
			String sourceResponseStatusCodeMessage = federationProxySamlManager.extractSamlResponseStatusMessage(proxyBean.getSourceSamlObject());
			boolean sourceSamlResponseStatusSuccessFlag = true;			
			logger.debug("[FederationProxyServlet.executeProxy()] \t\t sourceResponseStatusCodeValue: " + sourceResponseStatusCodeValue);		
			if (sourceResponseStatusCodeValue != null){
				proxyBean.setProxyStatusCode(sourceResponseStatusCodeValue);
				proxyBean.setProxyStatusMessage(sourceResponseStatusCodeMessage);
				if (sourceResponseStatusCodeValue.equals(StatusCode.SUCCESS_URI) ||
						sourceResponseStatusCodeValue.equals(StatusCode.REQUEST_DENIED_URI)){
					sourceSamlResponseStatusSuccessFlag = true;
				} else {
					sourceSamlResponseStatusSuccessFlag = false;
				}
			}
			logger.debug("[FederationProxyServlet.executeProxy()] \t\t sourceSamlResponseStatusSuccessFlag: " + sourceSamlResponseStatusSuccessFlag);
			proxyBean.setProxySamlResponseStatusCodeSuccessFlag(sourceSamlResponseStatusSuccessFlag);
			//TODO creo que esto estaria bien comentado, tal y como esta
			//Al federation prosy le deberia dar igua si el StatusCode es Success o no.
			//if (!sourceSamlResponseStatusSuccessFlag){
			//	throw new UnSuccessStatusCodeException();
			//}
			
			
			
			
			
			
			
			
			logger.debug("[FederationProxyServlet.executeProxy()] \t Process Request-Response");
			logger.debug("[FederationProxyServlet.executeProxy()] \t\t sourceSamlObjectType: " + federationProxySamlManager.getSamlObjectType(proxyBean.getSourceSamlObject()));
			switch (federationProxySamlManager.getSamlObjectType(proxyBean.getSourceSamlObject())){
				case SamlManager.SAMLTYPE_AUTHNREQUEST:
					proxyBean = processAuthenticationRequest(request, response, proxyBean);
					break;
				case SamlManager.SAMLTYPE_ATTRIBUTEQUERY:
					proxyBean = processAttributeQueryRequest(request, response, proxyBean);
					break;
				case SamlManager.SAMLTYPE_AUTHDECISIONQUERY:
					proxyBean = processAuthorizationQueryRequest(request, response, proxyBean);
					break;
				case SamlManager.SAMLTYPE_RESPONSEIMPL:
					String proxyBeanKey = federationProxySamlManager.extractSamlInResponseTo(proxyBean.getSourceSamlObject());
					ProxyBean requestProxyBean = federationProxyRouteManager.loadProxyBean(proxyBeanKey);					
					//translation
					if (requestProxyBean.isFromIA()) { //next step in flow
						proxyBean.setFromIA(false); //SAML came from FP
					} else proxyBean.setFromIA(true);
					proxyBean = processResponseImplRequest(request, response, proxyBean);
					break;
			}
			logger.debug("[FederationProxyServlet.executeProxy()] \t SignTargetSamlObject");
			SAMLObject targetSamlObject = proxyBean.getTargetSamlObject();	        
			boolean trustSignFlag = proxyBean.getProxySignFlag();
	        logger.debug("[FederationProxyServlet.executeProxy()] \t\t trustSignFlag: " + trustSignFlag);
            if(trustSignFlag){
                federationProxySamlManager.signSamlObject((SignableXMLObject)targetSamlObject, federationProxyTrustManager.getTrustPkcsPublicKey(), federationProxyTrustManager.getTrustPkcsPrivateKey());
                logger.debug("[FederationProxyServlet.executeProxy()] \t\t\t SamlObjectSigned");
				proxyBean.setProxySignAchievedFlag(true);
            }
			logger.debug("[FederationProxyServlet.executeProxy()] \t ManageDataManager");
			switch (federationProxySamlManager.getSamlObjectType(proxyBean.getSourceSamlObject())){
				case SamlManager.SAMLTYPE_AUTHNREQUEST:
				case SamlManager.SAMLTYPE_ATTRIBUTEQUERY:
				case SamlManager.SAMLTYPE_AUTHDECISIONQUERY:
					logger.debug("[FederationProxyServlet.executeProxy()] \t\t InitFederationProxyStep");
					federationProxyRouteManager.storeProxyBean(federationProxySamlManager.extractSamlObjectId(proxyBean.getTargetSamlObject()), proxyBean);
					logger.debug("[FederationProxyServlet.executeProxy()] \t\t\t proxyBean STORED");
					break;
				case SamlManager.SAMLTYPE_RESPONSEIMPL:
					logger.debug("[FederationProxyServlet.executeProxy()] \t\t CompleteFederationProxyStep");
					String proxyBeanKey = federationProxySamlManager.extractSamlInResponseTo(proxyBean.getSourceSamlObject());
					//TODO We must think about this
					//Audit
					ProxyBean requestProxyBean = federationProxyRouteManager.loadProxyBean(proxyBeanKey);
					ProxyBean responseProxyBean = proxyBean;
					String auditId = federationProxySamlManager.extractSamlObjectId(requestProxyBean.getSourceSamlObject());
					federationProxyAuditManager.audit(auditId, requestProxyBean, responseProxyBean);
					//RemoveCurrentProxyBean from ProxyBeanRouteTable
					federationProxyRouteManager.removeProxyBean(proxyBeanKey);
					logger.debug("[FederationProxyServlet.executeProxy()] \t\t\t proxyBean REMOVED");
					break;
				default: 
					throw new UnexpectedSourceSamlObjectTypeException();
			}
			logger.debug("[FederationProxyServlet.executeProxy()] \t\t\t proxyBean: " + proxyBean);
			logger.debug("[FederationProxyServlet.executeProxy()] \t SendTargetSamlObject");
			String targetUrl = proxyBean.getProxyUrl();
			String targetRelayState = proxyBean.getProxyRelayState();
			logger.debug("[FederationProxyServlet.executeProxy()] \t\t targetUrl: " + targetUrl);
			logger.debug("[FederationProxyServlet.executeProxy()] \t\t targetRelayState: " + targetRelayState);
			
			FederationProxyServlet.sendRedirectSamlObject(response, targetUrl, targetSamlObject, targetRelayState);
			return proxyBean;
		} finally {
			logger.debug("[FederationProxyServlet.executeProxy()] End");			
		}
	}
	
	/**
	 * processAuthenticationRequest
	 * @param request
	 * @param response
	 * @param proxyBean
	 * @return
	 * @throws Exception
	 */
	private ProxyBean processAuthenticationRequest(HttpServletRequest request, HttpServletResponse response, ProxyBean proxyBean) throws Exception {
		try {
			logger.debug("[FederationProxyServlet.processAuthenticationRequest()] Start");
			String targetUrl = proxyBean.getProxyUrl();
			String targetRelayState = proxyBean.getProxyRelayState();
			String targetSamlIssuer = proxyBean.getProxySamlIssuer();
			String targetSamlIdpProvider = proxyBean.getProxySamlIdpProvider();
			Conditions targetSamlConditions = proxyBean.getProxySamlConditions();
			logger.debug("[FederationProxyServlet.processAuthenticationRequest()] \t GenerateAuthnRequest");
			logger.debug("[FederationProxyServlet.processAuthenticationRequest()] \t\t targetUrl: " + targetUrl);
			logger.debug("[FederationProxyServlet.processAuthenticationRequest()] \t\t targetRelayState: " + targetRelayState);
			logger.debug("[FederationProxyServlet.processAuthenticationRequest()] \t\t targetSamlIssuer: " + targetSamlIssuer);
	        AuthnRequest authnRequest = SAMLUtil.createAuthNRequest(targetSamlIssuer, targetUrl, targetRelayState);
	        logger.debug("[FederationProxyServlet.processAuthenticationRequest()] \t InsertSamlIdpProviderId");
	        logger.debug("[FederationProxyServlet.processAuthenticationRequest()] \t\t targetSamlIdpProvider: " + targetSamlIdpProvider);
	        federationProxySamlManager.insertSamlIdpProviderId(authnRequest, targetSamlIdpProvider);
	        logger.debug("[FederationProxyServlet.processAuthenticationRequest()] \t InsertSamlConditions");
	        logger.debug("[FederationProxyServlet.processAuthenticationRequest()] \t\t targetSamlOneTimeUseFlag: " + federationProxySamlManager.extractSamlOneTimeUseFlag(targetSamlConditions));
	        logger.debug("[FederationProxyServlet.processAuthenticationRequest()] \t\t targetSamlNotOnAfter: " + federationProxySamlManager.extractSamlNotOnAfter(targetSamlConditions));
	        logger.debug("[FederationProxyServlet.processAuthenticationRequest()] \t\t targetSamlNotBefore: " + federationProxySamlManager.extractSamlNotBefore(targetSamlConditions));
	        logger.debug("[FederationProxyServlet.processAuthenticationRequest()] \t\t targetSamlAudienceRestrictionIdList: " + federationProxySamlManager.extractSamlAudienceRestrictionIdList(targetSamlConditions));
	        federationProxySamlManager.insertSamlConditions(authnRequest, targetSamlConditions);
            //saveTargetSamlObject
            proxyBean.setTargetSamlObject(authnRequest);                
			return proxyBean;
		} finally {
			logger.debug("[FederationProxyServlet.processAuthenticationRequest()] End");			
		}
	}

	/**
	 * processAuthenticationRequest
	 * @param request
	 * @param response
	 * @param proxyBean
	 * @return
	 * @throws Exception
	 */
	private ProxyBean processAttributeQueryRequest(HttpServletRequest request, HttpServletResponse response, ProxyBean proxyBean) throws Exception {
		try {
			logger.debug("[FederationProxyServlet.processAttributeQueryRequest()] Start");
			
			logger.debug("[FederationProxyServlet.processAttributeQueryRequest()] \t ManageSamlAttributes");
			List<SAMLAttribute> targetSamlQueriedAttributes = null;			
			{
				logger.debug("[FederationProxyServlet.processAttributeQueryRequest()] \t\t retrieveSourceSamlQueriedAttributes");
				List<SAMLAttribute> sourceSamlQueriedAttributes = federationProxySamlManager.extractSamlQueryAttributes(proxyBean.getSourceSamlObject());
				logger.debug("[FederationProxyServlet.processAttributeQueryRequest()] \t\t\t  sourceSamlQueriedAttributes: " + sourceSamlQueriedAttributes);
				logger.debug("[FederationProxyServlet.processAttributeQueryRequest()] \t\t generateTargetSamlQueriedAttributes");
				boolean denyRequest = false;
				String firstDeniedAttribute = "";
				if (!federationProxyTranslationManager.getTranslationEnabledFlag()){
					logger.debug("[FederationProxyServlet.processAttributeQueryRequest()] \t\t\t translationModuleDisabled so TargetQueriedAttributes are SourceQueriedAttributes");
					//targetSamlQueriedAttributes = sourceSamlQueriedAttributes;
					targetSamlQueriedAttributes = new ArrayList<SAMLAttribute>();				 
					if (sourceSamlQueriedAttributes != null){
						int sourceSamlQueriedAttributesSize = sourceSamlQueriedAttributes.size();
						String proxyFromAttributeStream = null;
						SAMLAttribute proxyToAttributeStream;
						boolean fromIA = proxyBean.isFromIA();
						int i = 0;
						while ((i < sourceSamlQueriedAttributesSize) && (!denyRequest)){
							boolean denyAttribute = false;
							proxyToAttributeStream = new SAMLAttribute("");
							proxyFromAttributeStream = sourceSamlQueriedAttributes.get(i).getName();
							if (fromIA){
								proxyToAttributeStream.setName(proxyFromAttributeStream);
								proxyToAttributeStream.setFriendlyName(sourceSamlQueriedAttributes.get(i).getFriendlyName());
								proxyToAttributeStream.setRequired(sourceSamlQueriedAttributes.get(i).isRequired());
							} else {
								RequestType xacmlRequest = federationProxyPolicyManager.PEPCreateRequest(proxyFromAttributeStream, "any");
								boolean resultPolicy = federationProxyPolicyManager.PEPHandleResponse(xacmlRequest);
								if (!resultPolicy){		//resultPolicy could be different from "deny"/"permit" ("not applicable", etc), careful				
									if (sourceSamlQueriedAttributes.get(i).isRequired()) {
										denyRequest = true;
										firstDeniedAttribute = proxyFromAttributeStream;
									} else denyAttribute = true;
								} else {
									proxyToAttributeStream.setName(proxyFromAttributeStream);
									
									proxyToAttributeStream.setFriendlyName(sourceSamlQueriedAttributes.get(i).getFriendlyName());
									proxyToAttributeStream.setRequired(sourceSamlQueriedAttributes.get(i).isRequired());
									}
							}
							if (!denyAttribute) targetSamlQueriedAttributes.add(proxyToAttributeStream);
							i++;
														
						}
					}
						//Send empty attribute request forward or error message back to FP origin
						//if (denyRequest) targetSamlQueriedAttributes = new ArrayList<SAMLAttribute>();
						if (denyRequest){
							//String proxyBeanKey = federationProxySamlManager.extractSamlInResponseTo(proxyBean.getSourceSamlObject());
							//ProxyBean requestProxyBean = federationProxyRouteManager.loadProxyBean(proxyBeanKey);
							
							String attributeQueryId = federationProxySamlManager.extractSamlObjectId((AttributeQuery) proxyBean.getSourceSamlObject());
							
							String targetUrl = proxyBean.getSourceRelayState();
							//String targetRelayState = proxyBean.getProxyRelayState();
							String targetSamlIssuer = proxyBean.getProxySamlIssuer();
							String targetSamlInResponseTo = attributeQueryId;
							//String targetSamlInResponseTo = attributeQueryId;
							String statusCode = StatusCode.REQUEST_DENIED_URI;
							String statusMessage = "Request denied because a mandatory attribute: " + firstDeniedAttribute + " cannot be released from the target federation";
							
							Response samlResponse = SAMLUtil.createErrorResponse(targetSamlIssuer, targetSamlInResponseTo, targetUrl, statusCode, statusMessage);
							logger.debug("[FederationProxyServlet.processAttributeQueryRequest()] \t SaveTargetSamlObject");
							{
								proxyBean.setTargetSamlObject(samlResponse);
							}
							proxyBean.setProxyUrl(proxyBean.getSourceRelayState());
							return proxyBean;
						}
						
				} else { //translationEnabled
					logger.debug("[FederationProxyServlet.processAttributeQueryRequest()] \t\t\t translationModuleEnabled so TargetQueriedAttributes are TranslatedSourceQueriedAttributes");
					logger.debug("[FederationProxyServlet.processAttributeQueryRequest()] \t\t\t\t retrieveTranslationParameters");
					/*String proxyFromResourceBundleBaseName = proxyBean.getProxyFromResourceBundleBaseName();
					Locale proxyFromLocale = proxyBean.getProxyFromLocale();
					String proxyToResourceBundleBaseName = proxyBean.getProxyToResourceBundleBaseName();
					Locale proxyToLocale = proxyBean.getProxyToLocale();
					logger.debug("[FederationProxyServlet.processAttributeQueryRequest()] \t\t\t\t\t proxyFromResourceBundleBaseName: " + proxyFromResourceBundleBaseName);
					logger.debug("[FederationProxyServlet.processAttributeQueryRequest()] \t\t\t\t\t proxyFromLocale: " + proxyFromLocale);
					logger.debug("[FederationProxyServlet.processAttributeQueryRequest()] \t\t\t\t\t proxyToResourceBundleBaseName: " + proxyToResourceBundleBaseName);
					logger.debug("[FederationProxyServlet.processAttributeQueryRequest()] \t\t\t\t\t proxyToLocale: " + proxyToLocale);
					logger.debug("[FederationProxyServlet.processAttributeQueryRequest()] \t\t\t\t translateAttributes");
					*/
					targetSamlQueriedAttributes = new ArrayList<SAMLAttribute>();
					//boolean denyRequest = false; //in case a mandatory attribute wont be able to be released by the FP target
					if (sourceSamlQueriedAttributes != null){
						int sourceSamlQueriedAttributesSize = sourceSamlQueriedAttributes.size();
						String proxyFromAttributeStream = null;
						SAMLAttribute proxyToAttributeStream;
						boolean fromIA = proxyBean.isFromIA(); //true if request comes from IA, false if it comes from FP
						int i = 0;
						while ((i < sourceSamlQueriedAttributesSize) && (!denyRequest)){
							
							boolean denyAttribute = false; //in case an optional attribute wont be able to be released by the FP target
							//AQUIIIIII
							proxyToAttributeStream = new SAMLAttribute("");
							proxyFromAttributeStream = sourceSamlQueriedAttributes.get(i).getName();
							//proxyToAttributeStream.setName(federationProxyTranslationManager.translateStream(proxyFromResourceBundleBaseName, proxyFromLocale, proxyToResourceBundleBaseName, proxyToLocale, proxyFromAttributeStream));
							//translation
							if (fromIA){
								proxyToAttributeStream.setName(federationProxyTranslationManager.translateLabelIntoEnglish(proxyFromAttributeStream));
								if (sourceSamlQueriedAttributes.get(i).getFriendlyName()!=null){
									proxyToAttributeStream.setFriendlyName(federationProxyTranslationManager.translateFriendlyName(sourceSamlQueriedAttributes.get(i).getFriendlyName(), true));
									
								}
								//proxyToAttributeStream.setFriendlyName(sourceSamlQueriedAttributes.get(i).getFriendlyName());
								proxyToAttributeStream.setRequired(sourceSamlQueriedAttributes.get(i).isRequired());
							} else { //request comes from FP
								
								//check if the FP will be able to release the attribute according to the policies.
								//if it wont be able to, then it is removed from the attribute list in case it is 
								//an optional attribute, or the whole request is denied in case it is a mandatory attribute
								//attributes in FP policies are in English if translationIsEnabled
								RequestType xacmlRequest = federationProxyPolicyManager.PEPCreateRequest(proxyFromAttributeStream, "any"); //requester could be proxyBean.getProxySamlIssuer()
								boolean resultPolicy = federationProxyPolicyManager.PEPHandleResponse(xacmlRequest);
								if (!resultPolicy){		//resultPolicy could be different from "deny"/"permit" ("not applicable", etc), careful				
									if (sourceSamlQueriedAttributes.get(i).isRequired()) {
										denyRequest = true;
										firstDeniedAttribute = proxyFromAttributeStream;
									} else denyAttribute = true;
									
								} else {
									//translation into FP language
									proxyToAttributeStream.setName(federationProxyTranslationManager.translateLabelIntoFPLanguage(proxyFromAttributeStream));
									if (sourceSamlQueriedAttributes.get(i).getFriendlyName()!=null){
										proxyToAttributeStream.setFriendlyName(federationProxyTranslationManager.translateFriendlyName(sourceSamlQueriedAttributes.get(i).getFriendlyName(), false));
										
									}
									//proxyToAttributeStream.setFriendlyName(sourceSamlQueriedAttributes.get(i).getFriendlyName());
									proxyToAttributeStream.setRequired(sourceSamlQueriedAttributes.get(i).isRequired());
								}	
								
							}
										
							//list of SAML Attributes, only those that will be able to be released according to policies
							//shall be added to the request. If a mandatory attribute is denied, then the request should 
							//not be forwarded (empty attribute list in the request or response sent straight back to the FP origin).
							if (!denyAttribute) targetSamlQueriedAttributes.add(proxyToAttributeStream);
							i++;
						}
					}
					//Instead of sending an empty attribute list, an improvement would be to send an empty response back notifying 
					//that the request was not successful because a mandatory attribute couldnt be released from the target federation.
					//if (denyRequest) targetSamlQueriedAttributes = new ArrayList<SAMLAttribute>();
					if (denyRequest){
						String attributeQueryId = federationProxySamlManager.extractSamlObjectId((AttributeQuery) proxyBean.getSourceSamlObject());			
						String targetUrl = proxyBean.getSourceRelayState();
						//String targetRelayState = proxyBean.getProxyRelayState();
						String targetSamlIssuer = proxyBean.getProxySamlIssuer();
						String targetSamlInResponseTo = attributeQueryId;
						String statusCode = StatusCode.REQUEST_DENIED_URI;
						String statusMessage = "Request denied because a mandatory attribute: " + firstDeniedAttribute + " cannot be released from the target federation";
						
						Response samlResponse = SAMLUtil.createErrorResponse(targetSamlIssuer, targetSamlInResponseTo, targetUrl, statusCode, statusMessage);
						logger.debug("[FederationProxyServlet.processAttributeQueryRequest()] \t SaveTargetSamlObject");
						{
							proxyBean.setTargetSamlObject(samlResponse);
						}
						proxyBean.setProxyUrl(proxyBean.getSourceRelayState());
						return proxyBean;
					}
					
					logger.debug("[FederationProxyServlet.processAttributeQueryRequest()] \t\t\t targetSamlQueriedAttributes: " + targetSamlQueriedAttributes);
				}
			}

			logger.debug("[FederationProxyServlet.processAttributeQueryRequest()] \t ManageSamlSubjectNameId");
			String targetSamlSubjectNameId = null;
			{
				//TODO JVR	This block is neccesary to pseudonims module implemenation (Ver IAServletBasic.processAttributeRequest(), translateRequestSubjectId(), subjectNameIdValue)
				//			But I think pseudonim module must be not implemented in FederationProxy, because it already done at IA
				//			I have talked with Gines about this subject and he is agree with me
				//			So, no pseudonim module for FP yet 
				String proxySamlSubjectNameId = proxyBean.getProxySamlSubjectNameId();
				if (proxySamlSubjectNameId != null){
					targetSamlSubjectNameId = proxySamlSubjectNameId; 
				}
				logger.debug("[FederationProxyServlet.processAttributeQueryRequest()] \t\t targetSamlSubjectNameId: " + targetSamlSubjectNameId);
			}
			
			logger.debug("[FederationProxyServlet.processAttributeQueryRequest()] \t GenerateAttributeRequest");
			AttributeQuery attributeQuery = null;
			{
				logger.debug("[FederationProxyServlet.processAttributeQueryRequest()] \t\t retrieveRequestParameters");
				String targetUrl = proxyBean.getProxyUrl();
				String targetRelayState = proxyBean.getProxyRelayState();
				String targetSamlIssuer = proxyBean.getProxySamlIssuer();
				String targetSamlSubjectnameQualifier = proxyBean.getProxySamlSubjectNameQualifier();
				logger.debug("[FederationProxyServlet.processAttributeQueryRequest()] \t\t\t targetUrl: " + targetUrl);
				logger.debug("[FederationProxyServlet.processAttributeQueryRequest()] \t\t\t targetRelayState: " + targetRelayState);
				logger.debug("[FederationProxyServlet.processAttributeQueryRequest()] \t\t\t targetSamlIssuer: " + targetSamlIssuer);
				logger.debug("[FederationProxyServlet.processAttributeQueryRequest()] \t\t\t targetSamlSubjectnameQualifier: " + targetSamlSubjectnameQualifier);
				logger.debug("[FederationProxyServlet.processAttributeQueryRequest()] \t\t generateAttributeQuery");
				
				X509Certificate targetSamlOriginCertificate = proxyBean.getProxySamlOriginCertificate();
				String	targetSamlOriginIssuer = proxyBean.getProxySamlOriginIssuer();
				//File file = new File("D:/SEMIRAMIS_2.0/workspace/IA_Or/WebContent/WEB-INF/certs/SP.crt");
				//AttributeQuery attrQuery = SAMLUtil.createAttrQuery(subjectId, subjectDomain, config.SEMIRAMIS_CLIENT_ID, originIssuer, provider.getUrl(), samlAttributes, attrEncrX509Cert);
				if 	(targetSamlOriginCertificate!=null){ 
					if (targetSamlOriginIssuer!=null){ //both not null
						attributeQuery = SAMLUtil.createAttrQuery(targetSamlSubjectNameId, targetSamlSubjectnameQualifier, targetSamlIssuer, targetSamlOriginIssuer, targetUrl, targetSamlQueriedAttributes, targetSamlOriginCertificate);
					} else {
						attributeQuery = SAMLUtil.createAttrQuery(targetSamlSubjectNameId, targetSamlSubjectnameQualifier, targetSamlIssuer, "test", targetUrl, targetSamlQueriedAttributes, targetSamlOriginCertificate);									
					}
				} else {//Only targetSamlOriginCertificate=null; both cannot be null, else if would have thrown exception when retrieving them from SAML request
					attributeQuery = SAMLUtil.createAttrQuery(targetSamlSubjectNameId, targetSamlSubjectnameQualifier, targetSamlIssuer, targetSamlOriginIssuer, targetUrl, targetSamlQueriedAttributes);
					
				}
			}
			
			logger.debug("[FederationProxyServlet.processAttributeQueryRequest()] \t ManageAssertions");
			{
				List<Assertion> targetSamlAssertionList = proxyBean.getProxySamlAssertionList();
				logger.debug("[FederationProxyServlet.processAttributeQueryRequest()] \t\t targetSamlAssertionList: " + targetSamlAssertionList);
				if(targetSamlAssertionList != null){
					SAMLUtil.insertAssertions(attributeQuery, targetSamlAssertionList);
				}
			}
			
			logger.debug("[FederationProxyServlet.processAttributeQueryRequest()] \t SaveTargetSamlObject");
			{
				proxyBean.setTargetSamlObject(attributeQuery);
			}
			
			return proxyBean;
		} finally {
			logger.debug("[FederationProxyServlet.processAttributeQueryRequest()] End");			
		}
	}
	
	/**
	 * processAuthorizationQueryRequest
	 * @param request
	 * @param response
	 * @param proxyBean
	 * @return
	 * @throws Exception
	 */
	private ProxyBean processAuthorizationQueryRequest(HttpServletRequest request, HttpServletResponse response, ProxyBean proxyBean) throws Exception {
		try {
			logger.debug("[FederationProxyServlet.processAuthorizationQueryRequest()] Start");
			String targetUrl = proxyBean.getProxyUrl();
			String targetRelayState = proxyBean.getProxyRelayState();
			String targetSamlIssuer = proxyBean.getProxySamlIssuer();
			String targetSamlSubjectNameId = proxyBean.getProxySamlSubjectNameId();
			String targetSamlSubjectnameQualifier = proxyBean.getProxySamlSubjectNameQualifier();
			String targetSamlActionId = proxyBean.getProxySamlActionId();
			String targetSamlResourceId = proxyBean.getProxySamlResourceId();
            List<Assertion> targetSamlAssertionList = proxyBean.getProxySamlAssertionList();
			logger.debug("[FederationProxyServlet.processAuthorizationQueryRequest()] \t GenerateAttributeRequest");
			logger.debug("[FederationProxyServlet.processAuthorizationQueryRequest()] \t\t targetUrl: " + targetUrl);
			logger.debug("[FederationProxyServlet.processAuthorizationQueryRequest()] \t\t targetRelayState: " + targetRelayState);
			logger.debug("[FederationProxyServlet.processAuthorizationQueryRequest()] \t\t targetSamlIssuer: " + targetSamlIssuer);
			logger.debug("[FederationProxyServlet.processAuthorizationQueryRequest()] \t\t targetSamlSubjectNameId: " + targetSamlSubjectNameId);
			logger.debug("[FederationProxyServlet.processAuthorizationQueryRequest()] \t\t targetSamlSubjectnameQualifier: " + targetSamlSubjectnameQualifier);			
            logger.debug("[FederationProxyServlet.processAuthorizationQueryRequest()] \t\t targetSamlActionId: " + targetSamlActionId);
            logger.debug("[FederationProxyServlet.processAuthorizationQueryRequest()] \t\t targetSamlResourceId: " + targetSamlResourceId);
            AuthzDecisionQuery authzDecisionQuery = SAMLUtil.createAuthzQuery(targetSamlSubjectNameId, targetSamlSubjectnameQualifier, targetSamlActionId, targetSamlResourceId, targetSamlIssuer, targetUrl, null);
			logger.debug("[FederationProxyServlet.processAuthorizationQueryRequest()] \t ManageAssertions");
            logger.debug("[FederationProxyServlet.processAuthorizationQueryRequest()] \t\t targetSamlAssertionList: " + targetSamlAssertionList);
            if(targetSamlAssertionList != null){
                SAMLUtil.insertAssertions(authzDecisionQuery, targetSamlAssertionList);
            }
            //saveTargetSamlObject
            proxyBean.setTargetSamlObject(authzDecisionQuery);                
			return proxyBean;
		} finally {
			logger.debug("[FederationProxyServlet.processAuthorizationQueryRequest()] End");			
		}
	}

	
	
	/**
	 * 
	 * @param request
	 * @param response
	 * @param proxyBean
	 * @return
	 * @throws Exception
	 */
	private ProxyBean processResponseImplRequest(HttpServletRequest request, HttpServletResponse response, ProxyBean proxyBean) throws Exception {
		try {
			logger.debug("[FederationProxyServlet.processResponseImplRequest()] Start");
			logger.debug("[FederationProxyServlet.processResponseImplRequest()] \t GenerateTargetSamlAssertionList");
			int targetSamlInResponseToType = proxyBean.getProxySamlInResponseToType();
			boolean targetIsAttributeResponseFlag = false;
			if (targetSamlInResponseToType == SamlManager.SAMLTYPE_ATTRIBUTEQUERY){
				targetIsAttributeResponseFlag = true;
			}
			List<Assertion> proxySamlAssertionList = proxyBean.getProxySamlAssertionList();
			if (targetIsAttributeResponseFlag){
				if (federationProxyTranslationManager.getTranslationEnabledFlag()){
					logger.debug("[FederationProxyServlet.processResponseImplRequest()] \t\t TranslateAttributes(" + targetIsAttributeResponseFlag + ")");
					logger.debug("[FederationProxyServlet.processAttributeQueryRequest()] \t\t\t retrieveTranslationParameters");					
					/*String proxyFromResourceBundleBaseName = proxyBean.getProxyFromResourceBundleBaseName();
					Locale proxyFromLocale = proxyBean.getProxyFromLocale();
					String proxyToResourceBundleBaseName = proxyBean.getProxyToResourceBundleBaseName();
					Locale proxyToLocale = proxyBean.getProxyToLocale();
					logger.debug("[FederationProxyServlet.processResponseImplRequest()] \t\t\t\t proxyFromResourceBundleBaseName: " + proxyFromResourceBundleBaseName);
					logger.debug("[FederationProxyServlet.processResponseImplRequest()] \t\t\t\t proxyFromLocale: " + proxyFromLocale);
					logger.debug("[FederationProxyServlet.processResponseImplRequest()] \t\t\t\t proxyToResourceBundleBaseName: " + proxyToResourceBundleBaseName);
					logger.debug("[FederationProxyServlet.processResponseImplRequest()] \t\t\t\t proxyToLocale: " + proxyToLocale);
					*/
					String proxyFromAttributeKeyStream = null;
					String proxyToAttributeKeyStream = null;
					String proxyFromAttributeValueStream = null;
					String proxyToAttributeValueStream = null;
	                Iterator<Assertion> proxySamlAssertionListIterator = proxySamlAssertionList.iterator();
	                while (proxySamlAssertionListIterator.hasNext()) {
	                    Assertion proxySamlAssertion = (Assertion) proxySamlAssertionListIterator.next();
	                    Iterator<AttributeStatement> proxySamlAssertionAttributeStatementsIterator = proxySamlAssertion.getAttributeStatements().iterator();
	                    while (proxySamlAssertionAttributeStatementsIterator.hasNext()) {
	                        AttributeStatement proxySamlAssertionAttributeStatement = (AttributeStatement) proxySamlAssertionAttributeStatementsIterator.next();
	                        Iterator<Attribute> proxySamlAssertionAttributeStatementAttributesIterator = proxySamlAssertionAttributeStatement.getAttributes().iterator();
	                        while (proxySamlAssertionAttributeStatementAttributesIterator.hasNext()) {
	                            Attribute proxySamlAssertionAttributeStatementAttribute = (Attribute)proxySamlAssertionAttributeStatementAttributesIterator.next();
	                            //Translate AttributeKey
        						proxyFromAttributeKeyStream = (String) proxySamlAssertionAttributeStatementAttribute.getName();
        						//proxyToAttributeKeyStream = federationProxyTranslationManager.translateStream(proxyFromResourceBundleBaseName, proxyFromLocale, proxyToResourceBundleBaseName, proxyToLocale, proxyFromAttributeKeyStream);
        						if (federationProxyTranslationManager.getTranslationEnabledFlag()){ 
        							if (proxyBean.isFromIA()){
        								proxyToAttributeKeyStream = federationProxyTranslationManager.translateLabelIntoEnglish(proxyFromAttributeKeyStream);  
        								if ((String) proxySamlAssertionAttributeStatementAttribute.getFriendlyName()!=null){
        									proxySamlAssertionAttributeStatementAttribute.setFriendlyName(federationProxyTranslationManager.translateFriendlyName(proxySamlAssertionAttributeStatementAttribute.getFriendlyName(), true));				
        								}
        							} else {
        								proxyToAttributeKeyStream = federationProxyTranslationManager.translateLabelIntoFPLanguage(proxyFromAttributeKeyStream);
        								if ((String) proxySamlAssertionAttributeStatementAttribute.getFriendlyName()!=null){
        									proxySamlAssertionAttributeStatementAttribute.setFriendlyName(federationProxyTranslationManager.translateFriendlyName(proxySamlAssertionAttributeStatementAttribute.getFriendlyName(), false));				
        								}
        							}
        						} 
        						proxySamlAssertionAttributeStatementAttribute.setName(proxyToAttributeKeyStream);
        						
	        					//Translate AttributeValue
	        					/*List<XMLObject> attributeValuesXmlObjectList = proxySamlAssertionAttributeStatementAttribute.getAttributeValues();
	        					//TODO probar esto.- De momento a mi solo me llegaran atributos de tipo XSString... Tal vez sea necesario modificar esto cuando me lleguen atributos de otro tipo
	        					XMLObject attributeValuesXmlObject = attributeValuesXmlObjectList.get(0);
	        					if (attributeValuesXmlObject instanceof XSString){
	            					XSString attributeValueXSString = (XSString) attributeValuesXmlObject;
	            					proxyFromAttributeValueStream = attributeValueXSString.getValue();
	            					proxyToAttributeValueStream = federationProxyTranslationManager.translateStream(proxyFromResourceBundleBaseName, proxyFromLocale, proxyToResourceBundleBaseName, proxyToLocale, proxyFromAttributeValueStream);
	            					attributeValueXSString.setValue(proxyToAttributeValueStream);
	        					}*/
	                        }
	                    }
	                }
				}
			}
			//EncryptAssertion
			boolean targetEncryptFlag = proxyBean.getProxyEncryptFlag();
			String targetEncryptCertFilePath = proxyBean.getProxyEncryptCertFilePath();
			logger.debug("[FederationProxyServlet.processResponseImplRequest()] \t\t EncryptAssertions(" + proxyBean.getProxyEncryptFlag() + ")");
			List targetSamlAssertionList = null;
			if (!targetEncryptFlag){
            	//proxySamlAssertionList is stored as targetSamlAssertionList
            	targetSamlAssertionList = proxySamlAssertionList;
			} else {
				List<EncryptedAssertion> proxyEncryptedSamlAssertionList = null;				
				if (proxySamlAssertionList != null){
					proxyEncryptedSamlAssertionList = new ArrayList<EncryptedAssertion>();
					Iterator<Assertion> iterator = proxySamlAssertionList.iterator();
					Assertion assertion = null;
					EncryptedAssertion encryptedAssertion = null;
					while (iterator.hasNext()) {
						assertion = iterator.next();
						encryptedAssertion = SAMLUtil.encryptAssertion(assertion, targetEncryptCertFilePath);
						proxyEncryptedSamlAssertionList.add(encryptedAssertion);
					}
				}
				Integer proxyEncryptAchievedAssertionNumber = new Integer(proxyEncryptedSamlAssertionList.size());
				proxyBean.setProxyEncryptAchievedAssertionNumber(proxyEncryptAchievedAssertionNumber);
				//proxyEncryptedSamlAssertionList is stored as targetSamlAssertionList 
				targetSamlAssertionList = proxyEncryptedSamlAssertionList;
            }
			
            //TODO	This is a FederationProxy so it is not necessary to sign outcoming tokens
            //		They are already signed from origin IdentityAggregator
			//		Confirm this is the right way
			/*
			if(config.SIGN_OUTCOMING_TOKENS){
				Iterator<Assertion> it = assertions.iterator();
				while (it.hasNext()) {
					Assertion assertion = it.next();
					SAMLUtil.signXML(assertion, config.API_PKCS12_PATH, config.API_PKCS12_PASSWORD, config.API_PKCS12_CERT_ALIAS, config.API_PRIVATE_KEY_PASSWORD);
				}
			}
			*/
			//GenerateResponse
			logger.debug("[FederationProxyServlet.processResponseImplRequest()] \t GenerateResponse");
			String targetUrl = proxyBean.getProxyUrl();
			String targetRelayState = proxyBean.getProxyRelayState();
			String targetSamlIssuer = proxyBean.getProxySamlIssuer();
			String targetSamlInResponseTo = proxyBean.getProxySamlInResponseTo(); 
			logger.debug("[FederationProxyServlet.processResponseImplRequest()] \t\t targetUrl: " + targetUrl);
			logger.debug("[FederationProxyServlet.processResponseImplRequest()] \t\t targetRelayState: " + targetRelayState);
			logger.debug("[FederationProxyServlet.processResponseImplRequest()] \t\t targetSamlIssuer: " + targetSamlIssuer);
			logger.debug("[FederationProxyServlet.processResponseImplRequest()] \t\t targetSamlInResponseTo: " + targetSamlInResponseTo);
			logger.debug("[FederationProxyServlet.processResponseImplRequest()] \t\t targetSamlInResponseToType: " + targetSamlInResponseToType);
			logger.debug("[FederationProxyServlet.processResponseImplRequest()] \t\t targetEncryptFlag: " + targetEncryptFlag);
			logger.debug("[FederationProxyServlet.processResponseImplRequest()] \t\t targetEncryptCertFilePath: " + targetEncryptCertFilePath);
			logger.debug("[FederationProxyServlet.processResponseImplRequest()] \t\t targetSamlAssertionList: " + targetSamlAssertionList);
			Response samlResponse;
			if (proxyBean.getProxyStatusCode().equals("urn:oasis:names:tc:SAML:2.0:status:Success")){
				samlResponse = SAMLUtil.createResponse(
						targetSamlIssuer, 
						targetSamlInResponseTo,  
						targetUrl, 
						targetSamlAssertionList);
						//proxyBean.getProxyStatusCode());
	            //saveTargetSamlObject
			} else{
				samlResponse = SAMLUtil.createErrorResponse(targetSamlIssuer, targetSamlInResponseTo, targetUrl, proxyBean.getProxyStatusCode(), proxyBean.getProxyStatusMessage());
			}
			
            proxyBean.setTargetSamlObject(samlResponse);                
            return proxyBean;
		} finally {
			logger.debug("[FederationProxyServlet.processResponseImplRequest()] End");			
		}
	}

	
	/**
	 * getSamlObject
	 * @param request
	 * @return
	 * @throws Exception
	 */
	private SAMLObject getSamlObject(HttpServletRequest request) throws Exception{
		try {
			String requestMethod = request.getMethod();
			SAMLMessageDecoder samlMessageDecoder = federationProxySamlManager.getSamlMessageDecoderInstance(requestMethod);
			SAMLObject samlObject = SAMLUtil.decodeSAML(request, samlMessageDecoder);
			return samlObject;
		} catch (Exception e){
			throw new InvalidSourceSamlObjectException();
		}
		
	}
	/**
	 * Redirect the user generating a new HTTPPost request (instead of HTTPGet).
	 * This request contains the SAML message which will be sent.
	 * @throws Exception
	 */
	public static void sendRedirectSamlObject(HttpServletResponse response, String postTargetUrl, SAMLObject postSamlObject, String postResponseURL) throws Exception {
		try {
			logger.debug("[FederationProxyServlet.sendRedirectSamlObject()] Start");
			//BuildSamlEndPoint(TargetLocation)
			XMLObjectBuilderFactory xmlObjectBuilderFactory = Configuration.getBuilderFactory();
			SAMLObjectBuilder samlObjectBuilder = (SAMLObjectBuilder) xmlObjectBuilderFactory.getBuilder(AssertionConsumerService.DEFAULT_ELEMENT_NAME);
			Endpoint samlEndpoint = (Endpoint) samlObjectBuilder.buildObject();
			samlEndpoint.setLocation(postTargetUrl);
			Cookie cookie = new Cookie("test", "test1");
			cookie.setVersion(0);
			response.addCookie(cookie);
			response.addCookie(new Cookie("test2", "test2"));
			//BuidSamlMessage
			HttpServletResponseAdapter httpServletResponseAdapter = new HttpServletResponseAdapter(response, false);
			BasicSAMLMessageContext basicSAMLMessageContext = new BasicSAMLMessageContext();
			basicSAMLMessageContext.setOutboundMessageTransport(httpServletResponseAdapter);
			basicSAMLMessageContext.setOutboundSAMLMessage(postSamlObject);
			basicSAMLMessageContext.setPeerEntityEndpoint(samlEndpoint);
			basicSAMLMessageContext.setRelayState(postResponseURL);
			//InitVelocityEngine(samlMessageParser)
			VelocityEngine velocityEngine = new VelocityEngine();
			velocityEngine.setProperty(RuntimeConstants.ENCODING_DEFAULT, "UTF-8");
			velocityEngine.setProperty(RuntimeConstants.OUTPUT_ENCODING, "UTF-8");
			velocityEngine.setProperty(RuntimeConstants.RESOURCE_LOADER, "classpath");
			velocityEngine.setProperty("classpath.resource.loader.class", "org.apache.velocity.runtime.resource.loader.ClasspathResourceLoader");
			velocityEngine.init();
			//SetHttpPotsMessageTemplate
			HTTPPostEncoder encoder = new HTTPPostEncoder(velocityEngine, federationProxyConfigManager.getProxyTemplateResourcePath());
			//sendHttpPostEncodedRequest
			encoder.encode(basicSAMLMessageContext);
		} finally {
			logger.debug("[FederationProxyServlet.sendRedirectSamlObject()] End");
		}
	}
	/**
	 * processRequest
	 * @param request
	 * @param response
	 * @param proxyBean
	 * @throws ServletException
	 * @throws IOException
	 */
	protected ProxyBean processRequest(HttpServletRequest request, HttpServletResponse response, ProxyBean proxyBean) throws ServletException, IOException{
		return proxyBean;
	}


}
