package eu.semiramis.federationproxy.audit;

import java.io.File;
import java.util.List;
import java.util.Locale;

import org.apache.log4j.Logger;
import org.opensaml.common.SAMLObject;
import org.opensaml.saml2.core.Assertion;
import org.opensaml.saml2.core.Conditions;
import org.opensaml.xml.Configuration;
import org.opensaml.xml.XMLObject;
import org.opensaml.xml.io.Marshaller;
import org.opensaml.xml.io.MarshallerFactory;
import org.opensaml.xml.util.XMLHelper;

import eu.semiramis.federationproxy.ProxyBean;
import eu.semiramis.util.file.FileUtil;
import eu.semiramis.util.string.StringUtil;


public class AuditManager {
	protected static Logger logger = Logger.getLogger(AuditManager.class);
	private static AuditManager auditManagerSingleInstance = null;	
	private String auditBasePath = null;
	private boolean auditEnabledFlag = false;
	
    public static String getAuditBasePath(){
    	String auditBasePath = null;
    	if (auditManagerSingleInstance != null){
    		auditBasePath = auditManagerSingleInstance.auditBasePath;
    	}
    	return auditBasePath;
    }

    
	public boolean getAuditEnabledFlag() {
		return auditEnabledFlag;
	}


	/**
	 * init
	 * @param federationProxyConfigFilePath
	 * @throws Exception
	 */
	public static void init(AuditBean auditBean, boolean auditEnabledFlag) throws Exception{
		logger.debug("[AuditManager.init()] Start");
		if (auditManagerSingleInstance == null){
			auditManagerSingleInstance = new AuditManager(auditBean, auditEnabledFlag);
		}
		logger.debug("[AuditManager.init()] End");
	}
	/**
	 * getInstance
	 */
	public static AuditManager getInstance(){
		return auditManagerSingleInstance;
	}
	/**
	 * AuditManager()
	 * @throws Exception
	 */
	private AuditManager(){}
	private AuditManager(AuditBean auditBean, boolean auditEnabledFlag) throws Exception{
		String paramAuditBasePath = null;
		boolean paramAuditEnabledFlag = false;
		if (auditBean != null){
			paramAuditBasePath = auditBean.getAuditBasePath();
			paramAuditEnabledFlag = auditEnabledFlag;
		}
		if (paramAuditBasePath != null && FileUtil.isExistentDirectory(paramAuditBasePath)){
			this.auditBasePath = paramAuditBasePath;
		} else {
			throw new Exception("InvalidAuditBasePath: " + paramAuditBasePath);
		}
		this.auditEnabledFlag = paramAuditEnabledFlag;
	}

	
	/**************************************************************/
	/* AuditBeanMapAccesors                                       */
	/**************************************************************/
	public void audit(String auditId, ProxyBean requestProxyBean, ProxyBean responseProxyBean) throws Exception{
		logger.debug("[AuditManager.audit()] Start(" + this.getAuditEnabledFlag() + ")");
		if (this.getAuditEnabledFlag()){
			logger.debug("[AuditManager.audit()] \t AuditRequestSaml ");
			SAMLObject requestSourceSamlObject = requestProxyBean.getSourceSamlObject();
			String requestSourceSamlObjectXmlFilePath = this.auditBasePath + File.separator + auditId + ".req" + ".src" + ".xml";
			logger.debug("[AuditManager.audit()] \t\t requestSourceSamlObjectXmlFilePath: " + requestSourceSamlObjectXmlFilePath);
			FileUtil.save(requestSourceSamlObjectXmlFilePath, toXml(requestSourceSamlObject));
			//SAMLObject requestTargetSamlObject = auditBean.getRequestTargetSAMLObject();
			//String requestTargetSamlObjectXmlFilePath = this.auditBasePath + File.separator + auditId + ".req" + ".tgt" + ".xml";
			//logger.debug("[AuditManager.audit()] \t\t requestTargetSamlObjectXmlFilePath: " + requestTargetSamlObjectXmlFilePath);
			//FileUtil.save(requestTargetSamlObjectXmlFilePath, toXml(requestTargetSamlObject));
			logger.debug("[AuditManager.audit()] \t AuditResponseSaml ");
			//SAMLObject responseSourceSamlObject = auditBean.getResponseSourceSAMLObject();
			//String responseSourceSamlObjectXmlFilePath = this.auditBasePath + File.separator + auditId + ".res" + ".src" + ".xml";
			//logger.debug("[AuditManager.audit()] \t\t responseSourceSamlObjectXmlFilePath: " + responseSourceSamlObjectXmlFilePath);
			//FileUtil.save(responseSourceSamlObjectXmlFilePath, toXml(responseSourceSamlObject));
			SAMLObject responseTargetSamlObject = responseProxyBean.getTargetSamlObject();
			String responseTargetSamlObjectXmlFilePath = this.auditBasePath + File.separator + auditId + ".res" + ".tgt" + ".xml";
			logger.debug("[AuditManager.audit()] \t\t responseTargetSamlObjectXmlFilePath: " + responseTargetSamlObjectXmlFilePath);
			FileUtil.save(responseTargetSamlObjectXmlFilePath, toXml(responseTargetSamlObject));
		}
		logger.debug("[AuditManager.audit()] End");
	}
	
    private String toXml(XMLObject xmlObject){
    	String xmlObjectStream = null;
    	try {
    		MarshallerFactory marshallerFactory = Configuration.getMarshallerFactory();
    		Marshaller marshaller = marshallerFactory.getMarshaller(xmlObject);
            org.w3c.dom.Element subjectElement = marshaller.marshall(xmlObject);
            xmlObjectStream = XMLHelper.nodeToString(subjectElement);
            xmlObjectStream = StringUtil.replace(xmlObjectStream, "><", ">\n<");
        } catch(Exception e) {
        	xmlObjectStream = "[ToXmlException: " + e.getMessage() + "]";
        }
        return xmlObjectStream;
    }

}
