package eu.semiramis.federationproxy.auditclient;

import javax.servlet.ServletRequest;

import org.apache.log4j.Logger;

import eu.semiramis.commons.audit.AuditFactory;
import eu.semiramis.commons.audit.Transaction;
import eu.semiramis.federationproxy.audit.AuditManager;
import eu.semiramis.federationproxy.config.ConfigBean;
import eu.semiramis.federationproxy.trust.TrustBean;
import eu.semiramis.federationproxy.trust.TrustManager;

public class Audit {
	protected static Logger logger = Logger.getLogger(Audit.class);
    public enum Level {

        DEBUG, ERROR, INFO, TRACE, WARN
    }

    public enum EventId {
    	FP_INITIALIZATION_INFO, FP_INITIALIZATION_ERROR,
    	FP_CONTEXT_INFO, FP_CONTEXT_ERROR,
    	FP_TRUST_INFO, FP_TRUST_ERROR,
    	FP_ROUTE_INFO, FP_ROUTE_ERROR,
    	FP_TRANSLATION_INFO, FP_TRANSLATION_ERROR,
    	FP_PKCS_INFO, FP_PKCS_ERROR,
        TOKEN_PROCESSING_ERROR,
        SIGNATURE_VALIDATION_INFO, SIGNATURE_VALIDATION_ERROR, INVALID_SIGNATURE,
        REQUEST_PROCESSING_INFO, REQUEST_PROCESSING_ERROR,
        RESPONSE_PROCESSING_INFO, RESPONSE_PROCESSING_ERROR, RESPONSE_GENERATION_ERROR,
        ATTRIBUTE_TRANSLATION_INFO, ATTRIBUTE_TRANSLATION_ERROR,
        ID_TRANSLATION_INFO, ID_TRANSLATION_ERROR,
        AUTHORIZATION_INFO, AUTHORIZATION_ERROR, UNAUTHORIZED_REQUEST,
        POLICY_EVALUATION_INFO, POLICY_EVALUATION_ERROR, INTERNAL_POLICIES_NOT_FOUND,
        DISCOVERY_INFO, USER_ORIGIN_NOT_FOUND, DESTINATION_NOT_FOUND,
        WAYF_INFO, WAYF_ERROR
    }
    public static final String TRANSACTION_ATTRIBUTE = "AuditTransaction";
    private static boolean factoryInitialized = false;
    private boolean isActive = false;

    public static void initAuditFactory(ConfigBean config, TrustManager t) {
    	logger.debug("[AuditClient.init()] Start");
        if (!factoryInitialized && config.getProxyRemoteAuditEnabled()) {
            AuditFactory.init(
            		config.getProxyAuditServerCfgPath(), 
            		t.getTrustPkcsPrivateKey(), 
            		config.getProxyDomain().toLowerCase(), 
            		config.getProxyId().toLowerCase());
        }
        logger.debug("[AuditClient.init()] End");
    }
    private eu.semiramis.commons.audit.Audit audit;

    public Audit(String handlerName, boolean isActive) {
    		this.isActive = isActive;
    		if(isActive)
    			this.audit = AuditFactory.getAudit(handlerName);
    }

    /*public static void initRequest(ServletRequest request) {
        request.setAttribute(TRANSACTION_ATTRIBUTE, new Transaction());
    }

    public void audit(Level level, ServletRequest request, EventId eventId, String msg) {
        audit(level, getTransaction(request), eventId, msg);
    	System.out.println("adt : " + level.name() + "msg: " + msg);
    }

    public void audit(Level level, ServletRequest request, EventId eventId, Exception ex) {
        audit(level, getTransaction(request), eventId, ex);
    	System.out.println("adt : " + level.name() + "ex: " + ex.getMessage());
    }*/

    public void audit(Level level, Transaction transaction, EventId eventId, String msg) {
    	if(isActive)
    		audit.audit(translateLevel(level), transaction, eventId.name(), msg);
    }

    public void audit(Level level, Transaction transaction, EventId eventId, Exception ex) {
    	if(isActive)
    		audit.audit(translateLevel(level), transaction, eventId.name(), ex);
    }

    private Transaction getTransaction(ServletRequest request) {
        Object transaction = request.getAttribute(TRANSACTION_ATTRIBUTE);
        if (transaction == null) {
            throw new IllegalStateException("Request not initialized for auditing");
        } else {
            return (Transaction) transaction;
        }
    }

    private eu.semiramis.commons.audit.Audit.Level translateLevel(Level level) {
        eu.semiramis.commons.audit.Audit.Level newLevel = null;
        switch (level) {
            case DEBUG:
                newLevel = eu.semiramis.commons.audit.Audit.Level.DEBUG;
                break;
            case ERROR:
                newLevel = eu.semiramis.commons.audit.Audit.Level.ERROR;
                break;
            case INFO:
                newLevel = eu.semiramis.commons.audit.Audit.Level.INFO;
                break;
            case TRACE:
                newLevel = eu.semiramis.commons.audit.Audit.Level.TRACE;
                break;
            case WARN:
                newLevel = eu.semiramis.commons.audit.Audit.Level.WARN;
                break;
        }
        return newLevel;
    }
}
