package eu.semiramis.federationproxy.config;

import eu.semiramis.federationproxy.audit.AuditBean;
import eu.semiramis.federationproxy.logger.LoggerBean;
import eu.semiramis.federationproxy.policy.PolicyBean;
import eu.semiramis.federationproxy.route.RouteBean;
import eu.semiramis.federationproxy.translation_old.TranslationBean;
import eu.semiramis.federationproxy.trust.PkcsBean;
import eu.semiramis.federationproxy.trust.TrustBean;
import eu.semiramis.util.xstream.XStreamUtil;


/***********************************************************************/
/* ConfigurationSampleLayoutStart                                      */
/***********************************************************************/
/* 
<eu.semiramis.federationproxy.config.ConfigBean>
  <proxyId>fpid</proxyId>
  <proxyUrl>http://localhost:fpport/fpid/FederationProxyServlet</proxyUrl>
  <proxyTemplateResourcePath>/eu/semiramis/federationproxy/templates/federationproxy-saml2-post-simplesign-binding.vm</proxyTemplateResourcePath>
  <auditBean>
    <auditBasePath>/usr/share/SEMIRAMIS/usecase/fpid/audit</auditBasePath>
    <auditEnabledFlag>true</auditEnabledFlag>
  </auditBean>
  <loggerBean>
    <loggerConfFilePath>/usr/share/SEMIRAMIS/usecase/fpid/conf/log4j.properties</loggerConfFilePath>
  </loggerBean>
  <pkcsBean>
    <pkcsCertFilePath>/usr/share/SEMIRAMIS/usecase/fpid/certs/fpid.p12</pkcsCertFilePath>
    <pkcsCertFilePassword>Semir@mis1</pkcsCertFilePassword>
    <pkcsCertFilePrivateKeyAlias>1</pkcsCertFilePrivateKeyAlias>
    <pkcsPrivateKeyPassword>Semir@mis1</pkcsPrivateKeyPassword>
  </pkcsBean>
  <trustBeanArray>
    <eu.semiramis.federationproxy.trust.TrustBean>
      <trustKey>trustid</trustKey>
      <trustCertFilePath>/usr/share/SEMIRAMIS/usecase/fpid/certs/trustid.crt</trustCertFilePath>
      <trustSignFlag>true</trustSignFlag>
      <trustValidateSignFlag>true</trustValidateSignFlag>
      <trustEncryptFlag>false</trustEncryptFlag>
    </eu.semiramis.federationproxy.trust.TrustBean>
    <eu.semiramis.federationproxy.trust.TrustBean>
      <trustKey>trustid</trustKey>
      <trustCertFilePath>/usr/share/SEMIRAMIS/usecase/fpid/certs/trustid.crt</trustCertFilePath>
      <trustSignFlag>true</trustSignFlag>
      <trustValidateSignFlag>true</trustValidateSignFlag>
      <trustEncryptFlag>false</trustEncryptFlag>      
    </eu.semiramis.federationproxy.trust.TrustBean>
  </trustBeanArray>  
  <routeBeanArray>
    <eu.semiramis.federationproxy.route.RouteBean>
      <routeKey>routeid</routeKey>
      <routeTargetUrl>nextstepurl</routeTargetUrl>
      <routeTrustKey>nextsteptrustid</routeTrustKey>
      <routeTranslationKey>nextsteptranslationid</routeTranslationKey>
    </eu.semiramis.federationproxy.route.RouteBean>
    <eu.semiramis.federationproxy.route.RouteBean>
      <routeKey>routeid</routeKey>
      <routeTargetUrl>nextstepurl</routeTargetUrl>
      <routeTrustKey>nextsteptrustid</routeTrustKey>
      <routeTranslationKey>nextsteptranslationid</routeTranslationKey>
    </eu.semiramis.federationproxy.route.RouteBean>
  </routeBeanArray>
  <translationBeanArray>
    <eu.semiramis.federationproxy.translation.TranslationBean>
      <translationKey>translationid</translationKey>
      <translationBaseName>eu.semiramis.resources.Attributes</translationBaseName>      
      <translationLanguage>locale_lang</translationLanguage>
      <translationCountry>locale_country</translationCountry>
      <translationVariant>locale_variant</translationVariant>
    </eu.semiramis.federationproxy.translation.TranslationBean>
    <eu.semiramis.federationproxy.translation.TranslationBean>
      <translationKey>translationid</translationKey>
      <translationBaseName>eu.semiramis.resources.Attributes</translationBaseName>      
      <translationLanguage>locale_lang</translationLanguage>
      <translationCountry>locale_country</translationCountry>
      <translationVariant>locale_variant</translationVariant>
    </eu.semiramis.federationproxy.translation.TranslationBean>
  </translationBeanArray>
</eu.semiramis.federationproxy.config.ConfigBean>
*/
/***********************************************************************/
/* ConfigurationSampleLayoutEnd                                        */
/***********************************************************************/

public class ConfigBean {

	private String proxyId = null;
	private String proxyFriendlyName = null;
	private String proxyDomain = null;
	private String proxyUrl = null;  
	private String proxyAuditServerCfgPath = null;
	private String proxyTemplateResourcePath = null;
    private boolean proxyAuditEnabledFlag = false;
    private boolean proxyTranslationEnabledFlag = false;
    private boolean proxyRemoteAuditEnabled = false;

	private PkcsBean pkcsBean = null;
	private LoggerBean loggerBean = null;
	private eu.semiramis.federationproxy.translation.TranslationBean translationBean = null;
	private TrustBean[] trustBeanArray = null;
	private RouteBean[] routeBeanArray = null;
	
	private PolicyBean policyBean = null;
	private AuditBean auditBean = null;
	
	
	/****************************************************/
	/* Getters and Setters                              */
	/****************************************************/
	
	public PolicyBean getPolicyBean() {
		return policyBean;
	}
	public void setPolicyBean(PolicyBean policyBean) {
		this.policyBean = policyBean;
	}
	
	public String getProxyFriendlyName() {
		return proxyFriendlyName;
	}
	public void setProxyFriendlyName(String proxyFriendlyName) {
		this.proxyFriendlyName = proxyFriendlyName;
	}
	
	public eu.semiramis.federationproxy.translation.TranslationBean getTranslationBean() {
		return translationBean;
	}
	public void setTranslationBean(
			eu.semiramis.federationproxy.translation.TranslationBean translationBean) {
		this.translationBean = translationBean;
	}

	
	public PkcsBean getPkcsBean() {
		return pkcsBean;
	}
	public String getProxyId() {
		return proxyId;
	}
	public void setProxyId(String proxyId) {
		this.proxyId = proxyId;
	}
	public String getProxyUrl() {
		return proxyUrl;
	}
	public void setProxyUrl(String proxyUrl) {
		this.proxyUrl = proxyUrl;
	}
	public String getProxyTemplateResourcePath() {
		return proxyTemplateResourcePath;
	}
	public void setProxyTemplateResourcePath(String proxyTemplateResourcePath) {
		this.proxyTemplateResourcePath = proxyTemplateResourcePath;
	}
	public boolean getProxyAuditEnabledFlag() {
		return proxyAuditEnabledFlag;
	}
	public void setProxyAuditEnabledFlag(boolean proxyAuditEnabledFlag) {
		this.proxyAuditEnabledFlag = proxyAuditEnabledFlag;
	}
	public boolean getProxyTranslationEnabledFlag() {
		return proxyTranslationEnabledFlag;
	}
	public void setProxyTranslationEnabledFlag(boolean proxyTranslationEnabledFlag) {
		this.proxyTranslationEnabledFlag = proxyTranslationEnabledFlag;
	}
	public void setProxyRemoteAuditEnabled(boolean proxyRemoteAuditEnabled) {
		this.proxyRemoteAuditEnabled = proxyRemoteAuditEnabled;
	}
	public boolean getProxyRemoteAuditEnabled() {
		return proxyRemoteAuditEnabled;
	}
	public String getProxyDomain() {
		return proxyDomain;
	}
	public void setProxyDomain(String proxyDomain) {
		this.proxyDomain = proxyDomain;
	}
	
	
	public void setPkcsBean(PkcsBean pkcsBean) {
		this.pkcsBean = pkcsBean;
	}
	public LoggerBean getLoggerBean() {
		return loggerBean;
	}
	public void setLoggerBean(LoggerBean loggerBean) {
		this.loggerBean = loggerBean;
	}
	public AuditBean getAuditBean() {
		return auditBean;
	}
	public void setAuditBean(AuditBean auditBean) {
		this.auditBean = auditBean;
	}
	public TrustBean[] getTrustBeanArray() {
		return trustBeanArray;
	}
	public void setTrustBeanArray(TrustBean[] trustBeanArray) {
		this.trustBeanArray = trustBeanArray;
	}
	public RouteBean[] getRouteBeanArray() {
		return routeBeanArray;
	}
	public void setRouteBeanArray(RouteBean[] routeBeanArray) {
		this.routeBeanArray = routeBeanArray;
	}
	/*public TranslationBean[] getTranslationBeanArray() {
		return translationBeanArray;
	}
	public void setTranslationBeanArray(TranslationBean[] translationBeanArray) {
		this.translationBeanArray = translationBeanArray;
	}*/
	public String getProxyAuditServerCfgPath() {
		return proxyAuditServerCfgPath;
	}
	public void setProxyAuditServerCfgPath(String cfgPath) {
		this.proxyAuditServerCfgPath = cfgPath;
	}
	
	public String toString(){
		String xmlStream = XStreamUtil.toXml(this);
		return xmlStream;
	}
	
}
