package eu.semiramis.federationproxy.config;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.net.URL;

import javax.servlet.Servlet;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;

import eu.semiramis.commons.audit.Transaction;
import eu.semiramis.federationproxy.audit.AuditManager;
import eu.semiramis.federationproxy.auditclient.Audit;
import eu.semiramis.federationproxy.auditclient.Audit.Level;
import eu.semiramis.federationproxy.logger.LoggerManager;
import eu.semiramis.federationproxy.policy.PolicyManager;
import eu.semiramis.federationproxy.route.RouteManager;
import eu.semiramis.federationproxy.saml.SamlManager;
import eu.semiramis.federationproxy.translation_old.TranslationManager;
import eu.semiramis.federationproxy.trust.TrustManager;
import eu.semiramis.federationproxy.wayf.WAYFManager;
import eu.semiramis.util.string.StringUtil;
import eu.semiramis.util.xstream.XStreamUtil;


public abstract class FederationProxyConfigurator extends HttpServlet implements Servlet {

	private static final String CONFIG_PARAMNAME_SERVLETCONFIGFILESYSTEMVARNAME		= "configFileSystemVarName";
	private static final String CONFIG_PARAMNAME_SERVLETCONFIGFILERESOURCECONTEXTPATH = "/WEB-INF/federationProxyServletConfigFilePath.txt";
	public static final String CONFIGKEY_FEDERATIONPROXY_CONFIGFILEPATH	= "federationProxy.configfilepath";
	
	public static final String HTTP_PARAMNAME_RELAYSTATE = "RelayState";
	
	protected static Logger logger = Logger.getLogger(FederationProxyConfigurator.class);

	protected static LoggerManager federationProxyLoggerManager = null;
	protected static ConfigBean federationProxyConfigManager = null;
	protected static TrustManager federationProxyTrustManager = null;
	protected static RouteManager federationProxyRouteManager = null;
	protected static SamlManager federationProxySamlManager = null;
	protected static eu.semiramis.federationproxy.translation.TranslationManager federationProxyTranslationManager = null;
	protected static PolicyManager federationProxyPolicyManager = null;
	protected static AuditManager federationProxyAuditManager = null;
	protected static WAYFManager federationProxyWAYFManager = null;
	protected Audit fpAuditManager;
	protected boolean isAuditEnabled = false;
	private static Audit audit = null;
	private Transaction initializationTransaction = new Transaction();
	
	/****************************************************************************************/
	/* Servlet Configuration                                                                */
	/****************************************************************************************/
	/**
	 * init
	 */
	public void init() throws ServletException{
		try{
			System.out.println("[FederationProxyConfigurator.init()] Start");
			System.out.println("[FederationProxyConfigurator.init()] \t InitFederationProxyConfigmanager");			
			{
				String federationProxyConfigFilePath = loadConfigFilePath();
				federationProxyConfigManager = (ConfigBean) XStreamUtil.loadXmlBeanInstanceFromFilePath(federationProxyConfigFilePath);
				isAuditEnabled = federationProxyConfigManager.getProxyRemoteAuditEnabled();
				
			}
			System.out.println("[FederationProxyConfigurator.init()] \t InitFederationProxyLoggerManager");
			{
				LoggerManager.init(federationProxyConfigManager.getLoggerBean());
				federationProxyLoggerManager = LoggerManager.getInstance();
			}
			System.out.println("[FederationProxyConfigurator.init()] \t Init federationProxyTrustManager");
			{
				TrustManager.init(federationProxyConfigManager.getTrustBeanArray(), federationProxyConfigManager.getPkcsBean());
				federationProxyTrustManager = TrustManager.getInstance();
			}		
			System.out.println("[FederationProxyConfigurator.init()] \t Init federationProxyRouteManager");
			{
				RouteManager.init(federationProxyConfigManager.getRouteBeanArray());
				federationProxyRouteManager = RouteManager.getInstance();
			}
			
			System.out.println("[FederationProxyConfigurator.init()] \t Init federationProxySamlManager");
			{
				SamlManager.init();
				federationProxySamlManager = SamlManager.getInstance();
			}
			System.out.println("[FederationProxyConfigurator.init()] \t Init federationProxyTranslationManager");
			{
				eu.semiramis.federationproxy.translation.TranslationManager.init(federationProxyConfigManager.getTranslationBean(), 
						federationProxyConfigManager.getProxyTranslationEnabledFlag());
				federationProxyTranslationManager = eu.semiramis.federationproxy.translation.TranslationManager.getInstance();
				
			}
			System.out.println("[FederationProxyConfigurator.init()] \t Init federationProxyAuditManager");
			{
				AuditManager.init(federationProxyConfigManager.getAuditBean(), federationProxyConfigManager.getProxyAuditEnabledFlag());
				federationProxyAuditManager = AuditManager.getInstance();
			}
			System.out.println("[FederationProxyConfigurator.init()] \t Init federationProxyPolicyManager");
			{
				PolicyManager.init(federationProxyConfigManager.getPolicyBean());
				federationProxyPolicyManager = PolicyManager.getInstance();
			}
			System.out.println("[FederationProxyConfigurator.init()] \t Init federationProxyWAYFManager");
			{
				WAYFManager.init(federationProxyConfigManager.getRouteBeanArray());
				federationProxyWAYFManager = WAYFManager.getInstance(); 
			}
			System.out.println("[FederationProxyConfigurator.init()] \t RemoteAuditClientManager");
			{
				Audit.initAuditFactory(federationProxyConfigManager, federationProxyTrustManager);
				fpAuditManager = new Audit(FederationProxyConfigurator.class.getName(), federationProxyConfigManager.getProxyRemoteAuditEnabled());
				fpAuditManager
					.audit(Level.DEBUG, 
							initializationTransaction, 
							Audit.EventId.FP_INITIALIZATION_INFO, 
							"FederationProxy Initialization completed");
			}
			System.out.println("[FederationProxyConfigurator.init()] End");
		} catch (Exception e) {			
			e.printStackTrace();
			throw new ServletException(e);
		}
	}
	/**
	 * destroy
	 */
	public void destroy(){
		try{
			System.out.println("[FederationProxyConfigurator.destroy()] Init");
			logger.debug("***************************************************");
			logger.debug("FederationProxyConfigurator.destroy()");
			logger.debug("***************************************************");
			System.out.println("[FederationProxyConfigurator.destroy()] End");	
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	/**
	 * loadConfigFilePath
	 */
	private String loadConfigFilePath(){
		try {
			System.out.println("[FederationProxyConfigurator.loadConfigFilePath()] Start");
			String configFilePath = null;
			if (configFilePath == null){
				System.out.println("[FederationProxyConfigurator.loadConfigFilePath()] 1.- Configure as CONFIG_PARAMNAME_SERVLETCONFIGFILESYSTEMVARNAME");
				String configFileSystemVarName = getInitParameter(CONFIG_PARAMNAME_SERVLETCONFIGFILESYSTEMVARNAME);
				System.out.println("[FederationProxyConfigurator.loadConfigFilePath()] \t configDirSystemVarName: " + configFileSystemVarName);
				if (configFileSystemVarName != null){
					configFilePath = System.getProperty(configFileSystemVarName);
				}
			}
			if (configFilePath == null){
				System.out.println("[FederationProxyConfigurator.loadConfigFilePath()] 2.- Configure as CONFIG_PARAMNAME_SERVLETCONFIGFILERESOURCECONTEXTPATH");
				configFilePath = loadServletContextResourceFile(CONFIG_PARAMNAME_SERVLETCONFIGFILERESOURCECONTEXTPATH);
			}
			if (configFilePath != null){
				configFilePath = configFilePath.trim();
				configFilePath = StringUtil.replace(configFilePath, "/", File.separator);
				configFilePath = StringUtil.replace(configFilePath, "\\", File.separator);
			}
			System.out.println("[FederationProxyConfigurator.loadConfigFilePath()] \t\t configFilePath: " + configFilePath);		
			return configFilePath;
		} finally {
			System.out.println("[FederationProxyConfigurator.loadConfigFilePath()] End");
		}
	}
	/**
	 * loadServletContextResourceFile
	 * @param resourceContextPath
	 * @return
	 */
    private String loadServletContextResourceFile(String resourceContextPath) {
        ByteArrayInputStream byteArrayInputStream = null;
        try {
    		System.out.println("[FederationProxyConfigurator.loadServletContextResourceFile()] Start");        	
            URL url = this.getServletContext().getResource(resourceContextPath);
            byteArrayInputStream = (ByteArrayInputStream) url.getContent();
            int length = byteArrayInputStream.available();
            byte[] buff = new byte[length];
            byteArrayInputStream.read(buff);
            String resourceStream = new String(buff); 
            return resourceStream;
        } catch (IOException ioe) {
        	System.err.println(ioe);
        } finally {
            try {
            	if (byteArrayInputStream != null){
            		byteArrayInputStream.close();
            	}
            } catch (Exception uncatched) {
            	System.err.println(uncatched);
            }
    		System.out.println("[FederationProxyConfigurator.loadServletContextResourceFile()] End");
        }
        return null;
    }
	/****************************************************************************************/
	/* Servlet Execution                                                                    */
	/****************************************************************************************/
	public class InvalidSourceSamlObjectException extends Exception{
		public InvalidSourceSamlObjectException() {
	    	super();
	    }
	}
	
	public class NullSourceSamlObjectException extends Exception{
		public NullSourceSamlObjectException() {
	    	super();
	    }
	}
	public class NullSourceRelayStateException extends Exception{
		public NullSourceRelayStateException() {
			super();
		}
	}
	public class NoOriginIssuerOrCertificateException extends Exception{
		public NoOriginIssuerOrCertificateException() {
			super();
		}
	}
	public class NullPkcsPrivateKeyException extends Exception{
		public NullPkcsPrivateKeyException() {
			super();
		}
	}
	public class NullPkcsPublicKeyException extends Exception{
		public NullPkcsPublicKeyException() {
			super();
		}
	}
	public class UndefinedTrustBeanException extends Exception{
		public UndefinedTrustBeanException() {
			super();
		}
		public UndefinedTrustBeanException(String message) {
			super(message);
		}
	}
	public class UndefinedTranslationBeanException extends Exception{
		public UndefinedTranslationBeanException() {
			super();
		}
		public UndefinedTranslationBeanException(String message) {
			super(message);
		}
	}
	
	public class UnexistentTrustCertFilePathException extends Exception{
		public UnexistentTrustCertFilePathException() {
			super();
		}
		public UnexistentTrustCertFilePathException(String message) {
			super(message);
		}
	}
	public class NotTrustAchievedException extends Exception{
		public NotTrustAchievedException() {
			super();
		}
	}
	public class UndefinedRouteBeanException extends Exception{
		public UndefinedRouteBeanException() {
			super();
		}
		public UndefinedRouteBeanException(String message) {
			super(message);
		}
	}
	public class UnstoredProxyBeanException extends Exception{
		public UnstoredProxyBeanException() {
			super();
		}
		public UnstoredProxyBeanException(String message) {
			super(message);
		}
	}
	public class UnexpectedSourceSamlObjectTypeException extends Exception{
		public UnexpectedSourceSamlObjectTypeException() {
			super();
		}
	} 
	public class NullTargetUrlException extends Exception{
		public NullTargetUrlException() {
			super();
		}
	}
	public class UnSuccessStatusCodeException extends Exception{
		public UnSuccessStatusCodeException() {
			super();
		}
	}

	
	/**
	 * doGet
	 */
	public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		try {
			logger.debug("[FederationProxyConfigurator.doGet()] Start");
			manageServletExecution(request, response);
		} finally {
			logger.debug("[FederationProxyConfigurator.doGet()] End");
		}
	}
	/**
	 * doPost
	 */
	public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		try {
			logger.debug("[FederationProxyConfigurator.doPost()] Start");
			manageServletExecution(request, response);
		} finally {
			logger.debug("[FederationProxyConfigurator.doPost()] End");
		}
	}
	/**
	 * manageServletExecution
	 * @param request
	 * @param response
	 * @throws ServletException
	 * @throws IOException
	 */
	private void manageServletExecution(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		try {
			logger.debug("[FederationProxyConfigurator.manageServletExecution()] Start");
			executeServlet(request, response);
		} catch (InvalidSourceSamlObjectException issoe){
			logger.error("[FederationProxyConfigurator.manageServletExecution()] \t InvalidSourceSamlObjectException: " + issoe);
			response.sendError(HttpServletResponse.SC_BAD_REQUEST, "Required parameter SamlObject wrong.");
		} catch (NullSourceSamlObjectException nssoe){
			logger.error("[FederationProxyConfigurator.manageServletExecution()] \t NullSourceSamlObjectException: " + nssoe);
			response.sendError(HttpServletResponse.SC_BAD_REQUEST, "Required parameter SamlObject not found.");
			audit.audit(Level.ERROR, initializationTransaction, Audit.EventId.FP_INITIALIZATION_ERROR, nssoe);
		} catch (NullSourceRelayStateException nsrse){
			logger.error("[FederationProxyConfigurator.manageServletExecution()] \t NullSourceRelayStateException: " + nsrse);
			response.sendError(HttpServletResponse.SC_BAD_REQUEST, "Required parameter " + HTTP_PARAMNAME_RELAYSTATE + " not found.");
		} catch (UndefinedTrustBeanException utbe){
			logger.error("[FederationProxyConfigurator.manageServletExecution()] \t UndefinedTrustBeanException: " + utbe);
			response.sendError(HttpServletResponse.SC_FORBIDDEN, "Sorry, your request can not be trusted");
		} catch (UndefinedTranslationBeanException utrabe){
			logger.error("[FederationProxyConfigurator.manageServletExecution()] \t UndefinedTranslationBeanException: " + utrabe);
			response.sendError(HttpServletResponse.SC_FORBIDDEN, "Sorry, your request can not be translated");
		} catch (NotTrustAchievedException ntae){
			logger.error("[FederationProxyConfigurator.manageServletExecution()] \t NotTrustAchievedException: " + ntae);
			response.sendError(HttpServletResponse.SC_FORBIDDEN, "Sorry, your request is not trusted valid");
		} catch (UndefinedRouteBeanException urbe){
			logger.error("[FederationProxyConfigurator.manageServletExecution()] \t UndefinedRouteBeanException: " + urbe);
			response.sendError(HttpServletResponse.SC_BAD_REQUEST, "SAML2 Your request can not be routed");
		} catch (UnexpectedSourceSamlObjectTypeException ussote){
			logger.error("[FederationProxyConfigurator.manageServletExecution()] \t UnexpectedSourceSamlObjectTypeException: " + ussote);
			response.sendError(HttpServletResponse.SC_BAD_REQUEST, "The parameter SamlObject has an unexpected value.");
		} catch (UnSuccessStatusCodeException usce){
			logger.error("[FederationProxyConfigurator.manageServletExecution()] \t UnSuccessStatusCodeException: " + usce);
			response.sendError(HttpServletResponse.SC_BAD_REQUEST, "The received SamlObject has not success status code.");
		} catch (Exception e) {
			logger.error("[FederationProxyConfigurator.manageServletExecution()] EXCEPTION: " + e.getMessage());
			throw new ServletException(e);
		} finally {
			logger.debug("[FederationProxyConfigurator.manageServletExecution()] End");
		}
	}
	
	protected abstract void executeServlet(HttpServletRequest request, HttpServletResponse response) throws Exception;
}
