package eu.semiramis.federationproxy.logger;

import java.io.FileOutputStream;
import java.util.Properties;

import javax.servlet.ServletException;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

import eu.semiramis.util.file.FileUtil;
import eu.semiramis.util.properties.PropertiesUtil;

public class LoggerManager {

	protected static Logger logger = null;
	
	private static LoggerManager loggerManagerSingleInstance = null;	
	
	public static void init(LoggerBean loggerBean) throws ServletException{
		FileOutputStream fileOutputStream = null;
		try {
			System.out.println("[LoggerManager.init()] Start");
			String log4jConfigFilePath = loggerBean.getLoggerConfFilePath();
			Properties log4jProperties = null;
			System.out.println("[LoggerManager.init()] \t 1.- Load Configuration file");
			if (log4jProperties == null){
				System.out.println("[LoggerManager.init()] \t\t 1.1.- Try external configuration");
				System.out.println("[LoggerManager.init()] \t\t log4jConfigFilePath: " + log4jConfigFilePath);
				if (FileUtil.isExistentFile(log4jConfigFilePath)){
					log4jProperties = PropertiesUtil.loadAsProperties(log4jConfigFilePath);
					System.out.println("[LoggerManager.init()] \t\t log4jConfigFilePath: " + log4jConfigFilePath + " loaded with success");
				} else {
					System.out.println("[LoggerManager.init()] \t\t log4jConfigFilePath: " + log4jConfigFilePath + " not found");
				}
			}
			if (log4jProperties == null){
				try{
					System.out.println("[LoggerManager.init()] \t\t 1.2.- Try defaultLog4jClassLoader configuration");
					log4jProperties = PropertiesUtil.loadAsProperties(LoggerManager.class.getClassLoader(), "log4j.properties");
					System.out.println("[LoggerManager.init()] \t\t Default classLoader log4j.properties loaded with success");
				} catch (Exception e){
					System.out.println("[LoggerManager.init()] \t\t Unable to load default classLoader log4j.properties");
				}
			}
			if (log4jProperties != null){
				System.out.println("[LoggerManager.init()] \t 2.- Configure");
				System.out.println("[LoggerManager.init()] \t\t log4j.properties: " + log4jProperties);
				PropertyConfigurator.configure(log4jProperties);
				System.out.println("[LoggerManager.init()] \t\t Log4j configured with success");
			}
			logger = Logger.getLogger(LoggerManager.class);
			loggerManagerSingleInstance = new LoggerManager();
		} catch(Exception e){
			e.printStackTrace();
			System.err.println("[LoggerManager.init()] Exception: " + e);
			throw new ServletException(e);
		} finally {
			if (fileOutputStream != null){
				try{
					fileOutputStream.close();
				} catch (Exception uncatched){
				}
			}
			System.out.println("[LoggerManager.init()] End");
		}
	}
	/**
	 * getInstance
	 */
	public static LoggerManager getInstance(){
		return loggerManagerSingleInstance;
	}
	/**
	 * LoggerManager()
	 * @throws Exception
	 */
	private LoggerManager(){
	}

}
