package eu.semiramis.federationproxy.policy;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.Writer;
import java.util.Properties;

import org.apache.log4j.Logger;
import org.opensaml.DefaultBootstrap;

import eu.semiramis.federationproxy.audit.AuditBean;
import eu.semiramis.federationproxy.audit.AuditManager;
import eu.semiramis.policyManager.pdp.ConfigurationException;
import eu.semiramis.policyManager.pdp.ContextHandler;
import eu.semiramis.policyManager.pdp.EvaluationException;
import eu.semiramis.policyManager.pdp.impl.ContextHandlerFactory;
import eu.semiramis.util.file.FileUtil;

import org.opensaml.xacml.ctx.DecisionType;
import org.opensaml.xacml.ctx.RequestType;
import org.opensaml.xacml.ctx.ResponseType;
import org.opensaml.xacml.policy.PolicyType;
import org.opensaml.xml.io.MarshallingException;
import org.opensaml.xml.parse.BasicParserPool;
import org.w3c.dom.Document;
import eu.semiramis.policyManager.util.opensaml.XACMLUtils;
import eu.semiramis.policymodel.policy.*;
import eu.semiramis.policymodel.request.*;
import eu.semiramis.policymodel.response.*;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;

public class PolicyManager {
	
	protected static Logger logger = Logger.getLogger(PolicyManager.class);
	private static PolicyManager policyManagerSingleInstance = null;
	
	private String contextHandlerPath = null;
	private String policyDir = null;
	
	private ContextHandler handler = null;
	
	/**
	 * init
	 * @param federationProxyConfigFilePath
	 * @throws Exception
	 */
	public static void init(PolicyBean policyBean) throws Exception{
		logger.debug("[PolicyManager.init()] Start");
		if (policyManagerSingleInstance == null){
			policyManagerSingleInstance = new PolicyManager(policyBean);
		}
		logger.debug("[PolicyManager.init()] End");
		
	}
	
	/**
	 * getInstance
	 */
	public static PolicyManager getInstance(){
		return policyManagerSingleInstance;
	}
	
	/**
	 * AuditManager()
	 * @throws Exception
	 */
	private PolicyManager(){}
	
	private PolicyManager(PolicyBean policyBean) throws Exception{
		
		String localContextHandlerPath = null;
		String localPolicyDir = null;
		if (policyBean != null){
			localContextHandlerPath = policyBean.getContextHandlerPath();
			localPolicyDir = policyBean.getPolicyDir();
		}
		if (localContextHandlerPath != null){
			this.contextHandlerPath = localContextHandlerPath;
		} else {
			throw new Exception("InvalidContextHandlerPath " + localContextHandlerPath);
		}
		if (FileUtil.isExistentDirectory(localPolicyDir)) {
			this.policyDir = localPolicyDir;
		} else {
			//throw new Exception("InvalidPolicyDir: " + localPolicyDir);
		}	
		
		// Initialize OpenSAML, just in case
		//DefaultBootstrap.bootstrap();
		CreateContextHandler();
	}
	
	private void CreateContextHandler() throws ConfigurationException{
		
		File f = new File(this.contextHandlerPath);
		Properties pro = new Properties();
		if(f.exists()){
		  
		  FileInputStream i = null;
		try {
			i = new FileInputStream(f);
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		  try {
			pro.load(i);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		}
		this.handler = ContextHandlerFactory.createContextHandler(pro);
		
	}
	
	//In principle FP uses only AttributeRelease policies
	public void PAPCreatePolicy(String policyID, String description, String attrProvider, String attrOwner, String attrName, String requester, boolean consentRequired, boolean notificationRequired, String combiningAlgorithm) throws MarshallingException, IOException{
		
		//this creation needs to be updated to specify policyID and obligations
		AttributeReleasePolicy policy = new AttributeReleasePolicy(policyID, description, attrProvider, attrOwner, attrName, requester, consentRequired, notificationRequired, combiningAlgorithm);	    
		
		// translate to XACML policy
		PolicyType attrReleaseXACMLPolicy = policy.toXACMLPolicy();

		// convert to a string
		String attrReleaseXACMLPolicyStr = XACMLUtils.printXMLObject(attrReleaseXACMLPolicy);
		
		//get policy file
		File policyFile = new File(this.policyDir + File.separator + policyID + ".xml");
		
		//save it in policy folder
		Writer policyFileWriter = new BufferedWriter(new FileWriter(policyFile));
		policyFileWriter.write(attrReleaseXACMLPolicyStr);
		policyFileWriter.flush();
		
	}
	
	public RequestType PEPCreateRequest(String attributeName, String requester) {
		
		//Its a request for releasing (requesting) a certain attribute from a federation, 
		//independent of the AttributeProvider or AttributeOwner.
		//The "requester" parameter probably should have no effect on the policy either
		AttributeReleaseRequest request = new AttributeReleaseRequest(requester, "attr-p2", "", attributeName);
		RequestType xacmlRequest = request.toXACMLRequest();
		
		return xacmlRequest;
	}
	
	public boolean PEPHandleResponse(RequestType request) throws EvaluationException{
		
		ResponseType xacmlResponse = handler.handleRequest(request);
		AttributeReleaseResponseHandler attrRelResponseHandler = new AttributeReleaseResponseHandler();
		attrRelResponseHandler.handle(xacmlResponse);
		
		return attrRelResponseHandler.isAuthorized();
	}
	
}
