package eu.semiramis.federationproxy.route;

import java.util.HashMap;

import org.apache.log4j.Logger;

import eu.semiramis.federationproxy.ProxyBean;

public class RouteManager {
	
	protected static Logger logger = Logger.getLogger(RouteManager.class);
	private static RouteManager routeManagerSingleInstance = null;
	
	private HashMap<String, RouteBean> routeBeanMap = null;
	private HashMap<String, ProxyBean> proxyBeanMap = new HashMap<String, ProxyBean>();

	public static final String FP_TYPE = "FP";
	public static final String IA_TYPE = "IA";
	
	/**
	 * init
	 * @param federationProxyConfigFilePath
	 * @throws Exception
	 */
	public static void init(RouteBean[] routeBeanArray) throws Exception{
		logger.debug("[RouteManager.init()] Start");
		if (routeManagerSingleInstance == null){
			routeManagerSingleInstance = new RouteManager(routeBeanArray);
		}
		logger.debug("[RouteManager.init()] End");
	}
	/**
	 * getInstance
	 */
	public static RouteManager getInstance(){
		return routeManagerSingleInstance;
	}
	/**
	 * RouteManager()
	 * @throws Exception
	 */
	private RouteManager(){}
	private RouteManager(RouteBean[] routeBeanArray){
		//InitRouteBeanMap
		this.routeBeanMap = new HashMap<String, RouteBean>();
		if (routeBeanArray != null){
			int routeBeanArrayLength = routeBeanArray.length;
			RouteBean routeBean = null;
			for (int i = 0; i < routeBeanArrayLength; i ++){
				routeBean = routeBeanArray[i];
				this.routeBeanMap.put(routeBean.getRouteKey(), routeBean);
			}
		}
	}

	/**************************************************************/
	/* RouteBeanMapAccesors                                       */
	/**************************************************************/
	public RouteBean getRouteBean(String routeKey){
		RouteBean routeBean = this.routeBeanMap.get(routeKey);
		return routeBean;
	}
	/**************************************************************/
	/* ProxyBeanMapAccesors                                     */
	/**************************************************************/
	public void storeProxyBean(String key, ProxyBean proxyBean){
		proxyBeanMap.put(key, proxyBean);
	}
	public ProxyBean loadProxyBean(String key){
		ProxyBean proxyBean = null;
		if (proxyBeanMap.containsKey(key)){
			proxyBean = proxyBeanMap.get(key);
		}
		return proxyBean;
    }
	public void removeProxyBean(String key){
		if (proxyBeanMap.containsKey(key)){
			proxyBeanMap.remove(key);
		}
	}
	
	public HashMap<String, ProxyBean> getProxyBeanMap() {
		return proxyBeanMap;
	}
	
	
}
