package eu.semiramis.federationproxy.stork;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.opensaml.common.SAMLObject;
import org.opensaml.saml2.core.AuthnRequest;
import org.opensaml.saml2.core.Conditions;
import org.opensaml.saml2.core.StatusCode;
import org.opensaml.xml.signature.SignableXMLObject;

import eu.semiramis.commons.SAMLUtil;
import eu.semiramis.federationproxy.FederationProxyServlet;
import eu.semiramis.federationproxy.ProxyBean;
import eu.semiramis.federationproxy.saml.SamlManager;

public class FederationProxyStorkServlet extends FederationProxyServlet {

	/**
	 * executeProxy
	 * @param proxyBean
	 * @return
	 */
	private ProxyBean executeProxy(HttpServletRequest request, HttpServletResponse response, ProxyBean proxyBean) throws Exception{
		try {
			logger.debug("[FederationProxyStorkServlet.executeProxy()] Start");
			logger.debug("[FederationProxyStorkServlet.executeProxy()] \t ValidateSourceSign");
			if (proxyBean.getProxyValidateSignFlag()) {
				SignableXMLObject signableXmlObject = (SignableXMLObject)proxyBean.getSourceSamlObject();
				SAMLUtil.validateSign(signableXmlObject, proxyBean.getProxyValidateSignCertFilePath());
				boolean validationAchievedFlag = true;
				logger.debug("[FederationProxyStorkServlet.executeProxy()] \t\t validationAchievedFlag: " + validationAchievedFlag);
				proxyBean.setProxyValidateSignAchievedFlag(true);
				if (!validationAchievedFlag){
					throw new NotTrustAchievedException();
				}
			}
			logger.debug("[FederationProxyStorkServlet.executeProxy()] \t ValidateSourceResponseStatusCode");
			String sourceResponseStatusCodeValue = federationProxySamlManager.extractSamlResponseStatusCode(proxyBean.getSourceSamlObject());
			boolean sourceSamlResponseStatusSuccessFlag = true;			
			logger.debug("[FederationProxyStorkServlet.executeProxy()] \t\t sourceResponseStatusCodeValue: " + sourceResponseStatusCodeValue);
			if (sourceResponseStatusCodeValue != null){
				if (sourceResponseStatusCodeValue.equals(StatusCode.SUCCESS_URI)){
					sourceSamlResponseStatusSuccessFlag = true;
				} else {
					sourceSamlResponseStatusSuccessFlag = false;
				}
			}
			logger.debug("[FederationProxyStorkServlet.executeProxy()] \t\t sourceSamlResponseStatusSuccessFlag: " + sourceSamlResponseStatusSuccessFlag);
			proxyBean.setProxySamlResponseStatusCodeSuccessFlag(sourceSamlResponseStatusSuccessFlag);
			if (!sourceSamlResponseStatusSuccessFlag){
				throw new UnSuccessStatusCodeException();
			}
			logger.debug("[FederationProxyStorkServlet.executeProxy()] \t Process Request-Response");
			logger.debug("[FederationProxyStorkServlet.executeProxy()] \t\t sourceSamlObjectType: " + federationProxySamlManager.getSamlObjectType(proxyBean.getSourceSamlObject()));
			switch (federationProxySamlManager.getSamlObjectType(proxyBean.getSourceSamlObject())){
				case SamlManager.SAMLTYPE_AUTHNREQUEST:
					proxyBean = processAuthenticationRequest(request, response, proxyBean);
					break;
				case SamlManager.SAMLTYPE_ATTRIBUTEQUERY:
				case SamlManager.SAMLTYPE_AUTHDECISIONQUERY:
				case SamlManager.SAMLTYPE_RESPONSEIMPL:
					//TODO Estas tres tenfran que provocar una excepcion			
					break;
				default:
					//StorkResponsecase

					break;
			}
			logger.debug("[FederationProxyStorkServlet.executeProxy()] \t SignTargetSamlObject");
			SAMLObject targetSamlObject = proxyBean.getTargetSamlObject();	        
			boolean trustSignFlag = proxyBean.getProxySignFlag();
	        logger.debug("[FederationProxyStorkServlet.executeProxy()] \t\t trustSignFlag: " + trustSignFlag);
            if(trustSignFlag){
                federationProxySamlManager.signSamlObject((SignableXMLObject)targetSamlObject, federationProxyTrustManager.getTrustPkcsPublicKey(), federationProxyTrustManager.getTrustPkcsPrivateKey());
                logger.debug("[FederationProxyStorkServlet.executeProxy()] \t\t\t SamlObjectSigned");
				proxyBean.setProxySignAchievedFlag(true);
            }
			logger.debug("[FederationProxyStorkServlet.executeProxy()] \t ManageDataManager");
			switch (federationProxySamlManager.getSamlObjectType(proxyBean.getSourceSamlObject())){
				case SamlManager.SAMLTYPE_AUTHNREQUEST:
				case SamlManager.SAMLTYPE_ATTRIBUTEQUERY:
				case SamlManager.SAMLTYPE_AUTHDECISIONQUERY:
					logger.debug("[FederationProxyStorkServlet.executeProxy()] \t\t InitFederationProxyStep");
					federationProxyRouteManager.storeProxyBean(federationProxySamlManager.extractSamlObjectId(proxyBean.getTargetSamlObject()), proxyBean);
					logger.debug("[FederationProxyStorkServlet.executeProxy()] \t\t\t proxyBean STORED");
					break;
				case SamlManager.SAMLTYPE_RESPONSEIMPL:
					logger.debug("[FederationProxyStorkServlet.executeProxy()] \t\t CompleteFederationProxyStep");
					String proxyBeanKey = federationProxySamlManager.extractSamlInResponseTo(proxyBean.getSourceSamlObject());
					//TODO We must think about this
					//Audit
					ProxyBean requestProxyBean = federationProxyRouteManager.loadProxyBean(proxyBeanKey);
					ProxyBean responseProxyBean = proxyBean;
					String auditId = federationProxySamlManager.extractSamlObjectId(requestProxyBean.getSourceSamlObject());
					federationProxyAuditManager.audit(auditId, requestProxyBean, responseProxyBean);
					//RemoveCurrentProxyBean from ProxyBeanRouteTable
					federationProxyRouteManager.removeProxyBean(proxyBeanKey);
					logger.debug("[FederationProxyStorkServlet.executeProxy()] \t\t\t proxyBean REMOVED");
					break;
				default: 
					throw new UnexpectedSourceSamlObjectTypeException();
			}
			//logger.debug("[FederationProxyStorkServlet.executeProxy()] \t\t\t proxyBean: " + proxyBean);
			logger.debug("[FederationProxyStorkServlet.executeProxy()] \t SendTargetSamlObject");
			String targetUrl = proxyBean.getProxyUrl();
			String targetRelayState = proxyBean.getProxyRelayState();			
			logger.debug("[FederationProxyStorkServlet.executeProxy()] \t\t targetUrl: " + targetUrl);
			logger.debug("[FederationProxyStorkServlet.executeProxy()] \t\t targetRelayState: " + targetRelayState);
			FederationProxyServlet.sendRedirectSamlObject(response, targetUrl, targetSamlObject, targetRelayState);
			return proxyBean;
		} finally {
			logger.debug("[FederationProxyStorkServlet.executeProxy()] End");			
		}
	}

	
	/**
	 * processAuthenticationRequest
	 * @param request
	 * @param response
	 * @param proxyBean
	 * @return
	 * @throws Exception
	 */
	private ProxyBean processAuthenticationRequest(HttpServletRequest request, HttpServletResponse response, ProxyBean proxyBean) throws Exception {
		try {
			logger.debug("[FederationProxyStorkServlet.processAuthenticationRequest()] Start");
			String targetUrl = proxyBean.getProxyUrl();
			String targetRelayState = proxyBean.getProxyRelayState();
			String targetSamlIssuer = proxyBean.getProxySamlIssuer();
			String targetSamlIdpProvider = proxyBean.getProxySamlIdpProvider();
			Conditions targetSamlConditions = proxyBean.getProxySamlConditions();
			logger.debug("[FederationProxyStorkServlet.processAuthenticationRequest()] \t GenerateAuthnRequest");
			logger.debug("[FederationProxyStorkServlet.processAuthenticationRequest()] \t\t targetUrl: " + targetUrl);
			logger.debug("[FederationProxyStorkServlet.processAuthenticationRequest()] \t\t targetRelayState: " + targetRelayState);
			logger.debug("[FederationProxyStorkServlet.processAuthenticationRequest()] \t\t targetSamlIssuer: " + targetSamlIssuer);
	        AuthnRequest authnRequest = SAMLUtil.createAuthNRequest(targetSamlIssuer, targetUrl, targetRelayState);
	        logger.debug("[FederationProxyStorkServlet.processAuthenticationRequest()] \t InsertSamlIdpProviderId");
	        logger.debug("[FederationProxyStorkServlet.processAuthenticationRequest()] \t\t targetSamlIdpProvider: " + targetSamlIdpProvider);
	        federationProxySamlManager.insertSamlIdpProviderId(authnRequest, targetSamlIdpProvider);
	        logger.debug("[FederationProxyStorkServlet.processAuthenticationRequest()] \t InsertSamlConditions");
	        logger.debug("[FederationProxyStorkServlet.processAuthenticationRequest()] \t\t targetSamlOneTimeUseFlag: " + federationProxySamlManager.extractSamlOneTimeUseFlag(targetSamlConditions));
	        logger.debug("[FederationProxyStorkServlet.processAuthenticationRequest()] \t\t targetSamlNotOnAfter: " + federationProxySamlManager.extractSamlNotOnAfter(targetSamlConditions));
	        logger.debug("[FederationProxyStorkServlet.processAuthenticationRequest()] \t\t targetSamlNotBefore: " + federationProxySamlManager.extractSamlNotBefore(targetSamlConditions));
	        logger.debug("[FederationProxyStorkServlet.processAuthenticationRequest()] \t\t targetSamlAudienceRestrictionIdList: " + federationProxySamlManager.extractSamlAudienceRestrictionIdList(targetSamlConditions));
	        federationProxySamlManager.insertSamlConditions(authnRequest, targetSamlConditions);
            //saveTargetSamlObject
            proxyBean.setTargetSamlObject(authnRequest);                
			return proxyBean;
		} finally {
			logger.debug("[FederationProxyStorkServlet.processAuthenticationRequest()] End");			
		}
	}

}
