package eu.semiramis.federationproxy.translation;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.Properties;

import org.apache.commons.codec.binary.Base64;
import org.apache.log4j.Logger;

import eu.semiramis.attribute.AttributeBean;

public class TranslationManager {

	protected static Logger logger = Logger.getLogger(TranslationManager.class);
	private static TranslationManager translationManagerSingleInstance = null;
	private boolean translationEnabledFlag = false;
	private eu.semiramis.federationproxy.translation.TranslationBean translationBean = null;
	
	Connection conn = null;
	Statement s = null;
	
	public boolean getTranslationEnabledFlag() {
		return translationEnabledFlag;
	}

	/**
	 * init
	 * @param federationProxyConfigFilePath
	 * @throws Exception
	 */
	public static void init(eu.semiramis.federationproxy.translation.TranslationBean translationBean, boolean translationEnabledFlag) throws Exception{
		logger.debug("[TranslationManager.init()] Start");
		if (translationManagerSingleInstance == null){
			translationManagerSingleInstance = new TranslationManager(translationBean, translationEnabledFlag);
		}
		
		logger.debug("[TranslationManager.init()] End");
	}
	/**
	 * getInstance
	 */
	public static TranslationManager getInstance(){
		return translationManagerSingleInstance;
	}
	/**
	 * TranslationManager()
	 * @throws Exception
	 */
	private TranslationManager(){}
	
	private TranslationManager(eu.semiramis.federationproxy.translation.TranslationBean translationBean, boolean translationEnabledFlag) throws Exception{
		//InitTranslationEnabledFlag
		this.translationBean = translationBean;
		this.translationEnabledFlag = translationEnabledFlag;
		createConnection();
		dePopulateDB();
		populateDB();	
	}
	
	public class AttributeNotFoundException extends Exception {

	    public AttributeNotFoundException(String message) {
	        super(message);
	    }

	    public AttributeNotFoundException(String message, Throwable throwable) {
	        super(message, throwable);
	    }

	}
	
	public eu.semiramis.federationproxy.translation.TranslationBean getTranslationBean() {
		return translationBean;
	}

	public void setTranslationBean(
			eu.semiramis.federationproxy.translation.TranslationBean translationBean) {
		this.translationBean = translationBean;
	}

	public String translateLabelIntoEnglish(String name) throws AttributeNotFoundException{
		
		ResultSet rs = callQuery("select", "SELECT English FROM " + translationBean.getProxyTransMappingTableName() + " WHERE FP_Language='" + name + "'", null);				
		try {
			if (rs.next()){
				return rs.getString(1);
			} 
			else {
				if (rs != null){
					
					logger.error("------------------------------------------------------------------------");
					logger.error("The following attribute could not be found in the translation table: " + name);		
					logger.error("------------------------------------------------------------------------");
					throw new AttributeNotFoundException("The following attribute could not be found in the translation table: " + name);
				}
			}
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}
	
	
	
	public String translateLabelIntoFPLanguage(String name) throws AttributeNotFoundException {
		
		ResultSet rs = callQuery("select", "SELECT FP_Language FROM " + translationBean.getProxyTransMappingTableName() + " WHERE English='" + name + "'", null);				
		//ResultSet rs = callQuery("select", "SELECT FP_Language FROM " + "ttt" + " WHERE English='" + name + "'", null);				
		
		try {
			if (rs.next()){
				return rs.getString(1);
			} 
			else {
				if (rs != null){
					logger.error("------------------------------------------------------------------------");
					logger.error("The following attribute could not be found in the translation table: " + name);
					logger.error("------------------------------------------------------------------------");
					throw new AttributeNotFoundException("The following attribute could not be found in the translation table: " + name);
				}
			}
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			
		}
		return null;
	}
	
	public String translateFriendlyName (String sourceFriendlyName, boolean intoEnglish) throws AttributeNotFoundException {
		String targetFriendlyName = "";
		String[] friendlyNameItems = sourceFriendlyName.split(";", -1);
    	//List<String> structure = new ArrayList<String>();
    	for(String friendlyNameItem : friendlyNameItems) {
    		byte[] bytefriendlyNameItem = friendlyNameItem.getBytes();
            String decodedValue = new String(Base64.decodeBase64(bytefriendlyNameItem));            
            if (intoEnglish){
            	 targetFriendlyName += new String(Base64.encodeBase64(translateLabelIntoEnglish(decodedValue).getBytes())) + ";";
            } else {
            	targetFriendlyName += new String(Base64.encodeBase64(translateLabelIntoFPLanguage(decodedValue).getBytes())) + ";";
            }
        }
        targetFriendlyName = targetFriendlyName.substring(0, targetFriendlyName.lastIndexOf(";")); //removes last ";"
		return targetFriendlyName;
	}
	
	public Connection getConn() {
		return conn;
	}

	public void setConn(Connection conn) {
		this.conn = conn;
	}

	public Statement getS() {
		return s;
	}

	public void setS(Statement s) {
		this.s = s;
	}

	public int createConnection(){	
		
		/*
		//Utils u = new Utils();
		Properties props = Utils.loadPropertyFile();
		String DDBBPath = Utils.getProperties(props, "DDBB_Path");
		String DDBBuser = Utils.getProperties(props, "DDBB_user");
		String DDBBpass = Utils.getProperties(props, "DDBB_password");
		Utils.savePropertyFile(props);*/
	
		try {
			  Class.forName("com.mysql.jdbc.Driver").newInstance();
			  
			  conn = DriverManager.getConnection ("jdbc:mysql:" + translationBean.getProxyDBPath(),
					  translationBean.getProxyDBUser(), translationBean.getProxyDBPass());
			  		////192.168.10.59:3306/example","root","root");		 
			  
			  return 1;
        } 

      catch( Exception e ){
     	 e.printStackTrace();
     	 return 0;
     	 }	
		
	}
	
	public ResultSet callQuery(String queryType, String q, Statement s){
		ResultSet rs = null; 
		try{ 
     	if (s == null){
     		this.s = conn.createStatement(); 
     	}else{
     		this.s = s;
     	}
        if (q == null){
        	 PreparedStatement ps = (PreparedStatement)this.s;
        	 if ((queryType=="insert") || (queryType=="delete")) {
        		 ps.execute();
        	 }else if (queryType=="select"){
        		 rs = ps.executeQuery();
        	 }
        	 
        } else {
        	if ((queryType=="insert") || (queryType=="delete"))  {
        		this.s.execute(q);
       	 }else if (queryType=="select"){
       		rs = this.s.executeQuery(q);
       	 }
        }
        
        } //try

      catch( Exception e ){
     	 e.printStackTrace();    	 
     	 }		
      
      return rs;
	}
	
	public int closeConnection(){
		try { 
			s.close();
			conn.close();
			return 1;
        } 
      catch( Exception e ){
     	 e.printStackTrace();
     	 return 0;
     	 }		
	}
	
	public void populateDB(){
				
		//populates the DB from properties file
		File f = new File(translationBean.getProxyTranslationMappingPath());
		if(f.exists()){
		  Properties pro = new Properties();
		  FileInputStream in = null;
		try {
			in = new FileInputStream(f);
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		  try {
			pro.load(in);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		  String key_EN = null;
		  String value_FPLanguage = null;
		  Enumeration<Object> keys = pro.keys();
		  Enumeration<Object> values = pro.elements();
		  while (keys.hasMoreElements()){
			  key_EN = (String) keys.nextElement();
			  value_FPLanguage = (String) values.nextElement();
			  callQuery("insert", "INSERT INTO " + translationBean.getProxyTransMappingTableName() + " VALUES ('"+ key_EN + "', '"+ value_FPLanguage + "')", null);			
				
		  }
		 
		}
	}
	
	public void dePopulateDB(){
		
		callQuery("delete", "DELETE FROM " + translationBean.getProxyTransMappingTableName(), null);
		
	}

	
}
