package eu.semiramis.federationproxy.translation_old;

import java.util.Enumeration;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.ResourceBundle;

import org.apache.log4j.Logger;

import eu.semiramis.attribute.AttributeBean;


public class TranslationManager {
	
	
	protected static Logger logger = Logger.getLogger(TranslationManager.class);
	private HashMap<String, TranslationBean> translationBeanMap			= null;
	private static TranslationManager translationManagerSingleInstance = null;
	private boolean translationEnabledFlag = false;
	
	private Map<String, Map> resourceBundleLocaleLookupMap		= new HashMap<String, Map>();
	public TranslationBean getTranslationBean(String translationKey){
		TranslationBean translationBean = this.translationBeanMap.get(translationKey);
		return translationBean;
	}

	public boolean getTranslationEnabledFlag() {
		return translationEnabledFlag;
	}

	/**
	 * init
	 * @param federationProxyConfigFilePath
	 * @throws Exception
	 */
	public static void init(TranslationBean[] translationBeanArray, boolean translationEnabledFlag) throws Exception{
		logger.debug("[TranslationManager.init()] Start");
		if (translationManagerSingleInstance == null){
			translationManagerSingleInstance = new TranslationManager(translationBeanArray, translationEnabledFlag);
		}
		logger.debug("[TranslationManager.init()] End");
	}
	/**
	 * getInstance
	 */
	public static TranslationManager getInstance(){
		return translationManagerSingleInstance;
	}
	/**
	 * TranslationManager()
	 * @throws Exception
	 */
	private TranslationManager(){}
	/**
	 * TranslationManager
	 * @throws Exception
	 */
	private TranslationManager(TranslationBean[] translationBeanArray, boolean translationEnabledFlag) throws Exception{
		//InitTranslationEnabledFlag
		this.translationEnabledFlag = translationEnabledFlag;
		//InitTranslationBeanMap
		this.translationBeanMap = new HashMap<String, TranslationBean>();
		if (translationBeanArray != null){
			int translationBeanArrayLength = translationBeanArray.length;
			TranslationBean translationBean = null;
			for (int i = 0; i < translationBeanArrayLength; i ++){
				translationBean = translationBeanArray[i];
				this.translationBeanMap.put(translationBean.getTranslationKey(), translationBean);
			}
		}
	}

	
	/**************************************************************/
	/* ResourceBundleAccesors                                     */
	/**************************************************************/
	public static String translateLabel(String fromResourceBundleBaseName, Locale fromLocale, String toResourceBundleBaseName, Locale toLocale, String fromLabel){
		TranslationManager translationManagerInstance = TranslationManager.getInstance();
		String toLabel = translationManagerInstance.translateString(fromResourceBundleBaseName, fromLocale, toResourceBundleBaseName, toLocale, fromLabel);
		return toLabel;
	}
	public static String translateStream(String fromResourceBundleBaseName, Locale fromLocale, String toResourceBundleBaseName, Locale toLocale, String fromStream) throws Exception{
		String toStream = fromStream;
		if (fromStream != null){
			AttributeBean fromAttributeBean = AttributeBean.getInstance(fromStream);
			if (fromAttributeBean != null){
				AttributeBean toAttributeBean = fromAttributeBean.translate(fromResourceBundleBaseName, fromLocale, toResourceBundleBaseName, toLocale);
				if (toAttributeBean != null){
					toStream = toAttributeBean.toXml();
				}
			} else {
				toStream = translateLabel(fromResourceBundleBaseName, fromLocale, toResourceBundleBaseName, toLocale, fromStream);
			}
		}
		return toStream;
	}
	private String translateString(String fromResourceBundleBaseName, Locale fromLocale, String toResourceBundleBaseName, Locale toLocale, String fromString){
		String toString = fromString;		
		if (translationEnabledFlag){
			String resourceBundleKey = null;
			String resourceBundleValue = null;
			if (fromString != null){
				resourceBundleKey = getResourceBundleKey(fromResourceBundleBaseName, fromLocale, fromString);
				if (resourceBundleKey != null){
					resourceBundleValue = getResourceBundleValue(toResourceBundleBaseName, toLocale, resourceBundleKey);	
				}
			}
			if (resourceBundleValue != null){
				toString = resourceBundleValue;
			}
		}
		return toString;
	}
	
	private String getResourceBundleValue(String resourceBundleBaseName, Locale resourceBundleLocale, String resourceBundleKey){
		String resourceBundleValue = null;
		ResourceBundle resourceBundle = ResourceBundle.getBundle(resourceBundleBaseName, resourceBundleLocale);
		resourceBundleValue = resourceBundle.getString(resourceBundleKey);
		return resourceBundleValue;
	}
	private String getResourceBundleKey(String resourceBundleBaseName, Locale resourceBundleLocale, String resourceBundleValue){
		String resourceBundleKey = null;
		Map<String, String> lookupMap = null;
		synchronized(this.resourceBundleLocaleLookupMap) {
			Map<Locale, Map> localeLookupMap = (Map<Locale, Map>) this.resourceBundleLocaleLookupMap.get(resourceBundleBaseName);
			if (localeLookupMap == null) {
				ResourceBundle resourceBundle = ResourceBundle.getBundle(resourceBundleBaseName, resourceBundleLocale);
				localeLookupMap = getLocaleLookupMapInstance(resourceBundle);
				this.resourceBundleLocaleLookupMap.put(resourceBundleBaseName, localeLookupMap);
			}
			lookupMap = (Map<String, String>) localeLookupMap.get(resourceBundleLocale);
			if (lookupMap == null) {
				ResourceBundle resourceBundle = ResourceBundle.getBundle(resourceBundleBaseName, resourceBundleLocale);
				lookupMap = getLookupMapInstance(resourceBundle);
				localeLookupMap.put(resourceBundleLocale, lookupMap);
				this.resourceBundleLocaleLookupMap.put(resourceBundleBaseName, localeLookupMap);
			}
		}
		resourceBundleKey = (String) lookupMap.get(resourceBundleValue);
		return resourceBundleKey;
	}
	private Map<Locale, Map> getLocaleLookupMapInstance(ResourceBundle resourceBundle) {
		Map<Locale, Map> localeLookupMap = new HashMap<Locale, Map>();
		Locale localeLookupMapKey = resourceBundle.getLocale();
		Map<String, String> localeLookupMapValue = getLookupMapInstance(resourceBundle);
		localeLookupMap.put(localeLookupMapKey, localeLookupMapValue);
		return localeLookupMap;
	}
	private static Map<String, String> getLookupMapInstance(ResourceBundle resourceBundle) {
		Map<String, String> lookupMap = new HashMap<String, String>();
		Enumeration<String> keysEnumeration = resourceBundle.getKeys();
		while (keysEnumeration.hasMoreElements()){
			String resourceBundleKey = keysEnumeration.nextElement();
			String resourceBundleValue = resourceBundle.getString(resourceBundleKey);
			// Found key and haven't added to Map yet, so add the text
			if ((resourceBundleValue != null) && !lookupMap.containsKey(resourceBundleValue)) {
				lookupMap.put(resourceBundleValue, resourceBundleKey);
			}
		}
		return lookupMap;
	}


}
