package eu.semiramis.federationproxy.trust;

import java.io.FileInputStream;
import java.security.KeyStore;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.util.HashMap;

import org.apache.log4j.Logger;


public class TrustManager {
	
	//private static final String INSTANCE_PKCS12 = "JKS";
	protected static Logger logger = Logger.getLogger(TrustManager.class);
	private static TrustManager trustManagerSingleInstance = null;
	private HashMap<String, TrustBean> trustBeanMap = null;
	private PrivateKey trustPkcsPrivateKey = null;
	private PublicKey trustPkcsPublicKey = null;
	public static final String FP_TYPE = "FP";
	public static final String IA_TYPE = "IA";
	
	/**
	 * init
	 * @param federationProxyConfigFilePath
	 * @throws Exception
	 */
	public static void init(TrustBean[] trustBeanArray, PkcsBean pkcsBean) throws Exception{
		logger.debug("[TrustManager.init()] Start");
		if (trustManagerSingleInstance == null){
			trustManagerSingleInstance = new TrustManager(trustBeanArray, pkcsBean);
		}
		logger.debug("[TrustManager.init()] End");
	}
	
	
	/**
	 * getInstance
	 */
	public static TrustManager getInstance(){
		return trustManagerSingleInstance;
	}
	/**
	 * TrustManager()
	 * @throws Exception
	 */
	private TrustManager(){}
	private TrustManager(TrustBean[] trustBeanArray, PkcsBean pkcsBean) throws Exception{
		//InitTrustBeanMap
		this.trustBeanMap = new HashMap<String, TrustBean>();
		if (trustBeanArray != null){
			int trustBeanArrayLength = trustBeanArray.length;
			TrustBean trustBean = null;
			for (int i = 0; i < trustBeanArrayLength; i ++){
				trustBean = trustBeanArray[i];
				this.trustBeanMap.put(trustBean.getTrustKey(), trustBean);
			}
		}
		String keyInstance = pkcsBean.getPkcsCertFilePath().substring(pkcsBean.getPkcsCertFilePath().lastIndexOf(".") + 1);
		keyInstance = keyInstance.equals("jks") ? "jks" : "pkcs12";
		//InitPkcsKeys
		KeyStore keyStore = KeyStore.getInstance(keyInstance);
		keyStore.load(new FileInputStream(pkcsBean.getPkcsCertFilePath()), pkcsBean.getPkcsCertFilePassword().toCharArray());
		this.trustPkcsPrivateKey = (PrivateKey) keyStore.getKey(pkcsBean.getPkcsCertFilePrivateKeyAlias(), pkcsBean.getPkcsPrivateKeyPassword().toCharArray());
		this.trustPkcsPublicKey = keyStore.getCertificate(pkcsBean.getPkcsCertFilePrivateKeyAlias()).getPublicKey();
		if (this.trustPkcsPrivateKey == null || this.trustPkcsPublicKey == null){
			throw new Exception("PkcsInitException");
		}
	}
	
	
	
	public TrustBean getTrustBean(String trustKey){
		TrustBean trustBean = this.trustBeanMap.get(trustKey);
		return trustBean;
	}

	public PrivateKey getTrustPkcsPrivateKey() {
		return this.trustPkcsPrivateKey;
	}

	public PublicKey getTrustPkcsPublicKey() {
		return this.trustPkcsPublicKey;
	}

	/*
	public static void main(String[] args){
		String pkcsCertFilePath1 = "D:/projects/projectSemiramis/eaidProject/eaidConfig/UMU-FP/certs/UMU-FP.p12";
		String pkcsCertFilePath2 = "D:/projects/projectSemiramis/eaidProject/eaidConfig/USTUTT-FP/certs/USTUTT-FP.p12";
		String pkcsCertFilePassword = "Semir@mis1";
		String pkcsCertFilePrivateKeyAlias = "1";
		String pkcsPrivateKeyPassword = "Semir@mis1";
		try {
			System.out.println("Init: " + pkcsCertFilePath2);
			PkcsBean pkcsBean2 = new PkcsBean(pkcsCertFilePath2, pkcsCertFilePassword, pkcsCertFilePrivateKeyAlias, pkcsPrivateKeyPassword);
			System.out.println("pkcsBean2 SUCCESS");
			System.out.println("Init: " + pkcsCertFilePath1);
			PkcsBean pkcsBean1 = new PkcsBean(pkcsCertFilePath1, pkcsCertFilePassword, pkcsCertFilePrivateKeyAlias, pkcsPrivateKeyPassword);
			System.out.println("pkcsBean1 SUCCESS");
		} catch (Exception e){
			e.printStackTrace();
		}
	}
	*/



}
