package eu.semiramis.util.encode;

import java.io.IOException;

public class EncodeUtil {

	public static final String ENCODING_ISO88591					= "ISO-8859-1";
	public static final String ENCODING_ISO885915					= "ISO-8859-15";//Codificacion buena para carateres espaoles
	public static final String ENCODING_UNICODELITTLE				= "UnicodeLittle";
	public static final String ENCODING_UTF8 						= "UTF-8";
	//Funcionan bien con los XML
	public static final String ENCODING_SHIFT_JIS					= "Shift_JIS";//Codificacion buena para caracteres japoneses... Cuidado con el guion... 
	public static final String ENCODING_KOI8_R						= "KOI8-R";//Codificacion buena para caracteres rusos
	
	
	/**
	 * encode source from between differnet encodings
	 * @param 
	 * @return
	 */
	public static String encode(String original, String fromEncoding, String toEncoding) throws Exception{
		String encoded = null;		
		if (original != null){
			if (fromEncoding != null && toEncoding != null){
				encoded = new String(original.getBytes(fromEncoding), toEncoding);
			} else if (fromEncoding == null && toEncoding != null){
				encoded = new String(original.getBytes(), toEncoding);
			} else if (fromEncoding != null && toEncoding == null){
				encoded = new String(original.getBytes(fromEncoding));
			} else if (fromEncoding == null && toEncoding == null){
				encoded = new String(original.getBytes());
			}
		}
		return encoded;
	}
	
	
	
	/***************************************************************************************************/
	/* AsciiEncode.- Translates Strings from native to ascii code and vice versa.                      */
	/***************************************************************************************************/
	/**
	 * main
	 * @param args
	 * @throws IOException
	 */
	/*
	public static void main(String[] args) throws IOException {
		System.out.println("***************************************************");
		System.out.println("[EncodeUtil.main()] Start");
		System.out.println("***************************************************");
		if (args.length == 0) {
			System.out.println("[EncodeUtil.main()] Usage: com.vln.util.encode.EncodeUtil [-reverse] filepath [encoding]");
			return;
		}
		//Get reverseFlag, filaPath and Encoding
		String filePath = args[0];
		boolean reverse = false;
		if ("-reverse".equals(filePath)) {
			reverse = true;
			if (args.length == 1) {
				System.out.println("[EncodeUtil.main()] Usage: com.vln.util.encode.AsciiEncodeUtili [-reverse] filepath [encoding]");
				return;
			}
			filePath = args[1];
		}
		File file = new File(filePath);
		String encoding = null;
		if ( (args.length > 1 && !reverse) || args.length > 2 ) {
			if (reverse) {
				encoding = args[2];
			} else {
				encoding = args[1];
			}
		}
		System.out.println("[EncodeUtil.main()] Args:");
		System.out.println("[EncodeUtil.main()] \t filePath: " + filePath);
		System.out.println("[EncodeUtil.main()] \t reverse: " + reverse);
		System.out.println("[EncodeUtil.main()] \t encoding: " + encoding);
		//Convert
		//InputStream in = translateToAscii( new FileInputStream( file ), encoding );
		BufferedReader reader = encoding == null ?
				new BufferedReader(new InputStreamReader(new FileInputStream( file ))) :
				new BufferedReader(new InputStreamReader(new FileInputStream( file ), encoding));
		String line;
		//int i = 1;
		while ((line = reader.readLine()) != null) {
			System.out.println("[EncodeUtil.main()] readedLine: " + line);
			//System.out.print( i + ": [");
			if (reverse) {
				System.out.println("[EncodeUtil.main()] \t decodedLine: " + asciiDecode(line));				
			} else {
				System.out.println("[EncodeUtil.main()] \t encodedLine: " +  asciiEncode(line));
				System.out.println("[EncodeUtil.main()] \t exceptionEncodedLine: " +  asciiEncode(line, EncodeUtil.ASCII_EXCEPTION_CHARSTREAM, false));
			}
			//System.out.println("]");
			//i++;
		}
		//in.close();
		System.out.println("***************************************************");
		System.out.println("[EncodeUtil.main()] End");
		System.out.println("***************************************************");
	}
	*/
	public static void main(String[] args) throws IOException {
		System.out.println("***************************************************");
		System.out.println("[EncodeUtil.main()] Start");
		System.out.println("***************************************************");
		try {
			String input = "matire";
			String asciiEncoded = asciiEncode(input);
			System.out.println("asciiEncoded: " + asciiEncoded);
			String utf8 = encode(input, EncodeUtil.ENCODING_UTF8, null);
			System.out.println("utf8: " + asciiEncode(utf8));
			utf8 = encode(input, null, EncodeUtil.ENCODING_UTF8);
			System.out.println("utf8: " + asciiEncode(utf8));
		} catch (Exception e){
			
		}
		System.out.println("***************************************************");
		System.out.println("[EncodeUtil.main()] End");
		System.out.println("***************************************************");
	}
	
	/** A table of hex digits */
	public static final String ASCII_EXCEPTION_CHARSTREAM = "\\=:\t\r\n\f#!";	
	/**
	 * encodeAndEscape
	 * Converts native to encoded &#92;uxxxx
	 * and writes out any of the characters in exceptionChars
	 * with a preceding slash
	 */
	public static String asciiEncode(String theString, String exceptionCharStream, boolean escapeSpace){
		int len = theString.length();
		StringBuffer outBuffer = new StringBuffer(len*2);
		for (int x=0; x<len; x++) {
			char aChar = theString.charAt(x);
			switch (aChar) {
				case ' ':
					if (x == 0 || escapeSpace){
						outBuffer.append('\\');
					}
					outBuffer.append(' ');
					break;
				default:
					if ((aChar < 0x0020) || (aChar > 0x007e)) {
						outBuffer.append('\\');
						outBuffer.append('u');
						outBuffer.append(toHex((aChar >> 12) & 0xF));
						outBuffer.append(toHex((aChar >>  8) & 0xF));
						outBuffer.append(toHex((aChar >>  4) & 0xF));
						outBuffer.append(toHex( aChar        & 0xF));
					} else {
						if (exceptionCharStream.indexOf(aChar) != -1){
							outBuffer.append('\\');
						}
						outBuffer.append(aChar);
					}
					break;
			}
		}
		return outBuffer.toString();
	}
	
	
	
	/**
	 * Translates the given String into ASCII code.
	 * @param input the input which contains native characters like umlauts etc
	 * @return the input in which native characters are replaced through ASCII code
	 */
	public static String asciiDecode(String input) {
		if (input == null) {
			return null;
		}
		StringBuffer buffer = new StringBuffer( input.length() );
		boolean precedingBackslash = false;
		for (int i = 0; i < input.length(); i++) {
			char c = input.charAt(i);
			if (precedingBackslash) {
				switch (c) {
					case 'f': c = '\f'; break;
					case 'n': c = '\n'; break;
					case 'r': c = '\r'; break;
					case 't': c = '\t'; break;
					case 'u':
						String hex = input.substring( i + 1, i + 5 );
						c = (char) Integer.parseInt(hex, 16 );
						i += 4;
				}
				precedingBackslash = false;
			} else {
				precedingBackslash = (c == '\\');
			}
			if (!precedingBackslash) {
				buffer.append(c);
			}
		}
		return buffer.toString();
	}
	/**
	 * Translates the given String into ASCII code.
	 * @param input the input which contains native characters like umlauts etc
	 * @return the input in which native characters are replaced through ASCII code
	 */
	public static String asciiEncode( String input ) {
		if (input == null) {
			return null;
		}
		StringBuffer buffer = new StringBuffer( input.length() + 60 );
		for (int i = 0; i < input.length(); i++) {
			char c = input.charAt(i);
			if (c <= 0x7E) {
				buffer.append(c);
			} else {
				buffer.append("\\u");
				String hex = Integer.toHexString(c);
				for (int j = hex.length(); j < 4; j++ ) {
					buffer.append( '0' );
				}
				buffer.append( hex );
			}
		}
		return buffer.toString();
	}
	private static final char[] hexDigit = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};	
	/**
	 * Convert a nibble to a hex character
	 * @param   nibble  the nibble to convert.
	 */
	private static char toHex(int nibble) {
		return hexDigit[(nibble & 0xF)];
	}
//	/**
//	 * Translates the given input stream to ascii code
//	 * @param in the input stream
//	 * @param encoding the encoding, when null the default encoding is used
//	 * @return the input stream that contains the ascii representation of the given input stream
//	 * @throws IOException when the native2ascii tool could not be found.
//	 */
//	public static InputStream translateToAscii( InputStream in, String encoding ) throws IOException {
//		String javaHomePath = System.getProperty("java.home");
//		File javaHome = new File( javaHomePath );
//		if (!javaHome.exists()) {
//			throw new IOException("The system property java.home points to the invalid destination [" + javaHome.getAbsolutePath() + "].");
//		}
//		File executable;
//		if ( File.separatorChar == '\\') { // windows
//			executable = new File( javaHome, "bin\\native2ascii.exe");
//		} else {
//			executable = new File( javaHome, "bin/native2ascii");
//		}
//		if (!executable.exists()) {
//			if ( javaHomePath.endsWith("jre") || javaHomePath.endsWith("JRE") ) {
//				javaHomePath = javaHomePath.substring( 0, javaHomePath.length() - "/jre".length() );
//				javaHome = new File( javaHomePath );
//				if ( File.separatorChar == '\\') { // windows
//					executable = new File( javaHome, "bin\\native2ascii.exe");
//				} else {
//					executable = new File( javaHome, "bin/native2ascii");
//				}				
//			}
//			if (!executable.exists()) {
//				throw new IOException("Unable to find the native2ascii tool at [" + executable.getAbsolutePath() + "].");
//			}
//		}
//		String[] arguments;
//		if (encoding == null) {
//			arguments = new String[]{ executable.getAbsolutePath() };			
//		} else {
//			arguments = new String[]{
//					executable.getAbsolutePath(),
//					"-encoding",
//					encoding
//				};			
//		}
//		Runtime runtime = Runtime.getRuntime();
//		Process process = runtime.exec( arguments );
//		StreamWrapper wrapper = new StreamWrapper( in, process.getOutputStream() );
//		wrapper.start();
//		return process.getInputStream();
//	}
	
	/***************************************************************************************************/
	/* AsciiEncode                                                                                     */
	/***************************************************************************************************/

	/*********************************************************************************************************/
	/* HtmlEncode.- Translates Strings from native to html code.                                             */
	/*********************************************************************************************************/
	/* The HTML Document Character Set                                                                       */
	/*
		DecimalCode		Char 		Description 					Entity	 	Notes 	P
		&#00;			- &#08;		Unused
		&#09;						Horizontal tab		 	 
		&#10;						Line feed		 	 
		&#11;			- &#12;		Unused
		&#13;						Carriage Return		 	 
		&#14;			- &#31;		Unused
		&#32;						Space										space
		&#33;			!			Exclamation mark							exclam
		&#34;			"			Quotation mark					&quot;		quotedbl
		&#35;			#			Number sign	 	 							numbersign
		&#36;			$			Dollar sign	 	 							dollar
		&#37;			%			Percent sign	 							percent
		&#38;			&			Ampersand						&amp;		ampersand
		&#39;			'			Apostrophe	 	 							quotesingle
		&#40;			(			Left parenthesis							parenleft
		&#41;			)			Right parenthesis							parenright
		&#42;			*			Asterisk	 	 							asterisk
		&#43;			+			Plus sign	 	 							plus
		&#44;			,			Comma	 	 								comma
		&#45;			-			Hyphen	 	 								hyphen
		&#46;			.			Period (fullstop)							period
		&#47;			/			Solidus	 	 								slash
		&#48; - &#57;	0 - 9		Decimal digits				
		&#58;			:			Colon	 	 								colon
		&#59;			;			Semi-colon	 	 							semicolon
		&#60;			<			Less than						&lt;		less
		&#61;			=			Equals sign	 	 							equal
		&#62;			>			Greater than					&gt;		greater
		&#63;			?			Question mark	 							question
		&#64;			@			Commercial at	 	 						at
		&#65; - &#90;	A - Z		Uppercase letters			
		&#91;			[			Left square bracket						 	bracketleft
		&#92;			\			Reverse solidus	 	 						backslash
		&#93;			]			Right square bracket				 	 	bracketright
		&#94;			^			Caret	 	 								asciicircum
		&#95;			_			Horizontal bar	 	 						underscore
		&#96;			`			Grave accent	 	 						grave
		&#97; - &#122;	a - z		Lowercase letters				
		&#123;			{			Left curly brace	 				 		braceleft
		&#124;			|			Vertical bar	 	 						bar
		&#125;			}			Right curly brace	 				 		braceright
		&#126;			~			Tilde	 	 								asciitilde
		&#127; - &#159;				Unused (truly!)				
		&#160;						Nonbreaking space				&nbsp; 		nbspace nobreakspace
		&#192;						Capital A, grave accent			&Agrave;	Agrave
		&#224;						Small a, grave accent			&agrave;	agrave
		&#193;						CapitalA, acute accent			&Aacute;	Aacute
		&#225;						Small a, acute accent			&aacute;	aacute
		&#194;						Capital A, circumflex 			accent&Acirc;Acircumflex
		&#226;						Small a, circumflex accent		&acirc;	 	acircumflex
		&#195;						Capital A, tilde				&Atilde; 	Atilde
		&#227;						Small a, tilde					&atilde;	 atilde
		&#196;						Capital A, dieresisumlautmark	&Auml;	 	Adieresis
		&#228;						Small a, dieresisumlautmark		&auml;	 	adieresis
		&#197;						Capital A, ring					&Aring;	 	Aring
		&#229;						Small a, ring					&aring;	 	aring
		&#198;						Capital AE dipthong (ligature)	&AElig;	 	AE
		&#230;						Small ae dipthong (ligature)	&aelig;	 	ae
		&#199;						Capital C, cedilla				&Ccedil;	Ccedilla
		&#231;						Small c, cedilla				&ccedil;	ccedilla
		&#208;			GIF: 		Capital Eth, Icelandic			&ETH;		c1
		&#240;			GIF: 		Small eth, Icelandic			&eth;		c1
		&#200;						Capital E, grave accent			&Egrave;	Egrave
		&#232;						Small e, grave accent			&egrave; 	egrave
		&#201;						Capital E, acute accent			&Eacute; 	Eacute
		&#233;						Small e, acute accent			&eacute; 	eacute
		&#202;						Capital E, circumflex accent	&Ecirc;	 	Ecircumflex
		&#234;						Small e, circumflex accent		&ecirc;	 	ecircumflex
		&#203;						Capital E, dieresisumlautmark	&Euml;	 	Edieresis
		&#235;						Small e, dieresisumlautmark		&euml;	 	edieresis
		&#204;						Capital I, grave accent			&Igrave; 	Igrave
		&#236;						Small i, grave accent			&igrave; 	igrave
		&#205;						Capital I, acute accent			&Iacute; 	Iacute
		&#237;						Small i, acute accent			&iacute	 	iacute
		&#206;						Capital I, circumflex accent	&Icirc;	 	Icircumflex
		&#238;						Small i, circumflex accent		&icirc;	 	icircumflex
		&#207;						Capital I, dieresisumlautmark	&Iuml;	 	Idieresis
		&#239;						Small i, dieresisumlautmark		&iuml;	 	idieresis
		&#181;						Micro sign						&micro;	 	mu
		&#209;						Capital N, tilde				&Ntilde; 	Ntilde
		&#241;						Small n, tilde					&ntilde; 	ntilde
		&#210;						Capital O, grave accent			&Ograve; 	Ograve
		&#242;						Small o, grave accent			&ograve; 	ograve
		&#211;						Capital O, acute accent			&Oacute;	Oacute
		&#243;						Small o, acute accent			&oacute;	oacute
		&#212;						Capital O, circumflex accent	&Ocirc;	 	Ocircumflex
		&#244;						Small o, circumflex accent		&ocirc;	 	ocircumflex
		&#213;						Capital O, tilde				&Otilde;	Otilde
		&#245;						Small o, tilde					&otilde;	otilde
		&#214;						Capital O, dieresisumlautmark	&Ouml;	 	Odieresis
		&#246;						Small o, dieresisumlautmark		&ouml;	 	odieresis
		&#216;						Capital O, slash				&Oslash; 	Oslash
		&#248;						Small o, slash					&oslash; 	oslash
		&#223;						Small sharp s,German(szligature)&szlig;	 	germandbls
		&#222;			GIF: 		Capital THORN, Icelandic		&THORN;		c1	Thorn
		&#254;			GIF: 		Small thorn, Icelandic			&thorn;		c1	thorn
		&#217;						Capital U, grave accent			&Ugrave;	Ugrave
		&#249;						Small u, grave accent			&ugrave;	ugrave
		&#218;						Capital U, acute accent			&Uacute; 	Uacute
		&#250;						Small u, acute accent			&uacute;	uacute
		&#219;						Capital U, circumflex accent	&Ucirc;	 	Ucircumflex
		&#251;						Small u, circumflex accent		&ucirc;	 	ucircumflex
		&#220;						Capital U, dieresisumlautmark	&Uuml;	 	Udieresis
		&#252;						Small u, dieresisumlautmark		&uuml;	 	udieresis
		&#221;			GIF: 		Capital Y, acute accent			&Yacute;	c1	Yacute
		&#253;			GIF: 		Small y, acute accent			&yacute;	c1	yacute
		&#255;						Small y, dieresisumlautmark		&yuml;	 	ydieresis
		&#168;						Umlaut							&uml;	 	dieresis
		&#175;						Macron accent					&macr;	 	macron
		&#180;						Acute accent					&acute;	 	acute
		&#184;						Cedilla							&cedil;	 	cedilla
		&#161;						Inverted exclamation			&iexcl;	 	exclamdown
		&#191;						Inverted question mark			&iquest; 	questiondown
		&#183;						Middle dot						&middot; 	periodcentered
		&#166;			GIF: 		Broken vertical bar				&brvbar;	brokenbar
		&#171;						Left angle quote				&laquo;	 	guillemotleft
		&#187;						Right angle quote				&raquo;	 	guillemotright
		&#182;						Paragraph sign					&para;	 	paragraph
		&#167;						Section sign					&sect;	 	section
		&#169;						Copyright						&copy;	 	copyright
		&#174;						Registered trademark			&reg;	 	registered 
		&#185;			GIF: 		Superscript one					&sup1;		c1	onesuperior
		&#178;			GIF: 		Superscript two					&sup2;		c1	twosuperior
		&#179;			GIF: 		Superscript three				&sup3;		c1	threesuperior
		&#173;						Soft hyphen						&shy;		e1	minus(?)
		&#215;			GIF: 		Multiply sign					&times;		c1	multiply
		&#247;						Division sign					&divide;	divide
		&#188;			GIF: 		Fraction one-fourth				&frac14;	c1	onequarter
		&#189;			GIF: 		Fraction one-half				&frac12;	c1	onehalf
		&#190;			GIF: 		Fraction three-fourths			&frac34;	c1	threequarters
		&#170;						Feminine ordinal				&ordf;	 	ordfeminine
		&#186;						Masculine ordinal				&ordm;	 	ordmasculine
		&#172;						Not sign						&not;	 	logicalnot
		&#176;						Degree sign						&deg;	 	degree
		&#177;						Plus or minus					&plusmn;	plusminus
		&#164;						General currency sign			&curren;	currency
		&#162;						Cent sign						&cent;	 	cent
		&#163;						Pound sterling					&pound;	 	sterling
		&#165;						Yen sign						&yen;	 	yen
		Supplementary Characters (Unicode UCS-2 is "charset=UNICODE-1-1")
		&#916;			GIF: ?		delta							&Delta;		e0	Delta
		&#402;			GIF: 		florin							&fnof;		e0	florin
		&#937;			GIF: ?		omega							&Omega;		e0	Omega
		&#338;			GIF: 		OE ligature						&OElig;		e0	OE
		&#339;			GIF: 		oe ligature						&oelig;		e0	oe
		&#352;			GIF: 		Scaron							&Scaron;	e0	Scaron
		&#353;			GIF: 		scaron							&scaron;	e0	scaron
		&#376;			GIF: 		Ydieresis						&Yuml;		e0	Ydieresis
		&#305;			GIF: ?		dotless i						&inodot;	e0	dotlessi
		&#710;			GIF: 		circumflex						&circ;		e0	circumflex
		&#711;			GIF: ?		caron							&caron;		e0	caron
		&#728;			GIF: ?		breve							&breve;		e0	breve
		&#729;			GIF: ?		dot accent						&dot;		e0	dotaccent
		&#730;			GIF: ?		ring							&ring;		e0	ring
		&#731;			GIF: ?		ogonek							&ogon;		e0	ogonek
		&#732;			GIF: 		tilde							&tilde;		e0	tilde
		&#733;			GIF: ?		double acute accent				&dblac;		e0	hungarumlaut
		&#8211;			GIF: 		en dash							&ndash;		e1	endash
		&#8212;			GIF: 		em dash							&mdash;		e1	emdash
		&#8224;			GIF: 		dagger							&dagger;	e0	dagger
		&#8225;			GIF: 		double dagger					&Dagger;	e0	daggerdbl
		&#8226;			GIF: 		bullet							&bull;		e0	bullet
		&#8230;			GIF: 		ellipsis						&hellip;	e0	ellipsis
		&#8216;			GIF: 		quote left						&lsquo;		e0	quoteleft
		&#8217;			GIF: 		quote right						&rsquo;		e0	quoteright
		&#8218;			GIF: 		quote single base				&lsquor;	e0	quotesinglbase
		&#8220;			GIF: 		quote double left				&ldquo;		e0	quotedblleft
		&#8221;			GIF: 		quote double right				&rdquo;		e0	quotedblright
		&#8222;			GIF: 		quote double base				&ldquor;	e0	quotedblbase
		&#8249;			GIF: 		guille single left				&lsaquo;	e?	guilsinglleft
		&#8250;			GIF: 		guille single right				&rsaquo;	e?	guilsinglright
		&#8482;			GIF: 		trademark, TM					&trade;		e1	trademark trademarkserif
		&#8480;			GIF: ^(SM)	service mark, SM	 	 	 
		&#8471;			GIF: ?		sound recording copyright, (P)	 	 	 
		&#8730;			GIF: ?		radical							&radic;		e0	radical
		&#8734;			GIF: ?		infinity						&infin;		e0	infinity
		&#8747;			GIF: ?		integral						&int;		e0	integral
		&#8706;			GIF: ?		partial differential			&part;		e0	partialdiff
		&#8773;			GIF: ?		approximately equal				&ap;		e0	approxequal
		&#8800;			GIF: ?		not equal						&ne;		e0	notequal
		&#8804;			GIF: ?		less than or equal				&le;		e0	lessequal
		&#8805;			GIF: ?		greater than or equal			&ge;		e0	greaterequal
		&#931;			GIF: ?		summation						&sum;		e0	summation
		&#8240;			GIF: 		per thousand (mille)			&permil;	e0	perthousand
		&#8260;			GIF: ?		fraction separator	 	 		fraction
		&#8719;		 	product										&prod;		e0	product
		&#960;		 	pi											&pi;		e0	pi
		&#9674;	 					lozenge (diamond)				&loz;		e0	lozenge
		&#8355;	 		franc	 	franc
		&#63743;	 				Apple logo	apple
		&#8984;			GIF: ?		command key	 	 	 
		&#8997;			GIF: ?		option key	 	 	 
		&#9774;			GIF: ?		peace symbol	 	 	 
		&#9775;			GIF: ?		yin yang	 	 	  
		*/
		/********************************************************************************************************/
		/* The HTML Document Character Set                                                                      */
		/********************************************************************************************************/
		/**
		 * Variant of {@link #encode} where encodeNewline is false and encodeNbsp is true.
		 **/
		public static String htmlEncode (String string){
			return htmlEncode(string, false, true);
		}
		/**
		 * Variant of {@link #encode} where encodeNbsp is true.
		 **/
		public static String htmlEncode (String string, boolean encodeNewline){
			return htmlEncode(string, encodeNewline, true);
		}
		/**
	 	 * Encodes the given string, so that it can be used within a html page.
	 	 * @param string the string to convert
	 	 * @param encodeNewline if true newline characters are converted to &lt;br&gt;'s
	 	 * @param encodeSubsequentBlanksToNbsp if true subsequent blanks are converted to &amp;nbsp;'s
	 	 **/
		public static String htmlEncode (String string, boolean encodeNewline, boolean encodeSubsequentBlanksToNbsp){
			if (string == null){
				return "";
			}
			StringBuffer sb = null; //create later on demand
			String app;
			char c;
			for (int i = 0; i < string.length (); ++i) {
				app = null;
				c = string.charAt(i);
				switch (c){
					case '"': app = "&quot;";	break; //"
					case '&': app = "&amp;";	break; //&
					case '<': app = "&lt;";		break; //<
					case '>': app = "&gt;";		break; //>70
					case ' ':
						if (encodeSubsequentBlanksToNbsp && (i == 0 || (i - 1 >= 0 && string.charAt(i - 1) == ' '))) {
							//Space at beginning or after another space
							app = "&#160;";
						}
						break;
					case '\n':
						if (encodeNewline){
							app = "<br/>";
						}
						break;
					//german umlauts
					case '\u00E4' : app = "&auml;"; break;
					case '\u00C4' : app = "&Auml;"; break;
					case '\u00F6' : app = "&ouml;"; break;
					case '\u00D6' : app = "&Ouml;"; break;
					case '\u00FC' : app = "&uuml;"; break;
					case '\u00DC' : app = "&Uuml;"; break;
					case '\u00DF' : app = "&szlig;"; break;
					//misc
					//case 0x80: app = "&euro;"; break; sometimes euro symbol is ascii 128, should we suport it?
					case '\u20AC': app = "&euro;"; break;
					case '\u00AB': app = "&laquo;"; break;
					case '\u00BB': app = "&raquo;"; break;
					case '\u00A0': app = "&#160;"; break;
					default:
						if (((int)c) >= 0x80) {
							//encode all non basic latin characters
							app = "&#" + ((int)c) + ";";
						}
						break;
				}
				if (app != null){
					if (sb == null){
						sb = new StringBuffer(string.substring(0, i));
					}
					sb.append(app);
				} else {
					if (sb != null){
						sb.append(c);
					}
				}
			}
			if (sb == null){
				return string;
			} else {
				return sb.toString();
			}
		}
		/*
		public static String htmlEncode(String text) throws Exception{
			if (text == null){
				return "";
			}
			char[] source = text.toCharArray();
			int sourceLength = source.length;
			StringBuffer sb = new StringBuffer();
			for (int i = 0; i < sourceLength; i++) {
				char c = source[i];
				String code = HTMLEncoder.getHTMLCode((short) c);
				if (code != null) {
					sb.append(code);
				} else {
					if (((int) c) > 255) {
						sb.append("&#");
						sb.append((int) c);
						sb.append(";");
					} else {
						sb.append(c);
					}
				}
			}
			return sb.toString();
		}
		private static class HTMLEncoder {
			private static HashMap htmlEncodingMap = null;
			public static final String getHTMLCode(short code) {
				initHtmlEncodingMap();
				return (String) htmlEncodingMap.get(new Short(code));
			}
			public static void initHtmlEncodingMap(){
				if (htmlEncodingMap == null) {
					htmlEncodingMap = new HashMap(253);
					synchronized (htmlEncodingMap) {
						htmlEncodingMap.put(new Short((short) 39), "&#39;");
						htmlEncodingMap.put(new Short((short) 92), "&#92;");
						htmlEncodingMap.put(new Short((short) 160), "&nbsp;");
						htmlEncodingMap.put(new Short((short) 161), "&iexcl;");
						htmlEncodingMap.put(new Short((short) 162), "&cent;");
						htmlEncodingMap.put(new Short((short) 163), "&pound;");
						htmlEncodingMap.put(new Short((short) 164), "&curren;");
						htmlEncodingMap.put(new Short((short) 165), "&yen;");
						htmlEncodingMap.put(new Short((short) 166), "&brvbar;");
						htmlEncodingMap.put(new Short((short) 167), "&sect;");
						htmlEncodingMap.put(new Short((short) 168), "&uml;");
						htmlEncodingMap.put(new Short((short) 169), "&copy;");
						htmlEncodingMap.put(new Short((short) 170), "&ordf;");
						htmlEncodingMap.put(new Short((short) 171), "&laquo;");
						htmlEncodingMap.put(new Short((short) 172), "&not;");
						htmlEncodingMap.put(new Short((short) 173), "&shy;");
						htmlEncodingMap.put(new Short((short) 174), "&reg;");
						htmlEncodingMap.put(new Short((short) 175), "&macr;");
						htmlEncodingMap.put(new Short((short) 176), "&deg;");
						htmlEncodingMap.put(new Short((short) 177), "&plusmn;");
						htmlEncodingMap.put(new Short((short) 178), "&sup2;");
						htmlEncodingMap.put(new Short((short) 179), "&sup3;");
						htmlEncodingMap.put(new Short((short) 180), "&acute;");
						htmlEncodingMap.put(new Short((short) 181), "&micro;");
						htmlEncodingMap.put(new Short((short) 182), "&para;");
						htmlEncodingMap.put(new Short((short) 183), "&middot;");
						htmlEncodingMap.put(new Short((short) 184), "&cedil;");
						htmlEncodingMap.put(new Short((short) 185), "&sup1;");
						htmlEncodingMap.put(new Short((short) 186), "&ordm;");
						htmlEncodingMap.put(new Short((short) 187), "&raquo;");
						htmlEncodingMap.put(new Short((short) 188), "&frac14;");
						htmlEncodingMap.put(new Short((short) 189), "&frac12;");
						htmlEncodingMap.put(new Short((short) 190), "&frac34;");
						htmlEncodingMap.put(new Short((short) 191), "&iquest;");
						htmlEncodingMap.put(new Short((short) 192), "&Agrave;");
						htmlEncodingMap.put(new Short((short) 193), "&Aacute;");
						htmlEncodingMap.put(new Short((short) 194), "&Acirc;");
						htmlEncodingMap.put(new Short((short) 195), "&Atilde;");
						htmlEncodingMap.put(new Short((short) 196), "&Auml;");
						htmlEncodingMap.put(new Short((short) 197), "&Aring;");
						htmlEncodingMap.put(new Short((short) 198), "&AElig;");
						htmlEncodingMap.put(new Short((short) 199), "&Ccedil;");
						htmlEncodingMap.put(new Short((short) 200), "&Egrave;");
						htmlEncodingMap.put(new Short((short) 201), "&Eacute;");
						htmlEncodingMap.put(new Short((short) 202), "&Ecirc;");
						htmlEncodingMap.put(new Short((short) 203), "&Euml;");
						htmlEncodingMap.put(new Short((short) 204), "&Igrave;");
						htmlEncodingMap.put(new Short((short) 205), "&Iacute;");
						htmlEncodingMap.put(new Short((short) 206), "&Icirc;");
						htmlEncodingMap.put(new Short((short) 207), "&Iuml;");
						htmlEncodingMap.put(new Short((short) 208), "&ETH;");
						htmlEncodingMap.put(new Short((short) 209), "&Ntilde;");
						htmlEncodingMap.put(new Short((short) 210), "&Ograve;");
						htmlEncodingMap.put(new Short((short) 211), "&Oacute;");
						htmlEncodingMap.put(new Short((short) 212), "&Ocirc;");
						htmlEncodingMap.put(new Short((short) 213), "&Otilde;");
						htmlEncodingMap.put(new Short((short) 214), "&Ouml;");
						htmlEncodingMap.put(new Short((short) 215), "&times;");
						htmlEncodingMap.put(new Short((short) 216), "&Oslash;");
						htmlEncodingMap.put(new Short((short) 217), "&Ugrave;");
						htmlEncodingMap.put(new Short((short) 218), "&Uacute;");
						htmlEncodingMap.put(new Short((short) 219), "&Ucirc;");
						htmlEncodingMap.put(new Short((short) 220), "&Uuml;");
						htmlEncodingMap.put(new Short((short) 221), "&Yacute;");
						htmlEncodingMap.put(new Short((short) 222), "&THORN;");
						htmlEncodingMap.put(new Short((short) 223), "&szlig;");
						htmlEncodingMap.put(new Short((short) 224), "&agrave;");
						htmlEncodingMap.put(new Short((short) 225), "&aacute;");
						htmlEncodingMap.put(new Short((short) 226), "&acirc;");
						htmlEncodingMap.put(new Short((short) 227), "&atilde;");
						htmlEncodingMap.put(new Short((short) 228), "&auml;");
						htmlEncodingMap.put(new Short((short) 229), "&aring;");
						htmlEncodingMap.put(new Short((short) 230), "&aelig;");
						htmlEncodingMap.put(new Short((short) 231), "&ccedil;");
						htmlEncodingMap.put(new Short((short) 232), "&egrave;");
						htmlEncodingMap.put(new Short((short) 233), "&eacute;");
						htmlEncodingMap.put(new Short((short) 234), "&ecirc;");
						htmlEncodingMap.put(new Short((short) 235), "&euml;");
						htmlEncodingMap.put(new Short((short) 236), "&igrave;");
						htmlEncodingMap.put(new Short((short) 237), "&iacute;");
						htmlEncodingMap.put(new Short((short) 238), "&icirc;");
						htmlEncodingMap.put(new Short((short) 239), "&iuml;");
						htmlEncodingMap.put(new Short((short) 240), "&eth;");
						htmlEncodingMap.put(new Short((short) 241), "&ntilde;");
						htmlEncodingMap.put(new Short((short) 242), "&ograve;");
						htmlEncodingMap.put(new Short((short) 243), "&oacute;");
						htmlEncodingMap.put(new Short((short) 244), "&ocirc;");
						htmlEncodingMap.put(new Short((short) 245), "&otilde;");
						htmlEncodingMap.put(new Short((short) 246), "&ouml;");
						htmlEncodingMap.put(new Short((short) 247), "&divide;");
						htmlEncodingMap.put(new Short((short) 248), "&oslash;");
						htmlEncodingMap.put(new Short((short) 249), "&ugrave;");
						htmlEncodingMap.put(new Short((short) 250), "&uacute;");
						htmlEncodingMap.put(new Short((short) 251), "&ucirc;");
						htmlEncodingMap.put(new Short((short) 252), "&uuml;");
						htmlEncodingMap.put(new Short((short) 253), "&yacute;");
						htmlEncodingMap.put(new Short((short) 254), "&thorn;");
						htmlEncodingMap.put(new Short((short) 255), "&yuml;");
						htmlEncodingMap.put(new Short((short) 402), "&fnof;");
						htmlEncodingMap.put(new Short((short) 913), "&Alpha;");
						htmlEncodingMap.put(new Short((short) 914), "&Beta;");
						htmlEncodingMap.put(new Short((short) 915), "&Gamma;");
						htmlEncodingMap.put(new Short((short) 916), "&Delta;");
						htmlEncodingMap.put(new Short((short) 917), "&Epsilon;");
						htmlEncodingMap.put(new Short((short) 918), "&Zeta;");
						htmlEncodingMap.put(new Short((short) 919), "&Eta;");
						htmlEncodingMap.put(new Short((short) 920), "&Theta;");
						htmlEncodingMap.put(new Short((short) 921), "&Iota;");
						htmlEncodingMap.put(new Short((short) 922), "&Kappa;");
						htmlEncodingMap.put(new Short((short) 923), "&Lambda;");
						htmlEncodingMap.put(new Short((short) 924), "&Mu;");
						htmlEncodingMap.put(new Short((short) 925), "&Nu;");
						htmlEncodingMap.put(new Short((short) 926), "&Xi;");
						htmlEncodingMap.put(new Short((short) 927), "&Omicron;");
						htmlEncodingMap.put(new Short((short) 928), "&Pi;");
						htmlEncodingMap.put(new Short((short) 929), "&Rho;");
						htmlEncodingMap.put(new Short((short) 931), "&Sigma;");
						htmlEncodingMap.put(new Short((short) 932), "&Tau;");
						htmlEncodingMap.put(new Short((short) 933), "&Upsilon;");
						htmlEncodingMap.put(new Short((short) 934), "&Phi;");
						htmlEncodingMap.put(new Short((short) 935), "&Chi;");
						htmlEncodingMap.put(new Short((short) 936), "&Psi;");
						htmlEncodingMap.put(new Short((short) 937), "&Omega;");
						htmlEncodingMap.put(new Short((short) 945), "&alpha;");
						htmlEncodingMap.put(new Short((short) 946), "&beta;");
						htmlEncodingMap.put(new Short((short) 947), "&gamma;");
						htmlEncodingMap.put(new Short((short) 948), "&delta;");
						htmlEncodingMap.put(new Short((short) 949), "&epsilon;");
						htmlEncodingMap.put(new Short((short) 950), "&zeta;");
						htmlEncodingMap.put(new Short((short) 951), "&eta;");
						htmlEncodingMap.put(new Short((short) 952), "&theta;");
						htmlEncodingMap.put(new Short((short) 953), "&iota;");
						htmlEncodingMap.put(new Short((short) 954), "&kappa;");
						htmlEncodingMap.put(new Short((short) 955), "&lambda;");
						htmlEncodingMap.put(new Short((short) 956), "&mu;");
						htmlEncodingMap.put(new Short((short) 957), "&nu;");
						htmlEncodingMap.put(new Short((short) 958), "&xi;");
						htmlEncodingMap.put(new Short((short) 959), "&omicron;");
						htmlEncodingMap.put(new Short((short) 960), "&pi;");
						htmlEncodingMap.put(new Short((short) 961), "&rho;");
						htmlEncodingMap.put(new Short((short) 962), "&sigmaf;");
						htmlEncodingMap.put(new Short((short) 963), "&sigma;");
						htmlEncodingMap.put(new Short((short) 964), "&tau;");
						htmlEncodingMap.put(new Short((short) 965), "&upsilon;");
						htmlEncodingMap.put(new Short((short) 966), "&phi;");
						htmlEncodingMap.put(new Short((short) 967), "&chi;");
						htmlEncodingMap.put(new Short((short) 968), "&psi;");
						htmlEncodingMap.put(new Short((short) 969), "&omega;");
						htmlEncodingMap.put(new Short((short) 977), "&thetasym;");
						htmlEncodingMap.put(new Short((short) 978), "&upsih;");
						htmlEncodingMap.put(new Short((short) 982), "&piv;");
						htmlEncodingMap.put(new Short((short) 8226), "&bull;");
						htmlEncodingMap.put(new Short((short) 8230), "&hellip;");
						htmlEncodingMap.put(new Short((short) 8242), "&prime;");
						htmlEncodingMap.put(new Short((short) 8243), "&Prime;");
						htmlEncodingMap.put(new Short((short) 8254), "&oline;");
						htmlEncodingMap.put(new Short((short) 8260), "&frasl;");
						htmlEncodingMap.put(new Short((short) 8472), "&weierp;");
						htmlEncodingMap.put(new Short((short) 8465), "&image;");
						htmlEncodingMap.put(new Short((short) 8476), "&real;");
						htmlEncodingMap.put(new Short((short) 8482), "&trade;");
						htmlEncodingMap.put(new Short((short) 8501), "&alefsym;");
						htmlEncodingMap.put(new Short((short) 8592), "&larr;");
						htmlEncodingMap.put(new Short((short) 8593), "&uarr;");
						htmlEncodingMap.put(new Short((short) 8594), "&rarr;");
						htmlEncodingMap.put(new Short((short) 8595), "&darr;");
						htmlEncodingMap.put(new Short((short) 8596), "&harr;");
						htmlEncodingMap.put(new Short((short) 8629), "&crarr;");
						htmlEncodingMap.put(new Short((short) 8656), "&lArr;");
						htmlEncodingMap.put(new Short((short) 8657), "&uArr;");
						htmlEncodingMap.put(new Short((short) 8658), "&rArr;");
						htmlEncodingMap.put(new Short((short) 8659), "&dArr;");
						htmlEncodingMap.put(new Short((short) 8660), "&hArr;");
						htmlEncodingMap.put(new Short((short) 8704), "&forall;");
						htmlEncodingMap.put(new Short((short) 8706), "&part;");
						htmlEncodingMap.put(new Short((short) 8707), "&exist;");
						htmlEncodingMap.put(new Short((short) 8709), "&empty;");
						htmlEncodingMap.put(new Short((short) 8711), "&nabla;");
						htmlEncodingMap.put(new Short((short) 8712), "&isin;");
						htmlEncodingMap.put(new Short((short) 8713), "&notin;");
						htmlEncodingMap.put(new Short((short) 8715), "&ni;");
						htmlEncodingMap.put(new Short((short) 8719), "&prod;");
						htmlEncodingMap.put(new Short((short) 8721), "&sum;");
						htmlEncodingMap.put(new Short((short) 8722), "&minus;");
						htmlEncodingMap.put(new Short((short) 8727), "&lowast;");
						htmlEncodingMap.put(new Short((short) 8730), "&radic;");
						htmlEncodingMap.put(new Short((short) 8733), "&prop;");
						htmlEncodingMap.put(new Short((short) 8734), "&infin;");
						htmlEncodingMap.put(new Short((short) 8736), "&ang;");
						htmlEncodingMap.put(new Short((short) 8743), "&and;");
						htmlEncodingMap.put(new Short((short) 8744), "&or;");
						htmlEncodingMap.put(new Short((short) 8745), "&cap;");
						htmlEncodingMap.put(new Short((short) 8746), "&cup;");
						htmlEncodingMap.put(new Short((short) 8747), "&int;");
						htmlEncodingMap.put(new Short((short) 8756), "&there4;");
						htmlEncodingMap.put(new Short((short) 8764), "&sim;");
						htmlEncodingMap.put(new Short((short) 8773), "&cong;");
						htmlEncodingMap.put(new Short((short) 8776), "&asymp;");
						htmlEncodingMap.put(new Short((short) 8800), "&ne;");
						htmlEncodingMap.put(new Short((short) 8801), "&equiv;");
						htmlEncodingMap.put(new Short((short) 8804), "&le;");
						htmlEncodingMap.put(new Short((short) 8805), "&ge;");
						htmlEncodingMap.put(new Short((short) 8834), "&sub;");
						htmlEncodingMap.put(new Short((short) 8835), "&sup;");
						htmlEncodingMap.put(new Short((short) 8836), "&nsub;");
						htmlEncodingMap.put(new Short((short) 8838), "&sube;");
						htmlEncodingMap.put(new Short((short) 8839), "&supe;");
						htmlEncodingMap.put(new Short((short) 8853), "&oplus;");
						htmlEncodingMap.put(new Short((short) 8855), "&otimes;");
						htmlEncodingMap.put(new Short((short) 8869), "&perp;");
						htmlEncodingMap.put(new Short((short) 8901), "&sdot;");
						htmlEncodingMap.put(new Short((short) 8968), "&lceil;");
						htmlEncodingMap.put(new Short((short) 8969), "&rceil;");
						htmlEncodingMap.put(new Short((short) 8970), "&lfloor;");
						htmlEncodingMap.put(new Short((short) 8971), "&rfloor;");
						htmlEncodingMap.put(new Short((short) 9001), "&lang;");
						htmlEncodingMap.put(new Short((short) 9002), "&rang;");
						htmlEncodingMap.put(new Short((short) 9674), "&loz;");
						htmlEncodingMap.put(new Short((short) 9824), "&spades;");
						htmlEncodingMap.put(new Short((short) 9827), "&clubs;");
						htmlEncodingMap.put(new Short((short) 9829), "&hearts;");
						htmlEncodingMap.put(new Short((short) 9830), "&diams;");
						htmlEncodingMap.put(new Short((short) 34), "&quot;");
						htmlEncodingMap.put(new Short((short) 38), "&amp;");
						htmlEncodingMap.put(new Short((short) 60), "&lt;");
						htmlEncodingMap.put(new Short((short) 62), "&gt;");
						htmlEncodingMap.put(new Short((short) 338), "&OElig;");
						htmlEncodingMap.put(new Short((short) 339), "&oelig;");
						htmlEncodingMap.put(new Short((short) 352), "&Scaron;");
						htmlEncodingMap.put(new Short((short) 353), "&scaron;");
						htmlEncodingMap.put(new Short((short) 376), "&Yuml;");
						htmlEncodingMap.put(new Short((short) 710), "&circ;");
						htmlEncodingMap.put(new Short((short) 732), "&tilde;");
						htmlEncodingMap.put(new Short((short) 8194), "&ensp;");
						htmlEncodingMap.put(new Short((short) 8195), "&emsp;");
						htmlEncodingMap.put(new Short((short) 8201), "&thinsp;");
						htmlEncodingMap.put(new Short((short) 8204), "&zwnj;");
						htmlEncodingMap.put(new Short((short) 8205), "&zwj;");
						htmlEncodingMap.put(new Short((short) 8206), "&lrm;");
						htmlEncodingMap.put(new Short((short) 8207), "&rlm;");
						htmlEncodingMap.put(new Short((short) 8211), "&ndash;");
						htmlEncodingMap.put(new Short((short) 8212), "&mdash;");
						htmlEncodingMap.put(new Short((short) 8216), "&lsquo;");
						htmlEncodingMap.put(new Short((short) 8217), "&rsquo;");
						htmlEncodingMap.put(new Short((short) 8218), "&sbquo;");
						htmlEncodingMap.put(new Short((short) 8220), "&ldquo;");
						htmlEncodingMap.put(new Short((short) 8221), "&rdquo;");
						htmlEncodingMap.put(new Short((short) 8222), "&bdquo;");
						htmlEncodingMap.put(new Short((short) 8224), "&dagger;");
						htmlEncodingMap.put(new Short((short) 8225), "&Dagger;");
						htmlEncodingMap.put(new Short((short) 8240), "&permil;");
						htmlEncodingMap.put(new Short((short) 8249), "&lsaquo;");
						htmlEncodingMap.put(new Short((short) 8250), "&rsaquo;");
						htmlEncodingMap.put(new Short((short) 8364), "&euro;");
					}
				}
			}
		}
		*/
		
		/***************************************************************************************************/
		/* htmlEncode                                                                                      */
		/***************************************************************************************************/
}
