package eu.semiramis.util.encode;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;

/**
 * <p>Translates Strings from native to ascii code and vice versa.</p>
 * <p>Usage from the command-line: java de.enough.polish.util.Native2Ascii [-reverse] filepath [encoding]</p>
 * <p>Codificacion de caracteres 
 * <pre>
 * </pre>
 */
public class NativeToAscii {

	private NativeToAscii() {
		super();
	}
	/**
	 * 
	 * @param args
	 * @throws IOException
	 */
	public static void main(String[] args) throws IOException {
		System.out.println("***************************************************");
		System.out.println("[NativeToAscii.main()] Start");
		System.out.println("***************************************************");
		if (args.length == 0) {
			System.out.println("[NativeToAscii.main()] Usage: com.vln.util.encode.Native2Ascii [-reverse] filepath [encoding]");
			return;
		}
		//Get reverseFlag, filaPath and Encoding
		String filePath = args[0];
		boolean reverse = false;
		if ("-reverse".equals(filePath)) {
			reverse = true;
			if (args.length == 1) {
				System.out.println("[NativeToAscii.main()] Usage: com.vln.util.encode.Native2Ascii [-reverse] filepath [encoding]");
				return;
			}
			filePath = args[1];
		}
		File file = new File(filePath);
		String encoding = null;
		if ( (args.length > 1 && !reverse) || args.length > 2 ) {
			if (reverse) {
				encoding = args[2];
			} else {
				encoding = args[1];
			}
		}
		System.out.println("[Native2Ascii.main()] Args:");
		System.out.println("[Native2Ascii.main()] \t filePath: " + filePath);
		System.out.println("[Native2Ascii.main()] \t reverse: " + reverse);
		System.out.println("[Native2Ascii.main()] \t encoding: " + encoding);
		//Convert
		//InputStream in = translateToAscii( new FileInputStream( file ), encoding );
		BufferedReader reader = encoding == null ?
				new BufferedReader(new InputStreamReader(new FileInputStream( file ))) :
				new BufferedReader(new InputStreamReader(new FileInputStream( file ), encoding));
		String line;
		//int i = 1;
		while ((line = reader.readLine()) != null) {
			System.out.println("[Native2Ascii.main()] readedLine: " + line);
			//System.out.print( i + ": [");
			if (reverse) {
				System.out.println("[Native2Ascii.main()] \t ascii2NativeLine: " + asciiToNative(line));				
			} else {
				System.out.println("[Native2Ascii.main()] \t native2AsciiLine: " +  nativeToAscii(line));
				System.out.println("[Native2Ascii.main()] \t nativeToEscapedAsciiLine: " +  nativeToEscapedAscii(line, false));
			}
			//System.out.println("]");
			//i++;
		}
		//in.close();
		System.out.println("***************************************************");
		System.out.println("[NativeToAscii.main()] End");
		System.out.println("***************************************************");
	}
	
//	/**
//	 * Translates the given input stream to ascii code
//	 * @param in the input stream
//	 * @param encoding the encoding, when null the default encoding is used
//	 * @return the input stream that contains the ascii representation of the given input stream
//	 * @throws IOException when the native2ascii tool could not be found.
//	 */
//	public static InputStream translateToAscii( InputStream in, String encoding ) throws IOException {
//		String javaHomePath = System.getProperty("java.home");
//		File javaHome = new File( javaHomePath );
//		if (!javaHome.exists()) {
//			throw new IOException("The system property java.home points to the invalid destination [" + javaHome.getAbsolutePath() + "].");
//		}
//		File executable;
//		if ( File.separatorChar == '\\') { // windows
//			executable = new File( javaHome, "bin\\native2ascii.exe");
//		} else {
//			executable = new File( javaHome, "bin/native2ascii");
//		}
//		if (!executable.exists()) {
//			if ( javaHomePath.endsWith("jre") || javaHomePath.endsWith("JRE") ) {
//				javaHomePath = javaHomePath.substring( 0, javaHomePath.length() - "/jre".length() );
//				javaHome = new File( javaHomePath );
//				if ( File.separatorChar == '\\') { // windows
//					executable = new File( javaHome, "bin\\native2ascii.exe");
//				} else {
//					executable = new File( javaHome, "bin/native2ascii");
//				}				
//			}
//			if (!executable.exists()) {
//				throw new IOException("Unable to find the native2ascii tool at [" + executable.getAbsolutePath() + "].");
//			}
//		}
//		String[] arguments;
//		if (encoding == null) {
//			arguments = new String[]{ executable.getAbsolutePath() };			
//		} else {
//			arguments = new String[]{
//					executable.getAbsolutePath(),
//					"-encoding",
//					encoding
//				};			
//		}
//		Runtime runtime = Runtime.getRuntime();
//		Process process = runtime.exec( arguments );
//		StreamWrapper wrapper = new StreamWrapper( in, process.getOutputStream() );
//		wrapper.start();
//		return process.getInputStream();
//	}
	
	/** A table of hex digits */
	private static final String UXXXX_SPECIAL_SAVE_CHARS = "=: \t\r\n\f#!";	
	private static final char[] hexDigit = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
	/**
	 * nativeToEscapedAscii
	 * Converts native to encoded &#92;uxxxx
	 * and writes out any of the characters in SPECIAL_SAVE_CHARS
	 * with a preceding slash
	 */
	public static String nativeToEscapedAscii(String theString, boolean escapeSpace){
		int len = theString.length();
		StringBuffer outBuffer = new StringBuffer(len*2);
		for (int x=0; x<len; x++) {
			char aChar = theString.charAt(x);
			switch (aChar) {
			case ' ':
				if (x == 0 || escapeSpace){
					outBuffer.append('\\');
				}
				outBuffer.append(' ');
				break;
			case '\\':
				outBuffer.append('\\');
				outBuffer.append('\\');
				break;
			case '\t':
				outBuffer.append('\\');
				outBuffer.append('t');
				break;
			case '\n':
				outBuffer.append('\\');
				outBuffer.append('n');
				break;
			case '\r':
				outBuffer.append('\\');
				outBuffer.append('r');
				break;
			case '\f':
				outBuffer.append('\\');
				outBuffer.append('f');
				break;
			default:
				if ((aChar < 0x0020) || (aChar > 0x007e)) {
					outBuffer.append('\\');
					outBuffer.append('u');
					outBuffer.append(toHex((aChar >> 12) & 0xF));
					outBuffer.append(toHex((aChar >>  8) & 0xF));
					outBuffer.append(toHex((aChar >>  4) & 0xF));
					outBuffer.append(toHex( aChar        & 0xF));
				} else {
					if (UXXXX_SPECIAL_SAVE_CHARS.indexOf(aChar) != -1){
						outBuffer.append('\\');
					}
					outBuffer.append(aChar);
				}
				break;
			}
		}
		return outBuffer.toString();
	}
	/**
	 * Convert a nibble to a hex character
	 * @param   nibble  the nibble to convert.
	 */
	private static char toHex(int nibble) {
		return hexDigit[(nibble & 0xF)];
	}
	/**
	 * Translates the given String into ASCII code.
	 * 
	 * @param input the input which contains native characters like umlauts etc
	 * @return the input in which native characters are replaced through ASCII code
	 */
	public static String nativeToAscii( String input ) {
		if (input == null) {
			return null;
		}
		StringBuffer buffer = new StringBuffer( input.length() + 60 );
		for (int i = 0; i < input.length(); i++) {
			char c = input.charAt(i);
			if (c <= 0x7E) {
				buffer.append(c);
			} else {
				buffer.append("\\u");
				String hex = Integer.toHexString(c);
				for (int j = hex.length(); j < 4; j++ ) {
					buffer.append( '0' );
				}
				buffer.append( hex );
			}
		}
		return buffer.toString();
	}
	/**
	 * Translates the given String into ASCII code.
	 * 
	 * @param input the input which contains native characters like umlauts etc
	 * @return the input in which native characters are replaced through ASCII code
	 */
	public static String asciiToNative( String input ) {
		if (input == null) {
			return null;
		}
		StringBuffer buffer = new StringBuffer( input.length() );
		boolean precedingBackslash = false;
		for (int i = 0; i < input.length(); i++) {
			char c = input.charAt(i);
			if (precedingBackslash) {
				switch (c) {
					case 'f': c = '\f'; break;
					case 'n': c = '\n'; break;
					case 'r': c = '\r'; break;
					case 't': c = '\t'; break;
					case 'u':
						String hex = input.substring( i + 1, i + 5 );
						c = (char) Integer.parseInt(hex, 16 );
						i += 4;
				}
				precedingBackslash = false;
			} else {
				precedingBackslash = (c == '\\');
			}
			if (!precedingBackslash) {
				buffer.append(c);
			}
		}
		return buffer.toString();
	}
}
