package eu.semiramis.util.file;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;


public class FileManagerBean {
	private final static String KEY_LINENUMBER = "lineNumber";
	private final static String KEY_LINESTREAM = "lineStream";
	public final static String DIR_CURRENT = ".";
	public final static String DIR_PARENT = "..";
	public static final int ORDERBY_FILENAME 							= 1;
	public static final int ORDERBY_FILESIZE 							= 2;
	public static final int ORDERBY_FILETYPE 							= 3;
	public static final int ORDERBY_FILEDATE 							= 4;
	public static final int ORDERTYPE_ASC 								= 1;
	public static final int ORDERTYPE_DESC	 							= 2;
	
	private String dir = null;
	private String message = null;
	private String command = null;
	private String accessControlMode = null;
	private int orderBy = ORDERBY_FILENAME;
	private int orderType = ORDERTYPE_ASC;
	
	/**
	 * FileManagerBean
	 * @param jobDir
	 */
	public FileManagerBean(){
		setDir("./");
	}
	public FileManagerBean(String basePath){
		setDir(basePath);
	}
	
	/**
	 * pwd
	 * @return
	 */
	public File getWorkDirectory(){
		File workDirectory = null;
		if (dir != null){
			workDirectory = new File(dir);
		}
		return workDirectory;
	}
	/**
	 * @return
	 */
	public File[] getFileListArray() {
		ArrayList dirArrayList = new ArrayList();
		ArrayList fileArrayList = new ArrayList();
		File[] files = FileUtil.listFiles(dir);
		if (files != null){
			int filesLength = files.length;
			dirArrayList.add(new File(dir + File.separator + DIR_PARENT));
			for (int i = 0; i < filesLength; i ++){
				File file = files[i];
				if (file.isDirectory()){
					dirArrayList.add(file);
				} else if (file.isFile()){
					fileArrayList.add(file);
				}
			}
		}
		ArrayList arrayList = new ArrayList();
		arrayList.addAll(dirArrayList);
		arrayList.addAll(fileArrayList);
		File[] fileArray = new File[arrayList.size()];
		fileArray = (File[]) arrayList.toArray(fileArray);
		return fileArray;
	}
	
	
	
	public File[] getOrderedFileListArray(int orderBy, int orderType) {
		//Load elements in directory
		ArrayList dirArrayList = new ArrayList();
		ArrayList fileArrayList = new ArrayList();
		File[] files = FileUtil.listFiles(dir);
		if (files != null){
			int filesLength = files.length;
			for (int i = 0; i < filesLength; i ++){
				File file = files[i];
				if (file.isDirectory()){
					dirArrayList.add(file);
				} else if (file.isFile()){
					fileArrayList.add(file);
				}
			}
		}
		//GetComparator
		File[] dirArray = new File[dirArrayList.size()];
		dirArray = (File[]) dirArrayList.toArray(dirArray);
		Arrays.sort(dirArray, new FileManagerComparator(orderBy, orderType));
		int dirArrayLength = dirArray.length;
		File[] fileArray = new File[fileArrayList.size()];
		fileArray = (File[]) fileArrayList.toArray(fileArray);
		Arrays.sort(fileArray, new FileManagerComparator(orderBy, orderType));
		int fileArrayLength = fileArray.length;
		File[] orderedFileArray = new File[1 + dirArrayLength + fileArrayLength];
		orderedFileArray[0] = new File(dir + File.separator + DIR_PARENT);
		for (int i = 1; i < 1 + dirArrayLength; i ++){
			orderedFileArray[i] = dirArray[i - 1];
		}
		for (int i = 1 + dirArrayLength; i < 1 + dirArrayLength + fileArrayLength; i ++){
			orderedFileArray[i] = fileArray[i - 1 - dirArrayLength];
		}
		return orderedFileArray;
	}
	/**
	 * 
	 * FileManagerComparator
	 *
	 */
	private class FileManagerComparator implements Comparator{
		int orderBy = -1;
		int orderType = -1;
 
		public FileManagerComparator(int orderBy, int orderType){
			this.orderBy = orderBy;
			this.orderType = orderType;
		}
		public int compare(Object o1, Object o2){
			File file1 = (File) o1;
			File file2 = (File) o2;
			int result = 0;
			switch (orderBy){
				case ORDERBY_FILESIZE:
					int fileSize1 = FileUtil.getFileDataLength(file1.getAbsolutePath());				
					int fileSize2 = FileUtil.getFileDataLength(file2.getAbsolutePath());
					result = fileSize1 - fileSize2;
					break;
				case ORDERBY_FILETYPE:
					String fileType1 = FileUtil.getFileNameExtension(file1.getAbsolutePath());
					String fileType2 = FileUtil.getFileNameExtension(file2.getAbsolutePath());
					result = fileType1.compareTo(fileType2);
					break;
				case ORDERBY_FILEDATE:
					String fileLong1 = new Long(file1.lastModified()).toString();
					String fileLong2 = new Long(file2.lastModified()).toString();
					result = fileLong1.compareTo(fileLong2);
					break;
				case ORDERBY_FILENAME:
				default:
					String fileName1 = file1.getName().toLowerCase();
					String fileName2 = file2.getName().toLowerCase();
					result = fileName1.compareTo(fileName2);
					break;
			}
			if (orderType == ORDERTYPE_DESC){
				result = (-1) * result;
			}
			return result;
		}
	}
	/**
	 * @return
	 */
	public String getDir() {
		return dir;
	}


	/**
	 * @param string
	 */
	public void setDir(String newDir) {
		if (FileUtil.isExistentDirectory(newDir)){
			File file = new File(newDir);
			try {
				this.dir = file.getCanonicalPath();
			} catch (Exception e){
				setMessage(e.getMessage());
			}
		} else {
			setMessage("Inexistant Directory: " + newDir);
		}
	}

	/**
	 * @return
	 */
	public String getMessage() {
		return message;
	}

	/**
	 * @param string
	 */
	public void setMessage(String string) {
		message = string;
	}

	/**
	 * @return
	 */
	public String getAccessControlMode() {
		return accessControlMode;
	}

	/**
	 * @param string
	 */
	public void setAccessControlMode(String string) {
		accessControlMode = string;
	}
	
	/**
	 * isReadable
	 * @return
	 * @throws Exception
	 */	
	public boolean isReadable() throws Exception{
		boolean isReadable = false;
			if (accessControlMode != null && !accessControlMode.trim().equals("") && accessControlMode.trim().length() == 3){
				char readableBit = accessControlMode.trim().charAt(0);
				if (readableBit == '1'){
					isReadable = true;
				}
			}
		return isReadable;
	}

	/**
	 * isWritable
	 * @return
	 * @throws Exception
	 */	
	public boolean isWritable() throws Exception{
		boolean isWritable = false;
			if (accessControlMode != null && !accessControlMode.trim().equals("") && accessControlMode.trim().length() == 3){
				char writableBit = accessControlMode.trim().charAt(1);
				if (writableBit == '1'){
					isWritable = true;
				}
			}
		return isWritable;
	}
	
	/**
	 * isExecutable
	 * @return
	 * @throws Exception
	 */	
	public boolean isExecutable() throws Exception{
		boolean isExecutable = false;
			if (accessControlMode != null && !accessControlMode.trim().equals("") && accessControlMode.trim().length() == 3){
				char executableBit = accessControlMode.trim().charAt(2);
				if (executableBit == '1'){
					isExecutable = true;
				}
			}
		return isExecutable;
	}

	

	/**
	 * @return
	 */
	public String getCommand() {
		return command;
	}

	/**
	 * @param string
	 */
	public void setCommand(String string) {
		command = string;
	}

	/**
	 * @return
	 */
	public int getOrderBy() {
		return orderBy;
	}

	/**
	 * @return
	 */
	public int getOrderType() {
		return orderType;
	}

	/**
	 * @param i
	 */
	public void setOrderBy(int i) {
		orderBy = i;
	}
	public void setOrderBy(String s) {
		setOrderBy(new Integer(s).intValue());
	}

	/**
	 * @param i
	 */
	public void setOrderType(int i) {
		orderType = i;
	}
	public void setOrderType(String s) {
		setOrderType(new Integer(s).intValue());
	}
	
	
	

}

