package eu.semiramis.util.file;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Locale;
import java.util.StringTokenizer;

import javax.servlet.Servlet;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import eu.semiramis.federationproxy.audit.AuditManager;
import eu.semiramis.format.FormatUtil;
import eu.semiramis.util.string.StringUtil;
import eu.semiramis.util.updown.DownloadUtil;
import eu.semiramis.util.updown.MultipartUtil;

public class FileManagerServlet extends HttpServlet implements Servlet {
	
	public static final String PARAMNAME_CONFIGDIRSYSTEMVARNAME		= "configDirSystemVarName";
	public static final String PARAMNAME_CONFIGPUBLICKKEYFILENAME	= "configPublicKeyFileName";
	private static String configPublicKeyFilePath = null;
	private static String configFilePath = null;
	public static String getConfigPublicKeyFilePath() {
		return configPublicKeyFilePath;
	}
	
	
	
	//ACTIONS
	public final static int		ACTION_NONE											= 0;
	public final static int		ACTION_CHANGEDIRECTORY								= 1;
	public final static int		ACTION_UPLOAD										= 2;
	public final static int		ACTION_DOWNLOAD										= 3;
	public final static int		ACTION_REMOVE										= 4;
	public final static int		ACTION_EXECUTECOMMAND								= 5;
	public final static int		ACTION_LIST											= 6;
	//TARGETS
	public final static int		TARGET_NONE											= 0;
	public final static int		TARGET_FILEMANAGERPAGE								= 1;
	//PARAMETERS
	public final static String	PARAMETER_FILEMANAGERBEAN							= "fileManagerBean";
	public final static String	PARAMETER_FILEMANAGER_ACTION						= "action";
	public final static String	PARAMETER_FILEMANAGER_COMMAND						= "fileManagerCommand";
	public final static String	PARAMETER_FILEMANAGER_DIR							= "fileManagerDir";
	public final static String	PARAMETER_FILEMANAGER_FILE							= "fileManagerFile";
	public final static String	PARAMETER_FILEMANAGER_UPLOADFILE					= "uploadFile";
	public final static String	PARAMETER_FILEMANAGER_DOWNLOADCONTENTDISPOSITION	= "downloadContentDisposition";
	public final static String	PARAMETER_FILEMANAGER_ORDERBY						= "orderBy";
	public final static String	PARAMETER_FILEMANAGER_ORDERTYPE						= "orderType";
	//CONTENTDISPOSITION
	public final static int		CODE_CONTENTDISPOSITION_ATTACHMENT 					= 1;
	public final static int		CODE_CONTENTDISPOSITION_INLINE 						= 2;
	public final static int		CODE_CONTENTDISPOSITION_ZIPATTACHMENT				= 3;
	//COMMANDS
	public static final String 	COMMANDKEY_CD										= "cd";
	public static final String 	COMMANDKEY_CHMOD									= "chmod";
	public static final String 	COMMANDKEY_MKDIR									= "mkdir";
	public static final String 	COMMANDKEY_TOUCH									= "touch";
	
	
	
	
	/**
	 * doGet
	 */
	public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		executeServlet(request, response);		
	}
	/**
	 * doPost
	 */
	public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		executeServlet(request, response);		
	}
	/**
	 * executeServlet
	 * @param request
	 * @param response
	 * @throws IOException
	 */
	private void executeServlet(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
		try{
			String action = null;
			String fileManagerCommand = null;
			String fileManagerDir = null;
			String fileManagerFile = null;
			String fileManagerOrderBy = null;
			String fileManagerOrderType = null;
			MultipartUtil multipartUtil = null;
			Object downloadContentDisposition = null;
			String configFileManagerDir = getInitParameter(PARAMETER_FILEMANAGER_DIR);
			boolean isMultipart = MultipartUtil.isMultipartContent(request);
			if (isMultipart) {
				multipartUtil = new MultipartUtil(request);
				action = (String) multipartUtil.getParameter(PARAMETER_FILEMANAGER_ACTION);
				fileManagerCommand = (String) multipartUtil.getParameter(PARAMETER_FILEMANAGER_COMMAND);
				fileManagerDir = (String) multipartUtil.getParameter(PARAMETER_FILEMANAGER_DIR);
				fileManagerFile = (String) multipartUtil.getParameter(PARAMETER_FILEMANAGER_FILE);
				fileManagerOrderBy = (String) multipartUtil.getParameter(PARAMETER_FILEMANAGER_ORDERBY);
				fileManagerOrderType = (String) multipartUtil.getParameter(PARAMETER_FILEMANAGER_ORDERTYPE);
				downloadContentDisposition = (String) multipartUtil.getParameter(PARAMETER_FILEMANAGER_DOWNLOADCONTENTDISPOSITION);
			} else {
				action = request.getParameter(PARAMETER_FILEMANAGER_ACTION);
				fileManagerCommand = (String) request.getParameter(PARAMETER_FILEMANAGER_COMMAND);
				fileManagerDir = (String) request.getParameter(PARAMETER_FILEMANAGER_DIR);
				fileManagerFile = (String) request.getParameter(PARAMETER_FILEMANAGER_FILE);
				fileManagerOrderBy = (String) request.getParameter(PARAMETER_FILEMANAGER_ORDERBY);
				fileManagerOrderType = (String) request.getParameter(PARAMETER_FILEMANAGER_ORDERTYPE);
				downloadContentDisposition = (String) request.getParameter(PARAMETER_FILEMANAGER_DOWNLOADCONTENTDISPOSITION);
			}
			FileManagerBean fileManagerBean = loadFileManagerBean(request, configFileManagerDir);
			int targetCode = TARGET_FILEMANAGERPAGE;
			//Manage AccessControlMode
			String accessControlMode = fileManagerBean.getAccessControlMode();
			//Manage Action Code
			if (action != null){
				int actionCode = new Integer(action).intValue();
				switch (actionCode){
					case ACTION_UPLOAD:
						if (fileManagerBean.isWritable()){
							fileManagerBean = executeUpload(response, multipartUtil, fileManagerBean);
						}
						targetCode = TARGET_FILEMANAGERPAGE;
						break;
					case ACTION_REMOVE:
						if (fileManagerBean.isWritable()){
							fileManagerBean = executeRemove(multipartUtil, fileManagerFile, fileManagerBean);
						}
						targetCode = TARGET_FILEMANAGERPAGE;
						break;
					case ACTION_CHANGEDIRECTORY:
						if (fileManagerBean.isReadable()){
							fileManagerBean = executeChangeDirectory(fileManagerDir, fileManagerBean);
						}
						targetCode = TARGET_FILEMANAGERPAGE;
						break;
					case ACTION_DOWNLOAD:
						if (fileManagerBean.isReadable()){
							fileManagerBean = executeDownload(response, fileManagerFile, downloadContentDisposition, multipartUtil, fileManagerBean);
							targetCode = TARGET_NONE;
						} else {
							targetCode = TARGET_FILEMANAGERPAGE;
						}
						break;
					case ACTION_EXECUTECOMMAND:
						fileManagerBean = executeCommand(fileManagerCommand, fileManagerBean);
						targetCode = TARGET_FILEMANAGERPAGE;
						break;
					case ACTION_LIST:
						fileManagerBean = executeList(fileManagerOrderBy, fileManagerOrderType, fileManagerBean);
						targetCode = TARGET_FILEMANAGERPAGE;
						break;
					case ACTION_NONE:
						targetCode = TARGET_FILEMANAGERPAGE;
						break;
					default:
						targetCode = TARGET_FILEMANAGERPAGE;
						break;
				}
			}
			//Manage Response
			switch (targetCode){
				case TARGET_FILEMANAGERPAGE:
					storeFileManagerBean(request, fileManagerBean);
					sendFileManagerResponse(request, response, fileManagerBean);
					break;
				case TARGET_NONE:
					break;
			}
		} catch (Exception e){
			e.printStackTrace();
		}
	}
	/**
	 * loadFileManagerBean
	 * @param request
	 * @return
	 * @throws Exception
	 */
	private FileManagerBean loadFileManagerBean(HttpServletRequest request, String configFileManagerDir) throws Exception{
		HttpSession session = request.getSession();
		FileManagerBean fileManagerBean = (FileManagerBean) session.getAttribute(PARAMETER_FILEMANAGERBEAN);
		if (fileManagerBean == null){
			
			String auditBasePath = AuditManager.getAuditBasePath();
			if (auditBasePath != null){
				fileManagerBean = new FileManagerBean(auditBasePath);
			} else {
				fileManagerBean = new FileManagerBean();
			}
			fileManagerBean.setDir(configFileManagerDir);
		}
		fileManagerBean.setMessage(null);
		fileManagerBean.setCommand(null);
		return fileManagerBean;
	}
	/**
	 * storeFileManagerBean
	 * @param request
	 * @param fileManagerBean
	 * @throws Exception
	 */	
	private void storeFileManagerBean(HttpServletRequest request, FileManagerBean fileManagerBean) throws Exception{
		HttpSession session = request.getSession();
		session.setAttribute(PARAMETER_FILEMANAGERBEAN, fileManagerBean);
	}
	/**
	 * isRelativePath
	 * @param path
	 * @return
	 */
	private boolean isRelativePath(String path){
		boolean isRelativePath = true;
		if (path != null){
			path = path.trim().replace('\\', '/');
			//Case Unix
			if (path.startsWith("/")){
				isRelativePath = false;
			}
			//Case Windows
			if (path.indexOf(":") > -1){
				isRelativePath = false;
			}
		}
		return isRelativePath;
	}
	/**
	 * executeDownload
	 * @param multipartUtil
	 * @param fileManagerBean
	 * @return
	 * @throws Exception
	 */
	private FileManagerBean executeDownload(HttpServletResponse response, String fileItem, Object downloadContentDisposition, MultipartUtil multipartUtil, FileManagerBean fileManagerBean) throws Exception{
		if (downloadContentDisposition == null){
			downloadContentDisposition = new Integer(CODE_CONTENTDISPOSITION_ATTACHMENT).toString();
		}
		if (FileUtil.isExistentFile(fileItem)){
			String filePath = fileItem;
			if (filePath != null){
				String fileName = FileUtil.getFileName(filePath);
				int downloadContentDispositionCode = new Integer((String)downloadContentDisposition).intValue();
				switch (downloadContentDispositionCode){
					case CODE_CONTENTDISPOSITION_INLINE:
						DownloadUtil.downloadAsInline(response, fileName, filePath);
						break;
					case CODE_CONTENTDISPOSITION_ZIPATTACHMENT:
						DownloadUtil.downloadAsZipAttachment(response, fileName, filePath);
						//byte[] byteArray = FileUtil.loadAsByteArray(filePath);
						//DownloadUtil.downloadAsZipAttachment(response, fileName, byteArray);
					case CODE_CONTENTDISPOSITION_ATTACHMENT:
					default: 
						DownloadUtil.downloadAsAttachment(response, fileName, filePath);
						break;
						
				}
			}
		}
		return fileManagerBean;
	}
	/**
	 * executeUpload
	 * @param multipartUtil
	 * @param fileManagerBean
	 * @return
	 * @throws Exception
	 */
	private FileManagerBean executeUpload(HttpServletResponse response, MultipartUtil multipartUtil, FileManagerBean fileManagerBean) throws Exception{
		if (FileUtil.isExistentDirectory(fileManagerBean.getDir())){
			HashMap fileItemMap = (HashMap) multipartUtil.getParameter(PARAMETER_FILEMANAGER_UPLOADFILE);
			if (fileItemMap != null){
				String fileName = FileUtil.getFileName((String) fileItemMap.get(MultipartUtil.KEY_FILENAME));
				if (fileName != null){
					String fileContentType = (String) fileItemMap.get(MultipartUtil.KEY_FILECONTENTTYPE);
					Long fileSize = (Long) fileItemMap.get(MultipartUtil.KEY_FILESIZE);
					byte[] fileByteArray = (byte[]) fileItemMap.get(MultipartUtil.KEY_FILEBYTEARRAY);
					if (fileByteArray != null && fileByteArray.length > 0){
						String fileManagerDir = FileUtil.formatFilePath(fileManagerBean.getDir());
						String uploadFilePath = fileManagerDir + '/' + fileName;
						if (uploadFilePath != null){
							if (FileUtil.isExistent(uploadFilePath)){
								String fileManagerMessage = "UPLOAD ERROR: File already present -" + fileName + "-";
								fileManagerBean.setMessage(fileManagerMessage);
							} else {
								FileUtil.save(uploadFilePath, fileByteArray);
							}
						}
						
					}
				}
			}
		}
		return fileManagerBean;
	}
	/**
	 * executeRemove
	 * @param multipartUtil
	 * @param fileManagerBean
	 * @return
	 * @throws Exception
	 */
	private FileManagerBean executeRemove(MultipartUtil multipartUtil, String filePath, FileManagerBean fileManagerBean) throws Exception{
		if (FileUtil.isExistentFile(filePath)){
			FileUtil.removeFile(filePath);				
		} else if (FileUtil.isExistentDirectory(filePath)){
			FileUtil.removeDirectory(filePath);
		}
		return fileManagerBean;
	}
	/**
	 * executeCommand
	 * @param multipartUtil
	 * @param fileManagerBean
	 * @return
	 * @throws Exception
	 */
	public FileManagerBean executeCommand(String command, FileManagerBean fileManagerBean) throws Exception{
		String commandKey = command;
		ArrayList commandArgsArrayList = new ArrayList();
		String commandArg = null;		
		StringTokenizer st = new StringTokenizer(command, " ");
		if (st.hasMoreTokens()){
			commandKey = st.nextToken();  
		}
		while (st.hasMoreTokens()){
			commandArgsArrayList.add(st.nextToken());
		}
		if (commandKey != null && !commandKey.trim().equals("")){
			if (commandKey.equals(COMMANDKEY_CHMOD)){
				if (commandArgsArrayList != null && commandArgsArrayList.size() == 1){
					commandArg = (String)commandArgsArrayList.get(0);
					fileManagerBean = executeChangeAccessControlMode(commandArg, fileManagerBean);
				} else {
					fileManagerBean.setMessage("Invalid command arguments");
				}
			} else if (commandKey.equals(COMMANDKEY_CD)){
				if (fileManagerBean.isReadable()){
					if (commandArgsArrayList != null && commandArgsArrayList.size() == 1){
						commandArg = (String)commandArgsArrayList.get(0);
						fileManagerBean = executeChangeDirectory(commandArg, fileManagerBean);
					} else {
						fileManagerBean.setMessage("Invalid command arguments");
					}
				} else {
					fileManagerBean.setMessage("Not enough permissions");
				}
			} else if (commandKey.equals(COMMANDKEY_MKDIR)){
				if (fileManagerBean.isWritable()){
					if (commandArgsArrayList != null && commandArgsArrayList.size() == 1){
						commandArg = (String)commandArgsArrayList.get(0);
						fileManagerBean = executeCreateDirectory(commandArg, fileManagerBean);
					} else {
						fileManagerBean.setMessage("Invalid command arguments");
					}
				} else {
					fileManagerBean.setMessage("Not enough permissions");
				}
			} else if (commandKey.equals(COMMANDKEY_TOUCH)){
				if (fileManagerBean.isWritable()){
					if (commandArgsArrayList != null && commandArgsArrayList.size() == 1){
						commandArg = (String)commandArgsArrayList.get(0);
						fileManagerBean = executeCreateFile(commandArg, fileManagerBean);
					} else {
						fileManagerBean.setMessage("Invalid command arguments");
					}
				} else {
					fileManagerBean.setMessage("Not enough permissions");
				}
			} else {
				fileManagerBean.setMessage("Command Not Found");
			}
		}
		if (fileManagerBean.getMessage() != null){
			fileManagerBean.setCommand(command);
		}
		return fileManagerBean;
	}
	/**
	 * executeChangeDirectory
	 * @param multipartUtil
	 * @param fileManagerBean
	 * @return
	 * @throws Exception
	 */
	private FileManagerBean executeChangeDirectory(String path, FileManagerBean fileManagerBean) throws Exception{
		if (path != null){
			String formatedPath = null;
			formatedPath = path.trim().replace('\\', '/');
			if (formatedPath.equals("home")){
				formatedPath = "./";
			} else if (isRelativePath(formatedPath)){
				String currentPath = fileManagerBean.getWorkDirectory().getCanonicalPath().replace('\\', '/');
				formatedPath = currentPath + '/' + formatedPath;
			}
			fileManagerBean.setDir(formatedPath);			
		}
		return fileManagerBean;
	}

	/**
	 * executeChangeAccessControlMode
	 * @param fileManagerBean
	 * @return
	 * @throws Exception
	 */
	public FileManagerBean executeChangeAccessControlMode(String accessControlMode, FileManagerBean fileManagerBean) throws Exception{
		fileManagerBean.setAccessControlMode(accessControlMode);
		return fileManagerBean;
	}
	/**
	 * executeCreateDirectory
	 * @param directoryName
	 * @param fileManagerBean
	 * @return
	 * @throws Exception
	 */
	public FileManagerBean executeCreateDirectory(String directoryName, FileManagerBean fileManagerBean) throws Exception{
		String formatedDirectoryName = directoryName.replace('\\', '/');
		if (isRelativePath(formatedDirectoryName)){
			if (formatedDirectoryName.endsWith("/")){
				formatedDirectoryName = formatedDirectoryName.substring(0, formatedDirectoryName.length() -1);
			}
			if (!formatedDirectoryName.startsWith(FileManagerBean.DIR_CURRENT) && !formatedDirectoryName.startsWith(FileManagerBean.DIR_PARENT)){
				String currentPath = fileManagerBean.getWorkDirectory().getCanonicalPath().replace('\\', '/');
				formatedDirectoryName = currentPath + '/' + formatedDirectoryName;
				if (FileUtil.isExistentFile(formatedDirectoryName)){
					fileManagerBean.setMessage("Directory already exist as file");
				} else if (FileUtil.isExistentDirectory(formatedDirectoryName)){
					fileManagerBean.setMessage("Directory already exist");
				} else {
					try {
						FileUtil.createDirectory(formatedDirectoryName);
					} catch (Exception e){
						fileManagerBean.setMessage(e.getMessage());
					}
				}
			} else {
				fileManagerBean.setMessage("DirectoryName can not starts with '.' or '..'");
			}
		} else {
			fileManagerBean.setMessage("Directory Name must be relative to the current directory");
		}
		return fileManagerBean;
	}

	/**
	 * executeCreateFile
	 * @param fileName
	 * @param fileManagerBean
	 * @return
	 * @throws Exception
	 */
	public FileManagerBean executeCreateFile(String fileName, FileManagerBean fileManagerBean) throws Exception{
		String formatedFileName = fileName.replace('\\', '/');
		if (isRelativePath(formatedFileName)){
			if (!formatedFileName.startsWith(FileManagerBean.DIR_CURRENT) && !formatedFileName.startsWith(FileManagerBean.DIR_PARENT)){
				String currentPath = fileManagerBean.getWorkDirectory().getCanonicalPath().replace('\\', '/');
				formatedFileName = currentPath + '/' + formatedFileName;
				if (FileUtil.isExistentFile(formatedFileName)){
					fileManagerBean.setMessage("File already exist");
				} else if (FileUtil.isExistentDirectory(formatedFileName)){
					fileManagerBean.setMessage("File already exist as directory");
				} else {
					try {
						FileUtil.createFile(formatedFileName);
					} catch (Exception e){
						fileManagerBean.setMessage(e.getMessage());				
					}
				}
			} else {
				fileManagerBean.setMessage("FileName can not starts with '.' or '..'");
			}
		} else {
			fileManagerBean.setMessage("FileName must be relative to the current directory");
		}
		return fileManagerBean;
	}
	/**
	 * executeList
	 * @param fileManagerOrderBy
	 * @param fileManagerOrderType
	 * @param fileManagerBean
	 * @return
	 */
	public FileManagerBean executeList(String fileManagerOrderBy, String fileManagerOrderType, FileManagerBean fileManagerBean){
		fileManagerBean.setOrderBy(fileManagerOrderBy);
		fileManagerBean.setOrderType(fileManagerOrderType);
		return fileManagerBean;
	}
	/**
	 * sendFileManagerResponse
	 * @param request
	 * @param response
	 * @param fileManagerBean
	 * @throws Exception
	 */
	private void sendFileManagerResponse(HttpServletRequest request, HttpServletResponse response, FileManagerBean fileManagerBean) throws Exception{
		//request.setAttribute(PARAMETER_FILEMANAGERBEAN, fileManagerBean);
		//String target = "fileManager.jsp";
		//RequestDispatcher requestDispatcher = request.getRequestDispatcher(target);
		//requestDispatcher.forward(request, response);
		String fileManagerAccessControlMode = fileManagerBean.getAccessControlMode();
		String fileManagerCommand = "";
		File[] fileManagerFileListArray = new File[0];
		String fileManagerMessage = "";
		String fileManagerDir = "";
		int fileManagerOrderBy = FileManagerBean.ORDERBY_FILENAME;
		int fileManagerOrderType = FileManagerBean.ORDERTYPE_ASC;
		if (fileManagerBean != null){
			fileManagerOrderBy = fileManagerBean.getOrderBy();
			fileManagerOrderType = fileManagerBean.getOrderType();
			fileManagerFileListArray = fileManagerBean.getOrderedFileListArray(fileManagerOrderBy, fileManagerOrderType);
			fileManagerCommand = fileManagerBean.getCommand();
			if (fileManagerCommand == null){
				fileManagerCommand = "";
			}
			fileManagerMessage = fileManagerBean.getMessage();
			if (fileManagerMessage == null){
				fileManagerMessage = "";
			}
			fileManagerDir = fileManagerBean.getDir();
			if (fileManagerDir == null){
				fileManagerDir = "";
			}
		}
		StringBuffer sb = new StringBuffer();
		sb.append("<HTML>").append("\n");
		sb.append("<HEAD>").append("\n");
		sb.append("<meta http-equiv='Expires' content='0'>").append("\n");
		sb.append("<meta http-equiv='Last-Modified' content='0'>").append("\n");
		sb.append("<meta http-equiv='Cache-Control' content='no-cache, mustrevalidate'>").append("\n");
		sb.append("<meta http-equiv='Pragma' content='no-cache'>").append("\n");
		sb.append("<STYLE>").append("\n");
		sb.append("	.titrePage {").append("\n");
		sb.append("		color:				#FFFFFF;").append("\n");
		sb.append("		font-family:		arial;").append("\n");
		sb.append("		font-size:			18px;").append("\n");
		sb.append("		font-weight:		bold;").append("\n");
		sb.append("		background-color: 	#008000;").append("\n");
		sb.append("		cellspacing:		1;").append("\n");
		sb.append("		width:				100%;").append("\n");
		sb.append("		vertical-align: 	middle;").append("\n");
		sb.append("	}").append("\n");
		sb.append("	.et {").append("\n");
		sb.append("		color: 				#FFFFFF;").append("\n");
		sb.append("		background-color: 	#299304;").append("\n");
		sb.append("		font-family:    	arial;").append("\n");
		sb.append("		font-size:      	12px;").append("\n");
		sb.append("		font-weight: 		bold;").append("\n");
		sb.append("		text-align: 		left;").append("\n");
		sb.append("		vertical-align: 	top;").append("\n");
		sb.append("	}").append("\n");
		sb.append("	.ct {").append("\n");
		sb.append("		color: 				#000000;").append("\n");
		sb.append("		background-color: 	#E4E4E4;").append("\n");
		sb.append("		font-family:    	arial;").append("\n");
		sb.append("		font-size:      	12px;").append("\n");
		sb.append("		vertical-align: 	top;").append("\n");
		sb.append("	}").append("\n");
		sb.append("	.bt {").append("\n");
		sb.append("		color: 				#000000;").append("\n");
		sb.append("		background-color: 	#F4F4F4;").append("\n");
		sb.append("		font-family:    	arial;").append("\n");
		sb.append("		font-size:      	12px;").append("\n");
		sb.append("		vertical-align: 	top;").append("\n");
		sb.append("	}").append("\n");
		sb.append("	div.fileinputs {").append("\n");
		sb.append("		position:			relative;").append("\n");
		sb.append("	}").append("\n");
		sb.append("	div.fakefile {").append("\n");
		sb.append("		position:			absolute;").append("\n");
		sb.append("		top:				0px;").append("\n");
		sb.append("		left:				0px;").append("\n");
		sb.append("		z-index:			1;").append("\n");
		sb.append("	}").append("\n");
		sb.append("	input.file {").append("\n");
		sb.append("		position:			relative;").append("\n");
		sb.append("		text-align:			right;").append("\n");
		sb.append("		-moz-opacity:		0 ;").append("\n");
		sb.append("		filter:				alpha(opacity: 0);").append("\n");
		sb.append("		opacity:			0;").append("\n");
		sb.append("		z-index:			2;").append("\n");
		sb.append("	}").append("\n");
		sb.append("</STYLE>").append("\n");
		sb.append("<SCRIPT>").append("\n");
		sb.append("function man_onClick(){").append("\n");
		sb.append("	var regexpLibWdw = window.open('', 'regexpLibWdw');").append("\n");
		sb.append("	with (regexpLibWdw.document) {").append("\n");
		sb.append("		open('text/html', 'FileManagerCommands');").append("\n");
		sb.append("		writeln('<H2>FileManagerCommands</H2>');").append("\n");
		sb.append("		writeln('<H3>cd</H3>');").append("\n");
		sb.append("		writeln('<P>');").append("\n");
		sb.append("		writeln('Permite cambiar el directorio de trabajo.');").append("\n");
		sb.append("		writeln('<BR>&nbsp;&nbsp;&nbsp;&nbsp;cd <I>ruta</I>');").append("\n");
		sb.append("		writeln('<BR>Para rutas relativas se aceptan los caracteres especiales como:');").append("\n");
		sb.append("		writeln('<BR>&nbsp;&nbsp;&nbsp;&nbsp;<B>.</B> Directorio actual');").append("\n");
		sb.append("		writeln('<BR>&nbsp;&nbsp;&nbsp;&nbsp;<B>..</B> Directorio padre');").append("\n");
		sb.append("		writeln('<H3>mkdir</H3>');").append("\n");
		sb.append("		writeln('<P>');").append("\n");
		sb.append("		writeln('Crea un directorio.');").append("\n");
		sb.append("		writeln('<BR>&nbsp;&nbsp;&nbsp;&nbsp;mkdir <I>ruta</I>');").append("\n");
		sb.append("		writeln('<H3>touch</H3>');").append("\n");
		sb.append("		writeln('<P>');").append("\n");
		sb.append("		writeln('Crea un fichero.');").append("\n");
		sb.append("		writeln('<BR>&nbsp;&nbsp;&nbsp;&nbsp;touch <I>ruta</I>');").append("\n");
		sb.append("		writeln('<H3>chmod</H3>');").append("\n");
		sb.append("		writeln('<P>');").append("\n");
		sb.append("		writeln('<B>Este comando no se corresponde para nada con el chmod de unix</B>.');").append("\n");
		sb.append("		writeln('<BR>Cambia el modo de trabajo de la aplicacin de acuerdo a tres digitos [rwx] de valor [0|1] que habilitan|desabilitan los permisos de lectura|escritura|ejecucion de la aplicacin.');").append("\n");
		sb.append("		writeln('<BR>&nbsp;&nbsp;&nbsp;&nbsp;chmod <I>[rwx]</I>');").append("\n");
		sb.append("		close();").append("\n");
		sb.append("	}").append("\n");
		sb.append("	regexpLibWdw.focus();").append("\n");
		sb.append("}").append("\n");
		sb.append("function fileManager_onLoad(){").append("\n");
		sb.append("}").append("\n");
		//sb.append("function initFileUpload(id) {").append("\n");
		//sb.append("	var W3CDOM = (document.createElement && document.getElementsByTagName);").append("\n");
		//sb.append("	if (!W3CDOM){").append("\n");
		//sb.append("		return;").append("\n");
		//sb.append("	}").append("\n");
		//sb.append("	var fakeFileUpload = document.createElement('div');").append("\n");
		//sb.append("	fakeFileUpload.className = 'fakefile';").append("\n");
		//sb.append("	fakeFileUpload.style.margin = '2 0 0 0';").append("\n");
		//sb.append("	var inputText = document.createElement('input');").append("\n");
		//sb.append("	inputText.style.font = '10px arial';").append("\n");
		//sb.append("	//inputText.style.border = 'solid black 1px';").append("\n");
		//sb.append("	inputText.style.verticalAlign = 'top';").append("\n");
		//sb.append("	fakeFileUpload.appendChild(inputText);").append("\n");
		//sb.append("	var inputButton = document.createElement('input');").append("\n");
		//sb.append("	inputButton.type = 'button';").append("\n");
		//sb.append("	inputButton.value = 'Browse';").append("\n");
		//sb.append("	inputButton.style.font = '10px arial';").append("\n");
		//sb.append("	inputButton.style.color = 'black';").append("\n");
		//sb.append("	inputButton.style.margin = '0 5 0 5';").append("\n");
		//sb.append("	inputButton.style.border = 'solid black 1px';").append("\n");
		//sb.append("	inputButton.style.verticalAlign = 'top';").append("\n");
		//sb.append("	fakeFileUpload.appendChild(inputButton);").append("\n");
		//sb.append("	var element = document.getElementById(id);").append("\n");
		//sb.append("	if (element.parentNode.className == 'fileinputs') {").append("\n");
		//sb.append("		element.className = 'file hidden';").append("\n");
		//sb.append("		var clone = fakeFileUpload.cloneNode(true);").append("\n");
		//sb.append("		element.parentNode.appendChild(clone);").append("\n");
		//sb.append("		element.relatedElement = clone.getElementsByTagName('input')[0];").append("\n");
		//sb.append("		element.onchange = element.onmouseout = function () {").append("\n");
		//sb.append("			this.relatedElement.value = this.value;").append("\n");
		//sb.append("		}").append("\n");
		//sb.append("	}").append("\n");
		//sb.append("}").append("\n");
		sb.append("function checkEnter(e, target){");
		sb.append("		var keyCode = e.keyCode;");
		sb.append("		if(keyCode == 13){");
		sb.append("			eval(target);");
		sb.append("		}");
		sb.append("}");
		sb.append("function setDir_onClick(){").append("\n");
		sb.append("	document.formFileManager.").append(FileManagerServlet.PARAMETER_FILEMANAGER_ACTION).append(".value = '").append(FileManagerServlet.ACTION_CHANGEDIRECTORY).append("';").append("\n");
		sb.append("	document.formFileManager.submit();").append("\n");
		sb.append("}").append("\n");
		sb.append("function executeCommand_onKeyup(){").append("\n");
		sb.append("}").append("\n");
		sb.append("function removeFile_onClick(fileId){").append("\n");
		if (fileManagerBean.isWritable()){
			sb.append("	var confirmFlag = confirm(\"WARNING.- The selected file will be removed. Continue anyway?\");").append("\n");
			sb.append("	if (confirmFlag){").append("\n");
			sb.append("		document.formFileManager.").append(FileManagerServlet.PARAMETER_FILEMANAGER_ACTION).append(".value = '").append(FileManagerServlet.ACTION_REMOVE).append("';").append("\n");
			sb.append("		document.formFileManager.").append(FileManagerServlet.PARAMETER_FILEMANAGER_FILE).append(".value = eval ('document.formFileManager.' + fileId + '.value');").append("\n");
			sb.append("		document.formFileManager.submit();").append("\n");
			sb.append("	}").append("\n");
		}
		sb.append("}").append("\n");
		sb.append("function removeDirectory_onClick(fileId){").append("\n");
		if (fileManagerBean.isWritable()){
			sb.append("	var confirmFlag = confirm(\"WARNING.- The selected directory and all its content will be deleted. Continue anyway?\");").append("\n");
			sb.append("	if (confirmFlag){").append("\n");
			sb.append("		document.formFileManager.").append(FileManagerServlet.PARAMETER_FILEMANAGER_ACTION).append(".value = '").append(FileManagerServlet.ACTION_REMOVE).append("';").append("\n");
			sb.append("		document.formFileManager.").append(FileManagerServlet.PARAMETER_FILEMANAGER_FILE).append(".value = eval ('document.formFileManager.' + fileId + '.value');").append("\n");
			sb.append("		document.formFileManager.submit();").append("\n");
			sb.append("	}").append("\n");
		}
		sb.append("}").append("\n");
		sb.append("function upload_onClick(){").append("\n");
		if (fileManagerBean.isWritable()){		
			sb.append("	document.formFileManager.").append(FileManagerServlet.PARAMETER_FILEMANAGER_ACTION).append(".value = '").append(FileManagerServlet.ACTION_UPLOAD).append("';").append("\n");
			sb.append("	document.formFileManager.submit();").append("\n");
		}
		sb.append("}").append("\n");
		sb.append("function download_onClick(fileId){").append("\n");
		if (fileManagerBean.isReadable()){
			//sb.append("	var contentDisposition = prompt('Enter donwload type: [ATTACHMENT:").append(CODE_CONTENTDISPOSITION_ATTACHMENT).append("][INLINE:").append(CODE_CONTENTDISPOSITION_INLINE).append("][ZIP:").append(CODE_CONTENTDISPOSITION_ZIPATTACHMENT).append("]', 'contentDispositionCode');").append("\n");
			sb.append("	var contentDisposition = '1';").append("\n");		 
			sb.append("	document.formFileManager.").append(FileManagerServlet.PARAMETER_FILEMANAGER_ACTION).append(".value = '").append(FileManagerServlet.ACTION_DOWNLOAD).append("';").append("\n");
			sb.append("	document.formFileManager.").append(FileManagerServlet.PARAMETER_FILEMANAGER_FILE).append(".value = eval ('document.formFileManager.' + fileId + '.value');").append("\n");
			sb.append("	document.formFileManager.").append(FileManagerServlet.PARAMETER_FILEMANAGER_DOWNLOADCONTENTDISPOSITION).append(".value = contentDisposition;").append("\n");
			sb.append("	document.formFileManager.submit();").append("\n");
		}
		sb.append("} ").append("\n");
		sb.append("function changeDirectory_onClick(fileId){").append("\n");
		if (fileManagerBean.isReadable()){
			sb.append("	document.formFileManager.").append(FileManagerServlet.PARAMETER_FILEMANAGER_ACTION).append(".value = '").append(FileManagerServlet.ACTION_CHANGEDIRECTORY).append("';").append("\n");
			sb.append("	document.formFileManager.").append(FileManagerServlet.PARAMETER_FILEMANAGER_DIR).append(".value = eval ('document.formFileManager.' + fileId + '.value');").append("\n");
			sb.append("	document.formFileManager.submit();").append("\n");
		}
		sb.append("} ").append("\n");
		sb.append("function executeCommand_onClick(){").append("\n");
		sb.append("	var command = document.formFileManager.").append(FileManagerServlet.PARAMETER_FILEMANAGER_COMMAND).append(".value;").append("\n");
		sb.append("	var confirmFlag = true;").append("\n");		
		sb.append("	if (command == 'mkdir'){").append("\n");
		sb.append("		confirmFlag = confirm(\"WARNING.- The new directory will be created. Continue anyway?\");").append("\n");		
		sb.append("	} else if (command == 'touch'){").append("\n");
		sb.append("		confirmFlag = confirm(\"WARNING.- The new file will be created. Continue anyway?\");").append("\n");
		sb.append("	}").append("\n");
		sb.append("	if (confirmFlag){").append("\n");
		sb.append("		document.formFileManager.").append(FileManagerServlet.PARAMETER_FILEMANAGER_ACTION).append(".value = '").append(FileManagerServlet.ACTION_EXECUTECOMMAND).append("';").append("\n");
		sb.append("		document.formFileManager.submit();").append("\n");
		sb.append("	}").append("\n");
		sb.append("} ").append("\n");
		sb.append("function list_onclick(orderBy, orderType){").append("\n");
		sb.append("	document.formFileManager.").append(FileManagerServlet.PARAMETER_FILEMANAGER_ACTION).append(".value = '").append(FileManagerServlet.ACTION_LIST).append("';").append("\n");
		sb.append("	document.formFileManager.").append(FileManagerServlet.PARAMETER_FILEMANAGER_ORDERBY).append(".value = orderBy;").append("\n");
		sb.append("	document.formFileManager.").append(FileManagerServlet.PARAMETER_FILEMANAGER_ORDERTYPE).append(".value = orderType;").append("\n");
		sb.append("	document.formFileManager.submit();").append("\n");
		sb.append("} ").append("\n");
		sb.append("</SCRIPT>").append("\n");
		sb.append("</HEAD>").append("\n");
		sb.append("<BODY onload='fileManager_onLoad();'>").append("\n");
		sb.append("	<FORM name='formFileManager' action='").append(request.getContextPath()).append("/FileManagerServlet' method='POST' ENCTYPE='multipart/form-data'>").append("\n");
		sb.append("	<INPUT type='hidden' name='").append(FileManagerServlet.PARAMETER_FILEMANAGER_ACTION).append("' />").append("\n");
		sb.append("	<INPUT type='hidden' name='").append(FileManagerServlet.PARAMETER_FILEMANAGER_DIR).append("' />").append("\n");		
		sb.append("	<INPUT type='hidden' name='").append(FileManagerServlet.PARAMETER_FILEMANAGER_FILE).append("' />").append("\n");
		sb.append("	<INPUT type='hidden' name='").append(FileManagerServlet.PARAMETER_FILEMANAGER_DOWNLOADCONTENTDISPOSITION).append("' />").append("\n");
		sb.append("	<INPUT type='hidden' name='").append(FileManagerServlet.PARAMETER_FILEMANAGER_ORDERBY).append("' />").append("\n");
		sb.append("	<INPUT type='hidden' name='").append(FileManagerServlet.PARAMETER_FILEMANAGER_ORDERTYPE).append("' />").append("\n");				
		sb.append("	<TABLE style='text-align:center; border: solid green 3px; width: 100%'>").append("\n");
		sb.append("		<TR>").append("\n");
		sb.append("			<TD class='titrePage'>").append("\n");
		sb.append("				<TABLE class='titrePage'>").append("\n");
		sb.append("					<TR>").append("\n");
		sb.append("						<TD class='titrePage' style='width: 1%; text-align:left; vertical-align: middle; color:white; text-decoration: none; font-size:14px; font-weight:bold;' nowrap>");
		sb.append("							<A href='#' onClick='man_onClick()' style='font-family: arial; font-size:14px; font-weight: bold; color:white; text-decoration:none; font-weight:bold'>").append("\n");		
		sb.append("								FileManager...").append("&nbsp;");
		sb.append("							</A>").append("&nbsp;");
		sb.append("						</TD>").append("\n");
		sb.append("						<TD class='titrePage' style='width: 100%; text-align:right; vertical-align: middle; color:white;text-decoration: none; font-size:14px; font-weight:bold;' nowrap>").append("\n");
		sb.append("							<input id='comandLine' type='text' onkeyup='executeCommand_onKeyup()' onkeypress='return checkEnter(event, \"executeCommand_onClick()\")' name='").append(FileManagerServlet.PARAMETER_FILEMANAGER_COMMAND).append("' style='width: 100%' value='").append(fileManagerCommand).append("'/>").append("\n");
		sb.append("						</TD>").append("\n");		
		sb.append("						<TD class='titrePage' style='width:1%; text-align: right; vertical-align: middle; color:white; padding-right: 5px;' nowrap>").append("\n");
		sb.append("							&nbsp;").append("\n");
		sb.append("							<A href='#' onclick='executeCommand_onClick()' class='titrePage' style='vertical-align: top; text-decoration: none; font-size:19px; font-weight:bold;' title='Execute Command'>").append("\n");   						
		sb.append("								&raquo;").append("\n");
		sb.append("							</A>").append("\n");
		sb.append("							&nbsp;").append("\n");
		sb.append("						</TD>").append("\n");
		sb.append("					</TR>").append("\n");
		sb.append("				</TABLE>").append("\n");
		sb.append("			</TD>").append("\n");
		sb.append("		</TR>").append("\n");
		sb.append("		<TR>").append("\n");
		sb.append("			<TD width='100%'>").append("\n");
		sb.append("				<TABLE width='100%' >").append("\n");
		sb.append("					<TR>").append("\n");
		sb.append("						<TD style='vertical-align: top; text-align: left; vertical-align:middle; width: 100%; border:solid black 0px' nowrap colspan='2'>").append("\n");
		sb.append("							<TABLE style='width: 100%;' >").append("\n");
		//MessageBar
		if (fileManagerMessage != null && !fileManagerMessage.trim().equals("")){
			sb.append("							<TR>").append("\n");
			sb.append("								<TD width='100%' colspan='6' style='border: solid black 1px; padding: 0px; margin: 0px'>").append("\n");
			sb.append("									<TABLE width='100%' style='padding: 0px; margin: 0px'>").append("\n");
			sb.append("										<TR>").append("\n");
			sb.append("											<TD style='width:100%'>").append("\n");
			sb.append("												<FIELDSET style='padding: 0px;'>").append("\n");
			sb.append("													<LEGEND><SPAN style='font-family: arial; font-size:11px; font-weight: bold; color:black'>&nbsp;Command Message&nbsp;</SPAN></LEGEND>").append("\n");
			sb.append("													<TABLE width='100%' height='15px'>").append("\n");
			sb.append("														<TR>").append("\n");
			sb.append("															<TD width='100%' style='text-align: center'>").append("\n");
			sb.append("																<SPAN style='color: red;'>").append(fileManagerMessage).append("</SPAN>").append("\n");
			sb.append("															</TD>").append("\n");
			sb.append("														</TR>").append("\n");
			sb.append("													</TABLE>").append("\n");
			sb.append("												</FIELDSET>").append("\n");
			sb.append("											</TD>").append("\n");
			sb.append("										</TR>").append("\n");
			sb.append("									</TABLE>").append("\n");
			sb.append("								</TD>").append("\n");
			sb.append("							</TR>").append("\n");			
		}
		//ToolBar
		if (fileManagerBean.isWritable()){
			sb.append("							<TR>").append("\n");
			sb.append("								<TD width='100%' colspan='6' style='border: solid black 1px; padding: 0px; margin: 0px'>").append("\n");
			sb.append("									<TABLE width='100%' style='padding: 0px; margin: 0px'>").append("\n");
			sb.append("										<TR>").append("\n");
			sb.append("											<TD style='width:100%'>").append("\n");
			sb.append("												<FIELDSET style='padding: 0px;'>").append("\n");
			sb.append("													<LEGEND><SPAN style='font-family: arial; font-size:11px; font-weight: bold; color:black'>&nbsp;Upload ToolBar&nbsp;</SPAN></LEGEND>").append("\n");
			sb.append("													<TABLE width='100%' height='15px'>").append("\n");
			sb.append("														<TR>").append("\n");
			sb.append("															<TD style='vertical-align: middle; width:120px'>").append("\n");
			sb.append("																<INPUT type='button' value='Upload File' onclick='upload_onClick()' style='width: 100px; font: 10px arial; color: black; margin: 0 5 0 5; border: solid black 1px'>").append("\n");
			sb.append("															</TD>").append("\n");
			sb.append("															<TD style='vertical-align: top; text-align: left; vertical-align:middle;' nowrap>").append("\n");
			sb.append("																<div class='fileinputs'>").append("\n");
			sb.append("																	&nbsp;<input id='uploadFileId' type='file' name='").append(FileManagerServlet.PARAMETER_FILEMANAGER_UPLOADFILE).append("'>&nbsp;").append("\n");
			//sb.append("																	<SCRIPT>//initFileUpload('uploadFileId');</SCRIPT>").append("\n");
			sb.append("																</div>").append("\n");
			sb.append("															</TD>").append("\n");
			sb.append("														</TR>").append("\n");
			sb.append("													</TABLE>").append("\n");
			sb.append("												</FIELDSET>").append("\n");
			sb.append("											</TD>").append("\n");
			sb.append("										</TR>").append("\n");
			sb.append("									</TABLE>").append("\n");
			sb.append("								</TD>").append("\n");
			sb.append("							</TR>").append("\n");			
		}
		sb.append("							</TABLE>").append("\n");
		sb.append("						</TD>").append("\n");
		sb.append("					</TR>").append("\n");
		sb.append("				</TABLE>").append("\n");
		sb.append("			</TD>").append("\n");
		sb.append("		</TR>").append("\n");
		sb.append("		<TR>").append("\n");
		sb.append("			<TD width='100%'>").append("\n");
		sb.append("				<TABLE width='100%' >").append("\n");
		sb.append("					<TR>").append("\n");
		sb.append("						<TD style='vertical-align: top; text-align: left; vertical-align:middle; width: 100%; border:solid black 0px' nowrap colspan='2'>").append("\n");
		sb.append("							<TABLE style='width: 100%;' >").append("\n");		
		//DirectoryBar
		sb.append("								<TR>").append("\n");
		sb.append("									<TD class='et' colspan='6' style='width:100%' nowrap>").append("\n");
		sb.append("										<TABLE class='et' style='width:100%'>").append("\n");
		sb.append("											<TR>").append("\n");
		sb.append("												<TD class='et' style='width: 1%; text-align:left; vertical-align: middle; color:white;text-decoration: none; font-size:14px; font-weight:bold;' nowrap>");
		sb.append("													Directory: ").append("\n");		
		sb.append("												</TD>").append("\n");
		sb.append("												<TD class='et' style='width:100%; text-align: left; vertical-align: middle;padding-right: 5px;' nowrap>").append("\n");
		sb.append("													").append(fileManagerDir).append("\n");
		sb.append("												</TD>").append("\n");
		sb.append("											</TR>").append("\n");
		sb.append("										</TABLE>").append("\n");
		sb.append("									</TD>").append("\n");
		sb.append("								</TR>").append("\n");
		//FileList
		int jobOrderType = -1;
		sb.append("								<TR>").append("\n");
		if (fileManagerBean.isWritable()){		
			sb.append("									<TD class='et' nowrap style='width: 20px; text-align:center; vertical-align: middle; color:white;text-decoration: none; font-size:20px; font-weight:bold;'>").append("\n");
			sb.append("										<A href='#' style='color: white; text-decoration: none; font-size:20px; font-weight:bold;' title='Remove File'>").append("\n");		
			sb.append("											&#215;").append("\n");
			sb.append("										</A>").append("\n");
			sb.append("									</TD>").append("\n");
		}
		sb.append("									<TD class='et' nowrap style='width: 1%; vertical-align: middle; text-align: center; color:white;text-decoration: none; font-size:10px; font-weight:bold;'>&nbsp;Kind&nbsp;</TD>").append("\n");
		jobOrderType = FileManagerBean.ORDERTYPE_ASC;
		if (fileManagerOrderBy == FileManagerBean.ORDERBY_FILENAME){
			if (fileManagerOrderType == FileManagerBean.ORDERTYPE_ASC){
				jobOrderType = FileManagerBean.ORDERTYPE_DESC;
			} else {
				jobOrderType = FileManagerBean.ORDERTYPE_ASC;
			}
		} 
		sb.append("									<TD class='et' nowrap style='width: 100%; vertical-align: middle; color:white;text-decoration: none; font-size:10px; font-weight:bold;'>").append("\n");
		sb.append("										&nbsp;").append("\n");
		sb.append("										<A onclick='javascript: list_onclick(").append(FileManagerBean.ORDERBY_FILENAME).append(", ").append(jobOrderType).append(")' >").append("\n");
		sb.append("											FileName").append("\n");
		sb.append("										</A>").append("\n");
		sb.append("										&nbsp;").append("\n");
		sb.append("									</TD>").append("\n");
		jobOrderType = FileManagerBean.ORDERTYPE_ASC;
		if (fileManagerOrderBy == FileManagerBean.ORDERBY_FILESIZE){
			if (fileManagerOrderType == FileManagerBean.ORDERTYPE_ASC){
				jobOrderType = FileManagerBean.ORDERTYPE_DESC;
			} else {
				jobOrderType = FileManagerBean.ORDERTYPE_ASC;
			}
		} 
		sb.append("									<TD class='et' nowrap style='width: 100%; vertical-align: middle; color:white;text-decoration: none; font-size:10px; font-weight:bold;'>").append("\n");
		sb.append("										&nbsp;").append("\n");
		sb.append("										<A onclick='javascript: list_onclick(").append(FileManagerBean.ORDERBY_FILESIZE).append(", ").append(jobOrderType).append(")' >").append("\n");		
		sb.append("											FileSize").append("\n");
		sb.append("										</A>").append("\n");		
		sb.append("										&nbsp;").append("\n");
		sb.append("									</TD>").append("\n");
		jobOrderType = FileManagerBean.ORDERTYPE_ASC;
		if (fileManagerOrderBy == FileManagerBean.ORDERBY_FILETYPE){
			if (fileManagerOrderType == FileManagerBean.ORDERTYPE_ASC){
				jobOrderType = FileManagerBean.ORDERTYPE_DESC;
			} else {
				jobOrderType = FileManagerBean.ORDERTYPE_ASC;
			}
		} 
		sb.append("									<TD class='et' nowrap style='width: 100%; vertical-align: middle; color:white;text-decoration: none; font-size:10px; font-weight:bold;'>").append("\n");
		sb.append("										&nbsp;").append("\n");
		sb.append("										<A onclick='javascript: list_onclick(").append(FileManagerBean.ORDERBY_FILETYPE).append(", ").append(jobOrderType).append(")' >").append("\n");		
		sb.append("											FileType").append("\n");
		sb.append("										</A>").append("\n");		
		sb.append("										&nbsp;").append("\n");
		sb.append("									</TD>").append("\n");
		jobOrderType = FileManagerBean.ORDERTYPE_ASC;
		if (fileManagerOrderBy == FileManagerBean.ORDERBY_FILEDATE){
			if (fileManagerOrderType == FileManagerBean.ORDERTYPE_ASC){
				jobOrderType = FileManagerBean.ORDERTYPE_DESC;
			} else {
				jobOrderType = FileManagerBean.ORDERTYPE_ASC;
			}
		} 
		sb.append("									<TD class='et' nowrap style='width: 100%; vertical-align: middle; color:white;text-decoration: none; font-size:10px; font-weight:bold;'>").append("\n");
		sb.append("										&nbsp;").append("\n");
		sb.append("										<A onclick='javascript: list_onclick(").append(FileManagerBean.ORDERBY_FILEDATE).append(", ").append(jobOrderType).append(")' >").append("\n");		
		sb.append("											Date").append("\n");
		sb.append("										</A>").append("\n");		
		sb.append("										&nbsp;").append("\n");
		sb.append("									</TD>").append("\n");
		sb.append("								</TR>").append("\n");									
		String classType = "ct";									
		if (fileManagerFileListArray != null){
			int fileManagerFileListArrayLength = fileManagerFileListArray.length;
			String fileManagerFileArrayItemKind = "";			
			String fileManagerFileArrayItemPath = "";
			String fileManagerFileArrayItemName = "";
			String fileManagerFileArrayItemSize = "";
			String fileManagerFileArrayItemType = "";
			long fileManagerFileArrayItemLastModified = 0;
			String fileManagerFileArrayItemDate = "";
			String fileManagerFileArrayItemProperties = "";
			String fileManagerFileArrayItemId = "";
			boolean isRemovable = false;
			for (int i = 0; i < fileManagerFileListArrayLength; i ++){
				File fileManagerFileArrayItem = fileManagerFileListArray[i];
				fileManagerFileArrayItemPath = fileManagerFileArrayItem.getCanonicalPath();
				fileManagerFileArrayItemPath = fileManagerFileArrayItemPath.replace('\\', '/');
				fileManagerFileArrayItemName = fileManagerFileArrayItem.getName();
				fileManagerFileArrayItemLastModified = new Long(fileManagerFileArrayItem.lastModified()).longValue();
				fileManagerFileArrayItemDate = FormatUtil.toString((new Date(fileManagerFileArrayItemLastModified)), "yyyy/MM/dd HH:mm", Locale.getDefault());				
				fileManagerFileArrayItemId = "fileManagerFile_" + i;
				fileManagerFileArrayItemProperties = "";
				isRemovable = false;
				if (fileManagerFileArrayItem.isFile()){
					//Kind
					fileManagerFileArrayItemKind = "[File]";
					//Name
					if (StringUtil.isNullOrEmpty(fileManagerFileArrayItemName)){
						fileManagerFileArrayItemName = "&nbsp;";
					}
					//Size
					fileManagerFileArrayItemSize = new Integer(FileUtil.getFileDataLength(fileManagerFileArrayItemPath)).toString();
					if (StringUtil.isNullOrEmpty(fileManagerFileArrayItemSize)){
						fileManagerFileArrayItemSize = "";
					}
					//Type
					fileManagerFileArrayItemType = "Archive " + FileUtil.getFileNameExtension(fileManagerFileArrayItemPath);
					//Properties
						//Length
						String fileManagerFileArrayItemDataLength = new Integer(FileUtil.getFileDataLength(fileManagerFileArrayItemPath)).toString();
						if (StringUtil.isNullOrEmpty(fileManagerFileArrayItemDataLength)){
							fileManagerFileArrayItemDataLength = "";
						} else {
							fileManagerFileArrayItemDataLength = "[Length: " + fileManagerFileArrayItemDataLength + " bytes]";
						}
						fileManagerFileArrayItemProperties = fileManagerFileArrayItemDataLength;
						//ContentType
						String fileManagerFileArrayItemExtension = FileUtil.getFileNameExtension(fileManagerFileArrayItemPath);
						//String extensionPropertiesKey = fileManagerFileArrayItemExtension.substring(".".length(), fileManagerFileArrayItemExtension.length());
						//String fileManagerFileArrayItemContentType = DownloadUtil.EXTENSIONPROPERTIES.getProperty(extensionPropertiesKey);
						//if (StringUtil.isNullOrEmpty(fileManagerFileArrayItemContentType)){
						//	fileManagerFileArrayItemContentType = "";
						//} else {
						//	fileManagerFileArrayItemContentType = "[ContentType: " + fileManagerFileArrayItemContentType + "]";
						//}
						fileManagerFileArrayItemProperties = "[File]" + "[" + fileManagerFileArrayItemName + "]" + fileManagerFileArrayItemProperties;
					//Removable
					isRemovable = true;
				} else if (fileManagerFileArrayItem.isDirectory()){
					//Kind
					fileManagerFileArrayItemKind = "[Directory]";
					//Name
					if (StringUtil.isNullOrEmpty(fileManagerFileArrayItemName)){
						fileManagerFileArrayItemName = "&nbsp";
					} else if (fileManagerFileArrayItemName.trim().equals(FileManagerBean.DIR_PARENT)){
						fileManagerFileArrayItemName = "[..]";
						fileManagerFileArrayItemKind = "[Parent]"; 
					}
					//Size
					fileManagerFileArrayItemSize = "&nbsp;";
					//Type
					fileManagerFileArrayItemType = "File Folder";
					//Properties
					fileManagerFileArrayItemProperties = "[Directory]" + "[" + fileManagerFileArrayItemName + "]";
					//Removable
					if (fileManagerFileArrayItemName != null && !fileManagerFileArrayItemName.trim().equals("")){
						if (!fileManagerFileArrayItemName.trim().equals(FileManagerBean.DIR_PARENT)){
							isRemovable = true;		 
						}
					}
				}
				if (classType.equals("ct")){
					classType = "bt";
				} else {
					classType = "ct";
				}
				sb.append("										<TR>").append("\n");
				sb.append("											<INPUT type='hidden' name='").append(fileManagerFileArrayItemId).append("' value='").append(fileManagerFileArrayItemPath).append("'>").append("\n");
				if (fileManagerFileArrayItem.isFile()){
					if (fileManagerBean.isWritable()){					
						sb.append("											<TD class='").append(classType).append("' style='vertical-align: middle; text-align: center' nowrap>").append("\n");
						if (isRemovable){
							sb.append("												<A href='#' onclick='removeFile_onClick(\"").append(fileManagerFileArrayItemId).append("\")' style='text-decoration: none; font-size:16px; text-align:center; font-weight:bold;' title='Remove File'>").append("\n");
							sb.append("													&#215;").append("\n");
							sb.append("												</A>").append("\n");
						} else {
							sb.append("													&nbsp;").append("\n");
						}
						sb.append("											</TD>").append("\n");
					}
					sb.append("											<TD class='").append(classType).append("' style='vertical-align: middle; text-align: center; font-size: 9px' nowrap>&nbsp;").append(fileManagerFileArrayItemKind).append("&nbsp;</TD>").append("\n");					
					sb.append("											<TD class='").append(classType).append("' style='vertical-align: middle; font-size: 9px' nowrap>").append("\n");
					sb.append("												&nbsp;").append("\n");
					sb.append("												<A ondblclick='download_onClick(\"").append(fileManagerFileArrayItemId).append("\")' title='").append(fileManagerFileArrayItemProperties).append("'>").append(fileManagerFileArrayItemName).append("</A>").append("\n");
					sb.append("												&nbsp;").append("\n");
					sb.append("											</TD>").append("\n");
					sb.append("											<TD class='").append(classType).append("' style='vertical-align: middle; text-align: right; font-size: 9px' nowrap>").append("\n");					
					sb.append("												&nbsp;").append(fileManagerFileArrayItemSize).append("&nbsp;").append("\n");
					sb.append("											</TD>").append("\n");
					sb.append("											<TD class='").append(classType).append("' style='vertical-align: middle; font-size: 9px' nowrap>").append("\n");
					sb.append("												&nbsp;").append(fileManagerFileArrayItemType).append("&nbsp;").append("\n");
					sb.append("											</TD>").append("\n");
					sb.append("											<TD class='").append(classType).append("' style='vertical-align: middle; font-size: 9px' nowrap>").append("\n");					
					sb.append("												&nbsp;").append(fileManagerFileArrayItemDate).append("&nbsp;").append("\n");
					sb.append("											</TD>").append("\n");
				} else {
					if (fileManagerBean.isWritable()){					
						sb.append("											<TD class='").append(classType).append("' style='vertical-align: middle; text-align: center' nowrap>").append("\n");
						if (isRemovable){
							sb.append("												<A href='#' onclick='removeDirectory_onClick(\"").append(fileManagerFileArrayItemId).append("\")' style='text-decoration: none; font-size:16px; text-align:center; font-weight:bold;' title='Remove Directory'>").append("\n");
							sb.append("													&#215;").append("\n");
							sb.append("												</A>").append("\n");
						} else {
							sb.append("													&nbsp;").append("\n");
						}
						sb.append("											</TD>").append("\n");
					}
					sb.append("											<TD class='").append(classType).append("' style='vertical-align: middle; text-align: center; font-size: 9px' nowrap>&nbsp;").append(fileManagerFileArrayItemKind).append("&nbsp;</TD>").append("\n");					
					sb.append("											<TD class='").append(classType).append("' style='vertical-align: middle; font-size: 9px' nowrap>").append("\n");
					sb.append("												&nbsp;").append("\n");
					sb.append("												<A ondblclick='changeDirectory_onClick(\"").append(fileManagerFileArrayItemId).append("\")' title='").append(fileManagerFileArrayItemProperties).append("'>").append(fileManagerFileArrayItemName).append("</A>").append("\n");
					sb.append("												&nbsp;").append("\n");
					sb.append("											</TD>").append("\n");
					sb.append("											<TD class='").append(classType).append("' style='vertical-align: middle; font-size: 9px' nowrap>").append("\n");					
					sb.append("												&nbsp;").append(fileManagerFileArrayItemSize).append("&nbsp;").append("\n");
					sb.append("											</TD>").append("\n");
					sb.append("											<TD class='").append(classType).append("' style='vertical-align: middle; font-size: 9px' nowrap>").append("\n");					
					sb.append("												&nbsp;").append(fileManagerFileArrayItemType).append("&nbsp;").append("\n");
					sb.append("											</TD>").append("\n");
					sb.append("											<TD class='").append(classType).append("' style='vertical-align: middle; font-size: 9px' nowrap>").append("\n");					
					sb.append("												&nbsp;").append(fileManagerFileArrayItemDate).append("&nbsp;").append("\n");
					sb.append("											</TD>").append("\n");
				}
				sb.append("										</TR>").append("\n");
			}
		}
		sb.append("							</TABLE>").append("\n");
		sb.append("						</TD>").append("\n");
		sb.append("					</TR>").append("\n");		
		sb.append("				</TABLE>").append("\n");
		sb.append("			</TD>").append("\n");
		sb.append("		</TR>").append("\n");
		sb.append("	</TABLE>").append("\n");
		sb.append("	</FORM>").append("\n");
		sb.append("</BODY>").append("\n");
		sb.append("</HTML>").append("\n");
		response.setHeader("Expires", (new Date()).toString());
		response.setHeader("Cache-Control","no-cache");
		response.setHeader("Pragma", "no-cache");
		response.setDateHeader("Expires", 0);
		PrintWriter out = response.getWriter();
		out.println(sb.toString());
	}

}
