package eu.semiramis.util.file;


import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.FilenameFilter;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;

public class FileUtil {
	public final static String FILE_LINEJUMP 					= "\r\n";
	
	
	/**********************************************************************************/
	/* File Management                                                                */
	/**********************************************************************************/
	
	/**
	 * listFiles
	 * @param dir
	 * @param filenameFilter
	 * @return
	 */	
	public static File[] listFiles(String dir){
		File[] fileArray = null;
		if (dir != null ){
			File dirFile = new File(dir);
			if (dirFile != null && dirFile.exists() && dirFile.isDirectory()){
				fileArray = dirFile.listFiles();
			}
		}
		return fileArray;
	}
	/**
	 * listFiles
	 * @param dir
	 * @param filenameFilter
	 * @return
	 */	
	public static File[] listFiles(String dir, FilenameFilter filenameFilter){
		File[] fileArray = null;
		if (dir != null && filenameFilter != null){
			File dirFile = new File(dir);
			if (dirFile != null && dirFile.exists() && dirFile.isDirectory()){
				fileArray = dirFile.listFiles(filenameFilter);
			}
		}
		return fileArray;
	}
	/**
	 * deleteFiles
	 * @param dir
	 * @param filenameFilter
	 */	
	public static void removeFiles(String dir, FilenameFilter filenameFilter){
		File[] fileArray = FileUtil.listFiles(dir, filenameFilter);
		if (fileArray != null && fileArray.length > 0){
			int fileArrayLength = fileArray.length;
			for (int i = 0; i < fileArrayLength; i ++){
				File file = fileArray[i];
				FileUtil.removeFile(file);
			}
		}
	}
	/**
	 * isExistent
	 * @param filePath
	 * @return
	 */
	public static boolean isExistent(String filePath){
		if (filePath != null){
			File file = new File(filePath);
			return (file.exists());
		} else {
			return false;
		}
	}
	/**
	 * isExistentFile
	 * @param file
	 * @return
	 */
	public static boolean isExistentFile(String filePath){
		if (filePath != null){
			File file = new File(filePath);
			return (isExistentFile(file));
		} else {
			return false;
		}
	}
	public static boolean isExistentFile(File file){
		if (file != null && file.exists() && file.isFile()){
			return true;
		} else {
			return false;
		}
	}
	/**
	 * isExistentDirectory
	 * @param folderPath
	 * @return
	 */
	public static boolean isExistentDirectory(String directoryPath){
		if (directoryPath != null){
			File file = new File(directoryPath);
			return (isExistentDirectory(file));
		} else {
			return false;
		}
	}
	public static boolean isExistentDirectory(File file){
		if (file != null && file.exists() && file.isDirectory()){
			return true;
		} else {
			return false;
		}
	}
	/**
	 * removeFile
	 * @param filePath
	 * @return
	 */
	public static boolean removeFile(String filePath){
		File file = new File(filePath);
		return removeFile(file);
	}
	public static boolean removeFile(File file){
		boolean result = false;
		if (file != null && file.exists() && file.isFile()){
			return file.delete();
		}
		return result;
	}
	/**
	 * removeDirectory
	 * @param filePath
	 * @return
	 */
	public static boolean removeDirectory(String filePath) {
		return removeDirectory(new File(filePath));
	}
	public static boolean removeDirectory(File file) {
		boolean result = false;
		if(file != null && file.exists() && file.isDirectory()) {
			File[] fileArray = file.listFiles();
			for(int i=0; i < fileArray.length; i++) {
				if(fileArray[i].isDirectory()) {
					removeDirectory(fileArray[i]);
				} else {
					fileArray[i].delete();
				}
			}
		}
		result = file.delete();
		return result;
	  }
	/**
	 * createDirectory
	 * @param path
	 */
	public static void createDirectory(String path) throws Exception{
		if (path != null){
			String formatedPath = path.replace('\\', '/');
			File file = new File(formatedPath);
			if (!file.exists()){
				int parentPathEndIndex = formatedPath.lastIndexOf("/");
				if (parentPathEndIndex > -1){
					String parentPath = formatedPath.substring(0, parentPathEndIndex);
					createDirectory(parentPath);
				}
				new File(formatedPath).mkdir();
			} else if (!file.isDirectory()){
				throw new Exception("Directory Path is a FilePath");
			}
		}
	}
	/**
	 * createFile
	 * @param path
	 */
	public static void createFile(String path) throws Exception{		
		if (path != null){
			String formatedPath = path.replace('\\', '/');
			int parentPathEndIndex = formatedPath.lastIndexOf("/");
			if (parentPathEndIndex > -1){			
				String parentPath = path.substring(0, parentPathEndIndex);
				createDirectory(parentPath);
			}
			new File(path).createNewFile();
		}
	}
	public static int getFileDataLength(File file) {
		int fileDataLength = -1;
		if (FileUtil.isExistentFile(file)){
			if (file.canRead()){
				fileDataLength = (int) file.length();				
			}
		}
		return fileDataLength;
	}
	public static int getFileDataLength(String filePath) {
		File file = new File(filePath);
		return getFileDataLength(file);
	}
	/**
	 * formatFilePath
	 * @param path
	 * @return
	 */
	public static String formatFilePath(String path){
		String formatedFilePath = path;
		if (formatedFilePath != null){
			formatedFilePath = formatedFilePath.replace('\\', File.separatorChar);
			formatedFilePath = formatedFilePath.replace('/', File.separatorChar);
		}
		return formatedFilePath;
	}
	/**
	 * getFileName
	 * @param path
	 * @return
	 */
	public static String getFileName(String path){
		String fileName = formatFilePath(path);
		if (fileName != null){
			int lastSeparatorIndex = fileName.lastIndexOf(File.separatorChar);
			if (lastSeparatorIndex > -1){
				fileName = fileName.substring(lastSeparatorIndex + 1, fileName.length());
			}
		}
		return fileName;
	}
	/**
	 * getFileNameExtension
	 * @param path
	 * @return
	 */
	public static String getFileNameExtension(String path){
		String fileExtension = "";
		String formatedFilePath = formatFilePath(path);
		if (formatedFilePath != null){
			int extensionSeparatorIndex = formatedFilePath.lastIndexOf(".");
			if (extensionSeparatorIndex > -1){
				fileExtension = formatedFilePath.substring(extensionSeparatorIndex, formatedFilePath.length());
			}
		}
		return fileExtension;
	}
	/**
	 * getFileNameKey
	 * @param fileName
	 * @return
	 */
	public static String getFileNameKey(String path){
		String fileName = getFileName(path);
		String fileKey = "";
		if (fileName != null){
			fileKey = fileName;
			int fileExtensionIndex = -1;
			if ((fileExtensionIndex = fileName.lastIndexOf(".")) >= 0){
				fileKey = fileName.substring(0, fileExtensionIndex);
			}
		}
		return fileKey;
	}
	
	
	
	/**
	 * hasExtension
	 */
	public static boolean hasExtension(File file, String ext){
		boolean hasExtension = false;
		if (file != null && file.getName() != null && file.getName().endsWith(ext)){
			hasExtension = true;
		}
		return hasExtension;
	}
	/*******************************************************************/
	/* File Management                                                 */
	/*******************************************************************/
	/*******************************************************************/
	/* File Load & Saving                                              */
	/*******************************************************************/
	/**
	 * copy
	 * @param inputFile
	 * @param outputFile
	 * @throws Exception
	 */
	public static void copy (File inputFile, File outputFile) throws Exception{ 
		FileReader fileReader = null;
		FileWriter fileWriter = null;
		try{
			fileReader = new FileReader(inputFile);
			fileWriter = new FileWriter(outputFile);
			int c;
			while ((c = fileReader.read()) != -1){
				fileWriter.write(c);
			}
		} finally {
			FileUtil.close(fileReader);
			FileUtil.close(fileWriter);
		}
	}
	/**
	 * save
	 * @param filePath
	 * @param fileDataStream
	 * @throws Exception
	 */
	public static void save (String filePath, String fileDataStream) throws Exception{
		FileOutputStream fileOutputStream = null;
		DataOutputStream dataOutputStream = null;
		try { 
			if (filePath != null){
				String formatedPath = filePath.replace('\\', '/');
				FileUtil.createDirectory(filePath.substring(0, formatedPath.lastIndexOf("/")));
				fileOutputStream = new FileOutputStream(filePath);
				dataOutputStream = new DataOutputStream(fileOutputStream);
				dataOutputStream.writeBytes(fileDataStream);
			}
		} finally {
			FileUtil.close(dataOutputStream);
			FileUtil.close(fileOutputStream);
		} 
	}
	public static void save (String filePath, byte[] fileByteArray) throws Exception{
		FileOutputStream fileOutputStream = null;
		try{
			if (fileByteArray != null){
				File file = new File(filePath);
				fileOutputStream = new FileOutputStream(file);
				fileOutputStream.write(fileByteArray);
			}
		} finally {
			if(fileOutputStream != null){
				fileOutputStream.close();
			}
		}
	}
	/**
	 * loadAsStream
	 * @param classLoader
	 * @param fileClassPath
	 * @return
	 * @throws Exception
	 */
	public static String loadAsStream(ClassLoader classLoader, String fileClassPath) throws Exception {
		InputStream imputStream = null;
		InputStreamReader inputStreamReader = null;
		BufferedReader bufferedReader = null;
		try{
			imputStream = classLoader.getResourceAsStream(fileClassPath);
			inputStreamReader = new InputStreamReader(imputStream);
			bufferedReader = new BufferedReader(inputStreamReader);
			StringBuffer sb = new StringBuffer();
			String readedLine = null;
			do { 
				readedLine = (String) bufferedReader.readLine();
				if (readedLine != null){
					sb.append(readedLine);sb.append(FILE_LINEJUMP);	
				}
			} while (readedLine != null);
			String fileStream = sb.toString();
			return fileStream;
		} finally {
			FileUtil.close(bufferedReader);
			FileUtil.close(inputStreamReader);
			FileUtil.close(imputStream);
		}
	}
	public static String loadAsStream(String filePath) throws Exception{
		FileInputStream fileInputStream = null; 
		InputStreamReader inputStreamReader = null;
		BufferedReader bufferedReader = null;
		try{
			String fileStream = null;
			fileInputStream = new FileInputStream(filePath);
			inputStreamReader = new InputStreamReader(fileInputStream);
			bufferedReader = new BufferedReader(inputStreamReader);
			String readedLine = null;
			StringBuffer sb = new StringBuffer();
			do { 
				readedLine = (String) bufferedReader.readLine();
				if (readedLine != null){
					sb.append(readedLine);sb.append(FILE_LINEJUMP);	
				}
			} while (readedLine != null);
			fileStream = sb.toString();
			return fileStream;
		} finally {
			FileUtil.close(bufferedReader);
			FileUtil.close(inputStreamReader);
			FileUtil.close(fileInputStream);			
		}
	}
	/**
	 * loadAsByteArray
	 * @param filePath
	 * @return
	 * @throws Exception
	 */
	public static byte[] loadAsByteArray(String filePath) throws Exception {
		InputStream inputStream = null;
		try{
			File file = new File(filePath);
			inputStream = new FileInputStream(filePath);
			long fileLength = file.length();
			byte[] byteArray = new byte[(int)fileLength];
			int offset = 0;
			int numRead = 0;
			while (offset < byteArray.length && (numRead = inputStream.read(byteArray, offset, byteArray.length-offset)) >= 0) {
				offset += numRead;
			}
			if (offset < byteArray.length) {
				throw new Exception("Could not completely read file "+ file.getName());
			}
			inputStream.close();
			return byteArray;			
		} finally {
			if (inputStream != null){
				inputStream.close();
			}
		}
	}	
	/**
	 * loadAsLineArray
	 * @param filePath
	 * @return
	 * @throws Exception
	 */
	public static String[] loadAsLineArray(String filePath) throws Exception{
		ArrayList arrayList = new ArrayList();
		FileInputStream fileInputStream = null;
		DataInputStream dataInputStream = null;
		try{
			fileInputStream = new FileInputStream(filePath);
			dataInputStream = new DataInputStream(fileInputStream);
			BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(dataInputStream));
			String line;
			while ((line = bufferedReader.readLine()) != null) {
				arrayList.add(line);
			}
			String[] lineArray = new String[arrayList.size()];
			lineArray = (String[]) arrayList.toArray(lineArray);
			dataInputStream.close();
			return lineArray; 
		} finally {
			if (dataInputStream != null){
				dataInputStream.close();
			}
			if (fileInputStream != null){
				fileInputStream.close();
			}
		}
	}
	/**
	 * close
	 * @param object
	 */
	private static void close(Object object){
		try{
			if (object != null){
				if (object instanceof BufferedReader){
					BufferedReader bufferedReader = (BufferedReader) object;
					bufferedReader.close();
					bufferedReader = null;
				} else if (object instanceof InputStreamReader){
					InputStreamReader inputStreamReader = (InputStreamReader) object;
					inputStreamReader.close();
					inputStreamReader = null;
				} else if (object instanceof FileInputStream){
					FileInputStream fileInputStream = (FileInputStream) object;
					fileInputStream.close();
					fileInputStream = null;
				} else if (object instanceof DataOutputStream){
					DataOutputStream dataOutputStream = (DataOutputStream) object;
					dataOutputStream.close();
					dataOutputStream = null;
				} else if (object instanceof FileOutputStream){
					FileOutputStream fileOutputStream = (FileOutputStream) object;
					fileOutputStream.close();
					fileOutputStream = null;
				} else if (object instanceof InputStream){
					InputStream inputStream = (InputStream) object;
					inputStream.close();
					inputStream = null;
				} else if (object instanceof FileReader){
					FileReader fileReader = (FileReader) object;
					fileReader.close();
					fileReader = null;
				} else if (object instanceof FileWriter){
					FileWriter fileWriter = (FileWriter) object;
					fileWriter.close();
					fileWriter = null;
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/*************************************************************************/
	/* Main                                                                  */
	/*************************************************************************/
	public static void main (String[] args){
		try{
			//String stream = FileUtil.loadAsStream("D:/projects/projectSia/siaWeb/JavaSource/ApplicationResources.properties");
			//System.out.println("stream: " + stream);

			//String stream = FileUtil.loadAsStream(FileUtil.class.getClassLoader(), "I18NFile.properties");
			//System.out.println("stream: " + stream);

			//Properties properties = FileUtil.loadAsProperties("D:/projects/projectSia/siaWeb/JavaSource/ApplicationResources.properties");
			//System.out.println("properties: " + properties);

			//Properties properties = FileUtil.loadAsProperties(FileUtil.class.getClassLoader(), "I18NFile.properties");
			//System.out.println("properties: " + properties);

			//ResourceBundle resourceBundle = FileUtil.loadAsResourceBundle("I18NFile");
			//Enumeration enum = resourceBundle.getKeys();
			//while (enum.hasMoreElements()){
			//	String key = (String) enum.nextElement();
			//	System.out.println(key + "=>" + resourceBundle.getString(key));
			//}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

}
