package eu.semiramis.util.properties;


import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileDescriptor;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.Properties;


public class PropertiesUtil {
	
	
	
	/**
	 * loadAsProperties
	 * @param configFilePath
	 * @return
	 * @throws Exception
	 */
	public static Properties loadAsProperties(String filePath) throws Exception{
		FileInputStream fileInputStream = null;
		try{
			Properties properties = new Properties();			
			fileInputStream = new FileInputStream(filePath);
			if (fileInputStream != null){
				properties.load(fileInputStream);
			}
			return properties;
		} finally {
			if (fileInputStream != null){
				fileInputStream.close();
				fileInputStream = null;
			}
		}
	}
	public static Properties loadAsProperties(ClassLoader classLoader, String fileClassPath) throws Exception {
		InputStream inputStream = null;
		try{
			Properties properties = new Properties();
			inputStream = classLoader.getResourceAsStream(fileClassPath);
			properties.load(inputStream);
			return properties;
		} finally {
			if (inputStream != null){
				inputStream.close();
				inputStream = null;
			}
		}
	}
	/**
	 * save
	 * @param fileName. The name of the file to save
	 * @param props: The properties to store
	 */
	public static void saveAsProperties(ClassLoader classLoader, String fileClassPath, Properties properties, String propertiesHeader) throws Exception{
		InputStream inputStream = null;
		FileDescriptor fileDescriptor = null;
		FileOutputStream fileOutputStream = null;
		try {
			inputStream = classLoader.getResourceAsStream(fileClassPath);
			if(inputStream instanceof FileInputStream){
				fileDescriptor = ((FileInputStream)inputStream).getFD();
				fileOutputStream = new FileOutputStream(fileDescriptor);
				properties.store(fileOutputStream, propertiesHeader);
			} 
		} finally {
			if (fileOutputStream != null){
				fileOutputStream.close();
				fileOutputStream = null;
			}
			if (inputStream != null){
				inputStream.close();
				inputStream = null;
			}
		} 
	}

	/**
	 * save
	 * @param fileName. The name of the file to save
	 * @param props: The properties to store
	 */
	public static void saveAsProperties(String filePath, Properties properties, String propertiesHeader) throws Exception{
		FileOutputStream fileOutputStream = null;
		try {
			File file = new File(filePath);
			fileOutputStream = new FileOutputStream(file);
			properties.store(fileOutputStream, propertiesHeader);

		} finally {
			if (fileOutputStream != null){
				fileOutputStream.close();
				fileOutputStream = null;
			}
		} 
	}
	

	/**
	 * toProperties
	 * @param propertiesStream
	 * @return
	 * @throws Exception
	 */	
	public static Properties toProperties(String propertiesStream) throws Exception{
		Properties properties = new Properties();
		ByteArrayInputStream byteArrayInputStream = null;
		try{
			if (propertiesStream != null){
				byte[] propertiesStreamByteArray = propertiesStream.getBytes();
				byteArrayInputStream = new ByteArrayInputStream(propertiesStreamByteArray);
				properties.load(byteArrayInputStream);
			}
		} finally {
			if (byteArrayInputStream != null){
				byteArrayInputStream.close();
				byteArrayInputStream = null;
			}
		}
		return properties;
	}


}
