package eu.semiramis.util.zip;
import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.jar.JarFile;
import java.util.zip.Deflater;
import java.util.zip.Inflater;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipOutputStream;

import eu.semiramis.util.file.FileUtil;

public class ZipUtil {

	/**
	 * zip
	 * @param unzippedByteArray
	 * @return
	 * @throws Exception
	 */
	public static byte[] zip(byte[] unzippedByteArray) throws Exception {
		//BEST_COMPRESSION Compression level for best compression.
		//BEST_SPEED Compression level for fastest compression.
		//DEFAULT_COMPRESSION
		Deflater deflater = new Deflater(Deflater.DEFAULT_COMPRESSION);
		deflater.setInput(unzippedByteArray);
		deflater.finish();
		byte[] zippedByteArray = new byte[unzippedByteArray.length];
		deflater.deflate(zippedByteArray);
		return zippedByteArray;
	}
	/**
	 * unzip
	 * @param zippedByteArray
	 * @return
	 * @throws java.util.zip.DataFormatException
	 */
	public static byte[] unzip(byte[] zippedByteArray) throws java.util.zip.DataFormatException {
		Inflater inflater = new Inflater();
		inflater.setInput(zippedByteArray);
		byte[] inflatedByteArray = new byte[6 * zippedByteArray.length];
		int unzippedBytesNumber = -1;
		unzippedBytesNumber = inflater.inflate(inflatedByteArray);
		byte[] unzippedByteArray = new byte[unzippedBytesNumber];
		System.arraycopy(inflatedByteArray, 0, unzippedByteArray, 0, unzippedBytesNumber);
		return unzippedByteArray;
	}	
	/**
	 * zipFile
	 * @param fileSourceFullPath
	 * @param fileTargetFullPath
	 * @return
	 * @throws Exception
	 */
	public static File zipFile(String fileSourceFullPath, String fileTargetFullPath) throws Exception {
		InputStream inputStream = null;
		ZipOutputStream zipOutputStream = null;
		int jobIndex = -1;
		try {		
			//Check fileSource
			if (fileSourceFullPath == null || !(FileUtil.isExistentFile(fileSourceFullPath))){
				throw new Exception("Invalid fileSourceFullPath: " + fileSourceFullPath);
			}
			//ManageFileSource
			fileSourceFullPath = fileSourceFullPath.replace('\\', '/');
			jobIndex = -1;
			String fileSourceName = null;
			if ((jobIndex = fileSourceFullPath.lastIndexOf("/")) > -1){
				fileSourceName = fileSourceFullPath.substring(jobIndex + 1, fileSourceFullPath.length());
			}
			//Check fileTarget
			if (fileTargetFullPath == null || FileUtil.isExistentDirectory(fileTargetFullPath)){
				throw new Exception("Invalid fileTargetFullPath: " + fileTargetFullPath);
			}
			//ManageFileTarget
			fileTargetFullPath = fileTargetFullPath.replace('\\', '/');
			//Remove preexistent fileTarget
			if (FileUtil.isExistentFile(fileTargetFullPath)){
				FileUtil.removeFile(fileTargetFullPath);	
			}
			//Manage fileTargetDirectory
			jobIndex = -1;
			String fileTargetDirectoryFullPath = null;
			if ((jobIndex = fileTargetFullPath.lastIndexOf("/")) > -1){
				fileTargetDirectoryFullPath = fileTargetFullPath.substring(0, jobIndex);
			}
			if (fileTargetDirectoryFullPath != null && !FileUtil.isExistentDirectory(fileTargetDirectoryFullPath)){
				FileUtil.createDirectory(fileTargetDirectoryFullPath);
			}
			//Zip fileSourceFullPath as fileTargetFullPath
			byte[] buffer = new byte[1024];
			File zippedFile = new File(fileTargetFullPath);			
			zipOutputStream = new ZipOutputStream(new FileOutputStream(zippedFile));  
			zipOutputStream.setLevel(Deflater.DEFAULT_COMPRESSION);
			ZipEntry zipEntry = new ZipEntry(fileSourceName);
			zipOutputStream.putNextEntry(zipEntry);
			inputStream = new FileInputStream(new File(fileSourceFullPath));
			int count;
			while((count = inputStream.read(buffer, 0, 1024)) != -1) {
				zipOutputStream.write(buffer, 0, count);
			}
			zipOutputStream.closeEntry();
			zipOutputStream.close();
			return zippedFile;
		} finally {
			if (inputStream != null){
				inputStream.close();
			}
		}
	}
	public static int BUFFERSIZE = 4096;

	
	/**
	 * zipFile
	 * @param fileSourceFullPath
	 * @return
	 * @throws Exception
	 */
	public static byte[] zipFile(String fileSourceFullPath) throws Exception {
		//Zip native stream
		ByteArrayOutputStream byteArrayOutputStream = null;
		byte[] zippedByteArray = null;
		try{
			byteArrayOutputStream = new ByteArrayOutputStream();
			byteArrayOutputStream = (ByteArrayOutputStream)ZipUtil.zipFile(fileSourceFullPath, byteArrayOutputStream);
			zippedByteArray = byteArrayOutputStream.toByteArray();
		} catch (Exception e){
			throw e;
		} finally {
			if (byteArrayOutputStream != null){
				byteArrayOutputStream.close();
				byteArrayOutputStream = null;
			}
		}
		return zippedByteArray;

	}
	/**
	 * zipFile
	 * @param fileSourceFullPath
	 * @param outputstream
	 * @throws Exception
	 */
	public static OutputStream zipFile(String fileSourceFullPath, OutputStream outputStream) throws Exception {
		BufferedInputStream bufferedInputStream = null;
		FileInputStream fileInputStream = null;
		ZipOutputStream zipOutputStream = null;
		try {
			String zipFileName = FileUtil.getFileName(fileSourceFullPath);
			fileInputStream = new FileInputStream(fileSourceFullPath);
			bufferedInputStream = new BufferedInputStream(fileInputStream);
			int j = 0;
			byte byteArray[] = new byte[BUFFERSIZE];
			zipOutputStream = new ZipOutputStream(outputStream);
			ZipEntry zipEntry = new ZipEntry(zipFileName);
			zipOutputStream.putNextEntry(zipEntry);
			while ((j = bufferedInputStream.read(byteArray)) != -1){
				zipOutputStream.write(byteArray, 0, j);
			}
			zipOutputStream.closeEntry();
			zipOutputStream.flush();
			zipOutputStream.close();
			fileInputStream.close();
			bufferedInputStream.close();
		} finally {
			if (zipOutputStream != null){
				zipOutputStream.close();
			}
			if (fileInputStream != null){
				fileInputStream.close();
			}
			if (bufferedInputStream != null){
				bufferedInputStream.close();
			}
		}
		return outputStream;
	}
	/**
	 * zipFile
	 * @param fileSourceFullPath
	 * @param outputstream
	 * @throws Exception
	 */
	public static OutputStream zipFile(byte[] unzippedByteArray, OutputStream outputStream, String fileName) throws Exception {
		BufferedInputStream bufferedInputStream = null;
		InputStream inputStream = null;
		ZipOutputStream zipOutputStream = null;
		try {
			inputStream = new ByteArrayInputStream(unzippedByteArray);
			bufferedInputStream = new BufferedInputStream(inputStream);
			int j = 0;
			byte byteArray[] = new byte[BUFFERSIZE];
			zipOutputStream = new ZipOutputStream(outputStream);
			ZipEntry zipEntry = new ZipEntry(fileName);
			zipOutputStream.putNextEntry(zipEntry);
			while ((j = bufferedInputStream.read(byteArray)) != -1){
				zipOutputStream.write(byteArray, 0, j);
			}
			zipOutputStream.closeEntry();
			zipOutputStream.flush();
			zipOutputStream.close();
			inputStream.close();
			bufferedInputStream.close();
		} finally {
			if (zipOutputStream != null){
				zipOutputStream.close();
			}
			if (inputStream != null){
				inputStream.close();
			}
			if (bufferedInputStream != null){
				bufferedInputStream.close();
			}
		}
		return outputStream;
	}
	/**
	 * zipFile
	 * @param unzippedByteArray
	 * @param fileName
	 * @return
	 * @throws Exception
	 */
	public static byte[] zipFile(byte[] unzippedByteArray, String fileName) throws Exception {
		//Zip native stream
		ByteArrayOutputStream byteArrayOutputStream = null;
		byte[] zippedByteArray = null;
		try{
			byteArrayOutputStream = new ByteArrayOutputStream();
			byteArrayOutputStream = (ByteArrayOutputStream)ZipUtil.zipFile(unzippedByteArray, byteArrayOutputStream, fileName);
			zippedByteArray = byteArrayOutputStream.toByteArray();
		} catch (Exception e){
			throw e;
		} finally {
			if (byteArrayOutputStream != null){
				byteArrayOutputStream.close();
				byteArrayOutputStream = null;
			}
		}
		return zippedByteArray;
	}
	
	/**
	 * unZipFile
	 * @param fileSourceFullPath
	 * @param fileTargetFullPath
	 * @return
	 * @throws Exception
	 */
	public static File unZipFile(String fileSourceFullPath, String fileTargetFullPath) throws Exception {
		File fileTarget = null;
		FileOutputStream fileOutputStream = null;
		InputStream inputStream = null;
		ZipFile zipedFile = null;
		try{
			//Target
			if (fileTargetFullPath == null){
				throw new Exception("Null fileTargetFullPath");
			} 
			if (!FileUtil.isExistentDirectory(fileTargetFullPath)){
				FileUtil.createDirectory(fileTargetFullPath);	
			}
			fileTarget = new File(fileTargetFullPath);
			//Source
			if (fileSourceFullPath == null){
				throw new Exception("Null fileSourceFullPath");
			}
			if (!FileUtil.isExistentFile(fileSourceFullPath)){
				throw new Exception("Inexsitant fileSourceFullPath");
			}
			File fileSource = new File(fileSourceFullPath); 
			//Create ZipContainer
			if (FileUtil.hasExtension(fileSource, ".zip")){
				zipedFile = new ZipFile(fileSource.getAbsolutePath());
			} else if (FileUtil.hasExtension(fileSource, ".jar")){
				zipedFile = new JarFile(fileSource.getAbsolutePath());
			} else if (FileUtil.hasExtension(fileSource, ".war")){
				zipedFile = new JarFile(fileSource.getAbsolutePath());
			}
			Enumeration entries = zipedFile.entries();
			ArrayList arrayList = new ArrayList();
			while(entries.hasMoreElements()) {
				ZipEntry zipEntry = (ZipEntry) entries.nextElement();
				if (zipEntry == null){
					throw new FileNotFoundException("Empty Zip Entry");
				}
				StringBuffer sb = new StringBuffer();
				sb.append(fileTargetFullPath);
				sb.append(File.separator);
				sb.append(zipEntry.getName());
				String zipEntryPath = sb.toString().replace('/', '\\');
				if (zipEntry.isDirectory() && !FileUtil.isExistentDirectory(zipEntryPath)){
					FileUtil.createDirectory(zipEntryPath);
				} else {
					if (FileUtil.isExistentFile(zipEntryPath)){
						FileUtil.removeFile(zipEntryPath);
					}
					FileUtil.createFile(zipEntryPath);
					fileOutputStream = new FileOutputStream(new File(zipEntryPath));
					inputStream = zipedFile.getInputStream(zipEntry);
					int readLength = 0;
					byte[] bufferLength = new byte[1024];
					while (-1 != readLength){
						readLength = inputStream.read(bufferLength);
						if (readLength > 0){
							fileOutputStream.write(bufferLength, 0, readLength);
						}
					}
				}
			}
		} catch (Exception e){
			e.printStackTrace();
			throw e;
		} finally {
			if (zipedFile != null){
				zipedFile.close();
			}
			if (inputStream != null){
				inputStream.close();
			}
			if (fileOutputStream != null){
				fileOutputStream.close();
			}
		}
		return fileTarget;		
	}
}
